/*
	Blokus SDL
		blapp.cpp

	This file includes many class APP functions.

	Copyright (c) 2003-2004 Antti Partanen


	This file is part of Blokus SDL.

	Blokus SDL is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	Blokus SDL is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with Blokus SDL; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#define ERROR_STRINGS_INUSE

//**********************************************************
//* INCLUDES
#ifdef _WIN32
#include <windows.h>
#endif

#include <cstdlib>

#include "derrors.h"
#include "txtfile.h"
#include "dpic.h"
#include "dmf.h"

#include "blst.h"
#include "blapp.h"
#include "blvideo.h"
#include "blsound.h"
#include "blnetwork.h"
#include "blfiles.h"
#include "blokus.h"
#include "blexceptions.h"
#include "bltextbox.h"
#include "blmisc.h"

//**********************************************************
//* METHODS

//**********************************************************
/*
	class APP.
*/

//**********************************************************
/**
	Constructor.
*/
APP::APP() :
	g(*new GAME)
{
	/* Check for class GAME reference for allocation failures. */
	if (!&g)
	{
		FILE_TextI("Failed to allocate class GAME.\n");
		throw OutOfMemory("Failed to allocate class GAME.\n",
			              __LINE__, __FILE__);
	}

	/* Set up the keyboard markers. */
	{
		memcpy((char *)&Key,
			   (char *)&game_st.key,
			   sizeof(KEYMAP));
		
		Key.left = false;
		Key.right = false;
		Key.hflip = false;
		Key.vflip = false;
	}

	run = true;

	mouse_x = 0;
	mouse_y = 0;
	mouse_bt = 0;
	mouse_dbt = 0;
	mouse_on = true;

	Mode = MMENU;

	tpf = 20;
	
	/* Set up colors. */
	LText.r = TEXT_LIGHT_R;
	LText.g = TEXT_LIGHT_G;
	LText.b = TEXT_LIGHT_B;

	DText.r = TEXT_DARK_R;
	DText.g = TEXT_DARK_G;
	DText.b = TEXT_DARK_B;

	MLText.r = MTEXT_LIGHT_R;
	MLText.g = MTEXT_LIGHT_G;
	MLText.b = MTEXT_LIGHT_B;

	MDText.r = MTEXT_DARK_R;
	MDText.g = MTEXT_DARK_G;
	MDText.b = MTEXT_DARK_B;

	MMText.r = MTEXT_MLIGHT_R;
	MMText.g = MTEXT_MLIGHT_G;
	MMText.b = MTEXT_MLIGHT_B;

	White.r = 255;
	White.g = 255;
	White.b = 255;

	Black.r = 0;
	Black.g = 0;
	Black.b = 0;

	Blue.r = 0;
	Blue.g = 0;
	Blue.b = 250;

	Green.r = 0;
	Green.g = 250;
	Green.b = 0;

	Red.r = 250;
	Red.g = 0;
	Red.b = 0;

	Yellow.r = 192;
	Yellow.g = 192;
	Yellow.b = 0;
	
	cChat.r = CHAT_TEXT_R;
	cChat.g = CHAT_TEXT_G;
	cChat.b = CHAT_TEXT_B;

	cTurn.r = GAME_TURN_TEXT_R;
	cTurn.g = GAME_TURN_TEXT_G;
	cTurn.b = GAME_TURN_TEXT_B;

	cOther.r = GAME_OTHER_TEXT_R;
	cOther.g = GAME_OTHER_TEXT_G;
	cOther.b = GAME_OTHER_TEXT_B;

	cbOther.r = GAME_OTHER_TEXTB_R;
	cbOther.g = GAME_OTHER_TEXTB_G;
	cbOther.b = GAME_OTHER_TEXTB_B;
	
	string_get = false;
	string_cursor = 0;
	string_max_len = 0;
	string_selstart = 0;
	string_selend = 0;

	reload =  true;

	use_scandinavians = true;
	
	selected_item = 0;
	tool_tip = -1;
	tool_tip_x = 0;
	tool_tip_y = 0;
	tool_tip_srf = NULL;

	input = NULL;

	Id_string = NULL;
	Version = NULL;

	fChat = NULL;
	iFont = NULL;
	dFont = NULL;
	MFont = NULL;

	key = SDLK_UNKNOWN;

#ifdef USE_WINAMP
	winamp_msg = 0;
	winamp_msg_delay = 0;
	memset(winamp_title, 0, sizeof(winamp_title));
#endif

	/* Return. */
	return;
}

//**********************************************************
/**
	Destructor.
*/
APP::~APP()
{
	/* Delete class GAME. */
	delete &g;

	/* Release sounds. */
	play_list.release_all();

	SND_Release(snd_gameover);
	SND_Release(snd_next);

}

//**********************************************************
/**
	Main application function. Here it all starts and ends.
	Atleast for class APP. (int main() calls this funtion)
*/
void APP::main(void)
{
	/* Some variables. */
	static bool Once = true;
	EVALUE err;

	/* Printing version numbers. */
	unsigned int major;
	unsigned int minor;
	unsigned int micro;

	/* For information about the current video settings. */
	static const SDL_VideoInfo* info = NULL;

	/* Not sure, if running yet. :) */
	run = false;

	/*
		FILE_TextD() is for debugging,
		normaly these lines wont do anything.
	*/
	FILE_TextD("DEBUG: Debug mode enabled.\n");

	/* First read settings. */
	ReadSettings(SETTINGS_FILE,
		         (char *)&st, sizeof(SETTINGS),
				 (char *)&game_st, sizeof(GAME_SETTINGS));

	/* Set random seed. */
	srand(st.rseed);

	/* Print DMF-library version to LOG. */
	DMF_GetVersion(&major, &minor, &micro);

	FILE_TextI("INFO: DMF dynamic library-version is: %u.%u.%u\n",
		       major, minor, micro);

	/* Init SDL and check errors at same time. */
	if (SDL_Init(SDL_INIT_VIDEO | SDL_INIT_AUDIO) < 0)
	{
		FILE_Text(TXTFILE_INFO, true, false, false,
			      "ERROR: Failed to init SDL, SDL error: %s.\n",
				  SDL_GetError());
		release();
		return;
	}

	/* DEBUG info. */
	FILE_TextD("DEBUG: SDL_Init() succesful.\n");
	
	/* Init video library. */
	err = VID_Init();

	/* Check errors. */
	if (err != A_OK)
	{
		FILE_Text(TXTFILE_INFO, true, false, false,
			      "ERROR: Failed to init video-library, error: %s (%lu).\n",
			      ERROR_STRINGS[err], err);
		release();
		return;
	}

	/* Init sound library. */
	err = SND_Init();

	/* Check errors. */
	if (err != A_OK)
	{
		FILE_Text(TXTFILE_INFO, true, false, false,
			      "WARNING: Failed to init sound-library, error: %s (%lu).\n"
			      "WARNING: Disabling sounds.\n",
			      ERROR_STRINGS[err], err);
		st.snd_enabled = 0;
	}

	/* DEBUG info. */
	FILE_TextD("DEBUG: VID_Init() succesful.\n");

	/* Init True Type Font -library. */
	if (TTF_Init() < 0)
	{
		FILE_Text(TXTFILE_INFO, true, false, false,
			      "ERROR: Failed to init TTF-library, SDL error: %s.\n",
				  SDL_GetError());
		release();
		return;
	}

	/* DEBUG info. */
	FILE_TextD("DEBUG: TTF_Init() succesful.\n");

	/* Init networking-library. */
	err = NET_Init();
	if (err != A_OK)
	{
		FILE_Text(TXTFILE_INFO, true, false, false,
			      "WARNING: Failed to init network-library, error: %s (%lu).\n",
			      ERROR_STRINGS[err], err);
	}
	else
	{
		NET_SetMulticastAddress(NET_MULTICASTIP);
		NET_SetPacketID(NET_ID);
		FILE_TextD("DEBUG: NET_Init() succesful.\n");

		NET_GetVersion(&major, &minor, &micro);

		FILE_TextI("INFO: DNET dynamic library-version is: %u.%u.%u\n",
			       major, minor, micro);
	}

	/* Let's get some video information. */
	if (Once)
	{
		info = SDL_GetVideoInfo();
	}

	/* Check failures. */
	if (!info)
	{
		/* This should probably never happen. */
		FILE_Text(TXTFILE_INFO, true, false, false,
			      "ERROR: Video query failed, SDL error: %s.\n",
				  SDL_GetError());
        release();
		return;
    }
	
	/* Get the video driver name. */
	char driver_name[LEN_SHORT_STRING + 1];
	SDL_VideoDriverName(driver_name, LEN_SHORT_STRING);
	
	/* DEBUG info. */
	FILE_TextD("DEBUG: Video driver name: %s.\n", driver_name);

	/* DEBUG info. */
	FILE_TextD("DEBUG: SDL_VideoInfo:\n"
		" Hardware surfaces supported: %s\n"
		" Window manager available: %s\n"
		" Hardware to hardware blits available: %s\n"
		" Hardware to hardware colorkey blits available: %s\n"
		" Hardware to hardware alpha blits available: %s\n"
		" Software to hardware blits available: %s\n"
		" Software to hardware colorkey blits available: %s\n"
		" Software to hardware alpha blits available: %s\n"
		" Color fills accelerated: %s\n"
		" Total amount of video memory: %lukB\n",
		info->hw_available > 0 ? "YES" : "NO",
		info->wm_available > 0 ? "YES" : "NO",
		info->blit_hw > 0 ? "YES" : "NO",
		info->blit_hw_CC > 0 ? "YES" : "NO",
		info->blit_hw_A > 0 ? "YES" : "NO",
		info->blit_sw > 0 ? "YES" : "NO",
		info->blit_sw_CC > 0 ? "YES" : "NO",
		info->blit_sw_A > 0 ? "YES" : "NO",
		info->blit_fill > 0 ? "YES" : "NO",
		info->video_mem);

	/* DEBUG info. */
	FILE_TextD("DEBUG: SDL_GetVideoInfo() succesful.\n");

	/* Set window icon. */
	if (Once)
	{
		/* Icon. */
		DPIC icon;

		/*
			Must set the pixel format manually,
			becose video is not initialized yet.
		*/
		SDL_PixelFormat fmt;
		memset(&fmt, 0, sizeof(SDL_PixelFormat));
		fmt.palette = NULL;
		fmt.BitsPerPixel = 32;
		fmt.BytesPerPixel = 4;
		fmt.Rmask = 0x000000ff;
		fmt.Gmask = 0x0000ff00;
		fmt.Bmask = 0x00ff0000;
		fmt.Amask = 0xff000000;
		fmt.Rshift = 0;
		fmt.Gshift = 8;
		fmt.Bshift = 16;
		fmt.Ashift = 24;
		fmt.Rloss = 0;
		fmt.Gloss = 0;
		fmt.Bloss = 0;
		fmt.Aloss = 0;
		fmt.colorkey = 0;
		fmt.alpha = 0;

		/* Load icon. */
		LoadDMF(&icon, BLOKUSI, DMF_A_NAME,
			    false, SDL_ALPHA_OPAQUE, &fmt);

		/* Set icon. */
		SDL_WM_SetIcon(icon.srf, NULL);

		/* Print DEBUG info. */
		FILE_TextD("DEBUG: SDL_WM_SetIcon() succesful.\n");

		/* Destruct image. */
		icon.~DPIC();
	}

	/* Initialize video. */
	err = change_video_mode();

	/* If failed. */
	if (err != A_OK)
	{
		return;
	}
	/* If not failed. */
	else
	{
		FILE_TextD("DEBUG: video initialization succesful.\n");
	}

	/* Startup stuff. */

	LoadDMF(&BG, TXTLOADING, DMF_A_NAME);
	VID_FillRect(NULL, 0);
	VID_Blit(vscreen->w / 2 - BG.w / 2, vscreen->h / 2 - BG.h / 2, 0, &BG);
	SDL_Flip(vscreen);
	BG.~DPIC();
	
	FILE_TextD("DEBUG: Loaded and displayed startup background.\n");

	/* Load needed textures, sounds and so on. */
	err = init();

	if (err != A_OK)
	{
		FILE_TextV("ERROR: Something went wrong in funtion AtStart().\n");
		release();
		return;
	}

	FILE_TextD("DEBUG: AtStart() succesful.\n");

	/* Now start normal looping. */
	FILE_TextD("DEBUG: Starting to run the main application loop...\n");

	run = true;
	
	/* Find winamp window handle, if winamp controlling enabled. */
#ifdef USE_WINAMP
	FILE_TextD("DEBUG: Trying to locate running winamp handle.\n");
	winamp_hwnd = FindWindow("Winamp v1.x", NULL); 

	if (!winamp_hwnd)
	{
		FILE_TextD("DEBUG: No running winamp found.\n");
	}
#endif

	/* Run the main loop until mode is MEXIT. */
	while (run && Mode != MEXIT)
	{
		switch (Mode)
		{
		case MMENU:
		case MSETTINGS:
		case MNICK:
		case MJOIN:
		case MGAME:
			menus_loop();
			break;
		
		default:
			FILE_TextI("ERROR: Unknown mode, WTF!?!?\n");
		case MEXIT:
			run = false;
			break;
		}
	}

	/* DEBUG info. */
	FILE_TextD("DEBUG: Exited the main application loop.\n");

	/* Save random seed. */
	st.rseed = SDL_GetTicks();
	
	/* Save settings and check errors. */
	if (SaveSettings(SETTINGS_FILE,
		             (char *)&st, sizeof(SETTINGS),
					 (char *)&game_st, sizeof(GAME_SETTINGS))
					 != A_OK)
	{
		FILE_TextD("DEBUG: Failed to save settings!\n");
	}
	/* If not failed to save settings. */
	else
	{
		FILE_TextD("DEBUG: Settings saved.\n");
	}

	/* Release resources. */
	release();

	/* Return. */
	return;
}

//**********************************************************
/**
	This is the function which runs the game itself.
*/
void APP::game(int button)
{
	while (button > -1)
	{
		if (bgam.state[button] == 2)
		{
			g.clicked_right();
		}

		if (bgam.state[button] > 1)
		{
			bgam.state[button] = 0;
			break;
		}

		switch (button)
		{
		case bGskip:
			g.clicked_skip();
			break;

		case bGsettings:
			g.clicked_settings();
			break;

		case bGexit:
			g.clicked_exit();
			Mode = MJOIN;
			rMode = MMENU;
			return;

		case bGchat:
			g.clicked_chat();
			string_get = true;
			break;

		case bGstore:
			g.clicked_store(mouse_x, mouse_y);
			break;

		case bGboard:
			g.clicked_board(mouse_x, mouse_y);
			break;

		default:
			string_get = false;
			break;
		}
		
		bgam.state[button] = 0;

		break;
	}

	SDL_BlitSurface(BG.srf, NULL, vscreen, NULL);

	game_buttons(selected_item);

	/* Draw everything in class GAME to screen. */
	g.draw(mouse_x, mouse_y, selected_item, vscreen);
	
	static bool closing = false;

	if (NET_GetState(NET_STATEOF_CONNECTION) == NET_STATE_CLOSED)
	{
		if (!closing)
		{
			NET_Close(false);
			closing = true;
		}
		
		if (NET_GetState(NET_STATEOF_CLOSE) && Mode == MGAME)
		{
			Mode = MJOIN;
			rMode = MMENU;
			closing = false;
		}
	}
	/* Handle networking for class GAME. */
	else
	{
		g.handle_network();
	}

	return;
}

//**********************************************************
/**
	Handle key presses in game.
*/
void APP::game_handle_key_down(SDL_keysym *keysym)
{
	if (keysym->sym == SDLK_F3)
	{
		if (st.snd_enabled && st.volume >= 3)
		{
			st.volume -= 3;
			SND_Volume(st.volume);
			return;
		}
	}
	else if (keysym->sym == SDLK_F4)
	{
		if (st.snd_enabled && st.volume <= 97)
		{
			st.volume += 3;
			SND_Volume(st.volume);
			return;
		}
	}
	
	if (g.get_brick() < 0)
	{
		return;
	}

	if (keysym->sym == Key.Hflip)
	{
		if (g.get_pos() < 4)
		{
			g.set_pos(g.get_pos() + 4);
		}
		else
		{
			g.set_pos(g.get_pos() - 4);
		}
	}
	else if (keysym->sym == Key.Vflip)
	{
		switch (g.get_pos())
		{
		default:
		case 0:
			g.set_pos(6);
			break;
		case 1:
			g.set_pos(7);
			break;
		case 2:
			g.set_pos(4);
			break;
		case 3:
			g.set_pos(5);
			break;
		case 4:
			g.set_pos(2);
			break;
		case 5:
			g.set_pos(3);
			break;
		case 6:
			g.set_pos(0);
			break;
		case 7:
			g.set_pos(1);
			break;
		}
	}
	else if (keysym->sym == Key.Left)
	{
		if (g.get_pos() == 0)
		{
			g.set_pos(3);
		}
		else if (g.get_pos() == 4)
		{
			g.set_pos(7);
		}
		else
		{
			g.set_pos(g.get_pos() - 1);
		}
	}
	else if (keysym->sym == Key.Right)
	{
		if (g.get_pos() == 3)
		{
			g.set_pos(0);
		}
		else if (g.get_pos() == 7)
		{
			g.set_pos(4);
		}
		else
		{
			g.set_pos(g.get_pos() + 1);
		}
	}
	else if (keysym->sym == Key.Set)
	{
	}
	else if (keysym->sym == Key.Skip)
	{
	}
	else if (keysym->sym == Key.Cancel)
	{
	}
}

//**********************************************************
/**
	Handle key releases in game.
*/
void APP::game_handle_key_up(SDL_keysym *keysym)
{
	if (keysym->sym == SDLK_F1)
	{
		static int pause = 1;

		if (pause)
		{
			pause = 0;
		}
		else
		{
			pause = 1;
		}

		SND_Pause(pause);
	}
	else if (keysym->sym == SDLK_F2)
	{
		play_list.next();
	}

//	if (selected_brick < 0)
	{
		return;
	}

	if (keysym->sym == Key.Hflip)
	{
	}
	else if (keysym->sym == Key.Vflip)
	{
	}
	else if (keysym->sym == Key.Left)
	{
	}
	else if (keysym->sym == Key.Right)
	{
	}
	else if (keysym->sym == Key.Set)
	{
	}
	else if (keysym->sym == Key.Skip)
	{
	}
	else if (keysym->sym == Key.Cancel)
	{
	}
}

