/*
	Blokus SDL
		blapp.h

	This file includes class APP definition for Blokus.

	Copyright (c) 2003-2004 Antti Partanen


	This file is part of Blokus SDL.

	Blokus SDL is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	Blokus SDL is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with Blokus SDL; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef BLAPP_H
#define BLAPP_H

//**********************************************************
//* INCLUDES
#include "SDL/SDL_ttf.h"

#include "derrors.h"
#include "dpic.h"
#include "DNET/DNET.h"

#include "bldef.h"
#include "blbuttons.h"
#include "blstruct.h"
#include "blsound.h"
#include "blgame.h"

//**********************************************************
//* CLASS DEFINITIONS

//**********************************************************
/**
	Class APP, this is the main application class.
*/
class APP
{
public:
	//*********************************
	/* Public methods. */

	/* Constructor and destructor. */
	APP();
	~APP();

	/* Main application method. */
	void main(void);

private:
	//*********************************
	/* Private methods. */
	
	/* Main menu handler. */
	EVALUE menus_loop(void);

	/* MGAME. */
	void game(int button);

	/* MJOIN. */
	void join(int button);
	EVALUE join_check_clicked(int button, NET_HOST *hosts);
	void join_draw_buttons(int i, EVALUE err);
	void join_keys(SDL_keysym *keysym, bool ud);
	
	/* MMENU. */
	void menu(int button);
	void menu_keys(SDL_keysym *keysym, bool ud);
	
	/* MNICK. */
	void nick(int button);
	void nick_keys(SDL_keysym *keysym, bool ud);
	
	/* MSETTINGS. */
	void settings(int button);
	void settings_keys(SDL_keysym *keysym, bool ud);
	EVALUE settings_create_list(void);

	/* Class GAME. */
	GAME &g;
	void game_handle_key_down(SDL_keysym *keysym);
	void game_handle_key_up(SDL_keysym *keysym);
	void game_buttons(int selected_item);
	void game_buttons_before(int selected_item);
	void game_buttons_in(int selected_item);
	
	/*
		Changes video mode.
		(toggless between fullscreen and windowed mode)
	*/
	EVALUE change_video_mode(void);

	/* Get string. */
	void get_string(void);

	/* Processes events from system. */
	void process_events(bool wait);

	/* Releases resources of class APP. */
	void release(void);

	/* Handles key presses. */
	void handle_key_down(SDL_keysym* keysym);

	/* Handles key releases. */
	void handle_key_up(SDL_keysym* keysym);

	/* Handles string input. */
	void handle_get_string(SDL_keysym *keysym);

	/* Handles activate events. */
	void handle_activate(SDL_ActiveEvent *event);

	/* Many things which must be done when starting up. */
	EVALUE init(void);

	/* Refresh mouse state. */
	void mouse_refresh(void);

	/* Check pushed buttons. */
	int check_pushed(BUTTON *bts);

	/* Load graphics. */
	EVALUE graph_load(void);

	/* Release graphics. */
	void graph_release(void);

	/* Reset things by mode. */
	EVALUE reset_by_mode(int mode);

	/* Load graphics by mode. */
	EVALUE graph_load_by_mode(int mode);

	/* Release things (mostly graphics) by mode. */
	EVALUE release_by_mode(int mode);

	/* Load TTF-font. */
	TTF_Font *load_ttf(unsigned short id, int size, char *dtf);

	//*********************************
	/* Private variables. */
	/** Width of video surface (always 640). */
	unsigned int width;

	/** Height of video surface (always 480). */
	unsigned int height;

	/** Bits per pixel. */
	unsigned char bpp;

	/** Fullscreen when 1, windowed when 0. */
	unsigned int fullscreen;

	/** Mouse x-coordinate. */
	int mouse_x;

	/** Mouse y-coordinate. */
	int mouse_y;

	/** Mouse clicks. */
	unsigned char mouse_bt;
	
	/** Mouse double clicks. */
	unsigned char mouse_dbt;

	/** Mouse visible or not. */
	bool mouse_on;

	/** Buttons for main-menu. */
	BUTTON bmen;

	/** Buttons for settings-menu. */
	BUTTON bset;

	/** Buttons for nick-menu. */
	BUTTON bnic;

	/** Buttons for join-menu. */
	BUTTON bcre;

	/** Determining whether intput is selected or not. */
	bool string_get;

	/** Pointer to where the cursor is currently. */
	int string_cursor;

	/** Maximum length of input string. */
	int string_max_len;

	/** Input selection start. */
	int string_selstart;

	/** Input selection end. */
	int string_selend;

	/** Pointer to input buffer. */
	char *input;
	
	/** Width of input. */
	int input_w;

	/** Height of input. */
	int input_h;

	/** x-coordinate of input. */
	int input_x;
	
	/** y-coordinate of input. */
	int input_y;

	/** Pointer to where to start drawing input. */
	int input_where;

	/** Font for input. */
	TTF_Font *input_font;

	/** Color for input font. */
	SDL_Color input_color;

	/** Kinda frame rate counter. */
	unsigned int tpf;

	/** If scandinavians in use. */
	bool use_scandinavians;

	/** Determines which button or similar item is selected currently. */
	int selected_item;

	/** Tooltip counter. */
	int tool_tip;
	
	/** Tooltip x-coordinate. */
	int tool_tip_x;

	/** Tooltip y-coordinate. */
	int tool_tip_y;

	/** Surface for tooltip. */
	SDL_Surface *tool_tip_srf;

	/** For creating menu-lists. */
	DPIC foreMenuList;

	/** Class for game settings. */
	GAME_SETTINGS game_st;

	/** Class for application settings. */
	SETTINGS st;

	/** Buttons in game screen. */
	BUTTON bgam;

	/** Keyboard keys. */
	KEYMAP Key;

	/* For sounds. */
	/** Sound ID for game over. */
	int snd_gameover;

	/** Sound ID for next. */
	int snd_next;

	/** Class for playlist. */
	SND_PLAYLIST play_list;

	/** ID-list for songs. */
	unsigned short songs[MAX_SONGS];

	/** Background surface. */
	DPIC BG;

	/** Mouse cursor surface. */
	DPIC Cursor;

	/** Surface for ID-string. */
	SDL_Surface *Id_string;

	/** Surface for version drawn at right bottom of screen. */
	SDL_Surface *Version;

	/** Run the game or not. */
	bool run;

	/** Tick-variable for timing. */
	Uint32 ticks;

	/** Font for little infos. */
	TTF_Font *iFont;

	/** Font. */
	TTF_Font *dFont;

	/** Font. */
	TTF_Font *MFont;

	/** Font for chat screen. */
	TTF_Font *fChat;

	/** Color. */
	SDL_Color LText;

	/** Color. */
	SDL_Color DText;

	/** Color. */
	SDL_Color MLText;

	/** Color. */
	SDL_Color MDText;

	/** Color. */
	SDL_Color MMText;

	/** Color. */
	SDL_Color White;

	/** Color. */
	SDL_Color Black;

	/** Color. */
	SDL_Color Blue;

	/** Color. */
	SDL_Color Green;

	/** Color. */
	SDL_Color Red;

	/** Color. */
	SDL_Color Yellow;

	/** Color. */
	SDL_Color cChat;

	/** Color. */
	SDL_Color cTurn;

	/** Color. */
	SDL_Color cOther;

	/** Color. */
	SDL_Color cbOther;

	/** Mode-variable, which determines in which mode the application is. */
	int Mode;

	/**
		Mode, which the application should
		go back when exiting current menu.
	*/
	int rMode;

	/** Whether to reload things or not when looping through menus. */
	bool reload;

	/** Whether to reload background when looping through menus. */
	bool reload_bg;

	/** Main video surface. */
	SDL_Surface *vscreen;

	/** SDLKey. */
	SDLKey key;

	/** Get or not to get a key. */
	int key_get;

	/* For winamp control. */
#ifdef USE_WINAMP
	/** Winamp-handle. */
	void *winamp_hwnd;

	/** Winamp message. */
	int winamp_msg;

	/** Winamp message delay. */
	unsigned int winamp_msg_delay;

	/** Current winamp title. */
	char winamp_title[LEN_STRING];
#endif
};
//* End of class APP definition

#endif

