/*
	Blokus SDL
		blbuttons.cpp

	This file declares methods for class BUTTONS.

	Copyright (c) 2003-2004 Antti Partanen


	This file is part of Blokus SDL.

	Blokus SDL is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	Blokus SDL is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with Blokus SDL; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

//**********************************************************
//* INCLUDES
#include <string.h>
#include "blbuttons.h"

//**********************************************************
//* METHODS

//**********************************************************
/*
	class BUTTONS.
*/

//**********************************************************
/**
	Constructor.
*/
BUTTONS::BUTTONS()
{
	int i;

	for (i = 0; i < MAX_BUTTONS; i++)
	{
		name[i] = NULL;
		valstr[i] = NULL;
	}

	current = 0;
}

//**********************************************************
/**
	Destructor.
*/
BUTTONS::~BUTTONS()
{
	int i;

	for (i = 0; i < MAX_BUTTONS; i++)
	{
		if (name[i])
		{
			delete[] name[i];
			name[i] = NULL;
		}

		if (valstr[i])
		{
			delete[] valstr[i];
			valstr[i] = NULL;
		}
	}

	current = 0;
}

//**********************************************************
/**
	This function adds new button to given buttons.

	@param x x-coordinate.
	@param y y-coordinate.
	@param size_x Width of button.
	@param size_y Height of button.
	@param name Name-string.
	@param valstr Value-string.
	@return
	 -1, if not able to add new button.
	 Otherwise, the ID for the new button.
*/
int BUTTONS::add(int x3, int y3, int size_x, int size_y,
				 char *name_, char *valstr_)
{
	int xx1, xx2, yy1, yy2, x4, y4;

	x4 = x3 + size_x;
	y4 = y3 + size_y;
	
	if (current >= MAX_BUTTONS)
	{
		return (-1);
	}
	
	xx1 = x3 < x4 ? x3 : x4;
	xx2 = x4 <= x3 ? x3 : x4;
	yy1 = y3 < y4 ? y3 : y4;
	yy2 = y4 <= y3 ? y3 : y4;
	
	x1[current] = xx1;
	y1[current] = yy1;
	x2[current] = xx2;
	y2[current] = yy2;
	
	w[current] = size_x;
	h[current] = size_y;

	name[current] = new char[32];
	valstr[current] = new char[32];

	if (name_)
	{
		strncpy(name[current], name_, 31);
	}
	else
	{
		memset(name[current], '\0', 32);
	}

	if (valstr_)
	{
		strncpy(valstr[current], valstr_, 31);
	}
	else
	{
		memset(valstr[current], '\0', 32);
	}

	pushed[current] = 0;

	current++;	
	
	return (current);
}

//**********************************************************
/**
	This function checks given buttons.

	@param x x-coordinate.
	@param y y-coordinate.
	@param start Where to start check.
	@param end Where to end the check.
*/
void BUTTONS::check(int x, int y, int start, int end)
{
	int i;

	if (start < 0) start = 0;
	if (end < 0 || current < end) end = current;
	if (current < start) start = current;
	
	for(i = start; i <= end; i++)
	{
		if (x1[i] < x && x2[i] > x &&
			y1[i] < y && y2[i] > y)
		{
			pushed[i] = 1;
		}
		else
		{
			pushed[i] = 0;
		}
	}
}

//**********************************************************
/**
	This function checks given buttons.

	@param x x-coordinate.
	@param y y-coordinate.
*/
void BUTTONS::check(int x, int y)
{
	int i;
	
	for(i = 0; i <= current; i++)
	{
		if(x1[i] < x && x2[i] > x &&
		   y1[i] < y && y2[i] > y)
		{
			pushed[i] = 1;
		}
		else
		{
			pushed[i] = 0;
		}
	}
}

