/*
	Blokus SDL
		blbuttons.h

	This file defines class BUTTONS.

	Copyright (c) 2003-2004 Antti Partanen


	This file is part of Blokus SDL.

	Blokus SDL is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	Blokus SDL is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with Blokus SDL; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef BUTTONS_H
#define BUTTONS_H

//**********************************************************
//* INCLUDES
#include <cstring>

#include "SDL/SDL.h"

//**********************************************************
//* DEFINES
#define MAX_BUTTONS 64

#define BT_FLAG_NONE 0
#define BT_FLAG_START 1
#define BT_FLAG_MIDDLE 2
#define BT_FLAG_END 4

//**********************************************************
//* STRUCT DEFINITIONS

//**********************************************************
/**
	Struct for buttons.
*/
typedef struct
{
	/** x-coordinate. */
	int x;

	/** y-coordinate. */
	int y;

	/** Width of button. */
	int w;

	/** Height of button. */
	int h;

	/** Button ID-number. */
	int id;

	/** Current value of button. */
	int value;

	/** Some flags. */
	int flags;

	/** Name of button. */
	char *name;

	/** Extra info for button. */ 
	char *xstr;

	/**
		Values for different button stages.
		(maximum number of stages is 11)
	*/
	char *valstr[11];
} BT;

//**********************************************************
//* CLASS DEFINITIONS

//**********************************************************
/**
	Class BUTTONS.
*/
class BUTTONS
{
public:
	/* Public methods. */
	/* Constructor and destructor. */
	BUTTONS();
	~BUTTONS();

	/* Add new button. */
	int add(int x1, int y1, int size_x, int size_y,
		    char *name = NULL, char *valstr = NULL);

	/* Check buttons. */
	void check(int x, int y);

	/* Check buttons advanced. */
	void check(int x, int y, int start, int end);

	/* Public variables. */
	
	/** Pointer to last button added. */
	int current;

	/** Top x-coordinate. */
	int x1[MAX_BUTTONS];

	/** Top y-coordinate. */
	int y1[MAX_BUTTONS];

	/** Bottom x-coordinate. */
	int x2[MAX_BUTTONS];

	/** Bottom y-coordinate. */
	int y2[MAX_BUTTONS];

	/** Width. */
	int w[MAX_BUTTONS];

	/** Height. */
	int h[MAX_BUTTONS];

	/** Determines if the buttons has been pushed. */
	int pushed[MAX_BUTTONS];

	/** Name of the button. */
	char *name[MAX_BUTTONS];

	/** Value of the button. */
	char *valstr[MAX_BUTTONS];
};

//**********************************************************
/**
	Class for buttons.
*/
class BUTTON
{
public:
	/* Public mehods. */
	
	/** Constructor. */
	BUTTON()
	{
		memset(state, 0, sizeof(state));
		bt = NULL;
		infos = NULL;
	};

	/* Public variables. */

	/** Array for storing state of buttons. */
	int state[MAX_BUTTONS];

	/** Stores button information. */
	BUTTONS B;

	/** Pointer to struct BT. */
	BT *bt;

	/** Info for buttons. */
	char **infos;
};

#endif

