/*
	Blokus SDL
		blcommand.cpp

	This file declares methods for class COMMAND.

	Copyright (c) 2003-2004 Antti Partanen


	This file is part of Blokus SDL.

	Blokus SDL is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	Blokus SDL is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with Blokus SDL; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

//**********************************************************
//* INCLUDES
#include <cstring>
#include <new>

#include "blcommand.h"
#include "blexceptions.h"

//**********************************************************
//* CLASS METHODS

//**********************************************************
/*
	class COMMAND.
*/

//**********************************************************
/**
	First constructor.
*/
COMMAND::COMMAND(char *command, void (*x)(int), int v)
{
	/* Allocate memory for command string. */
	cmd = new char[strlen(command) + 1];

	/* Check allocation errors. */
	if (!cmd)
	{
		throw OutOfMemory("Out of memory, couldn't allocate"
			              " memory for class COMMAND command.\n",
						  __LINE__, __FILE__);
	}

	/* Copy command. */
	strcpy(cmd, command);

	/* Set function pointers. */
	execvoid = NULL;
	execvalue = x;
	var = NULL;

	/* Set value. */
	value = v;

	/* Set next to NULL. */
	next = NULL;
}

//**********************************************************
/**
	Second constructor.
*/
COMMAND::COMMAND(char *command, void (*x)(void))
{
	/* Allocate memory for command string. */
	cmd = new char[strlen(command) + 1];

	/* Check allocation errors. */
	if (!cmd)
	{
		throw OutOfMemory("Out of memory, couldn't allocate"
			              " memory for class COMMAND command.\n",
						  __LINE__, __FILE__);
	}

	/* Copy command. */
	strcpy(cmd, command);

	/* Set function pointers. */
	execvoid = x;
	execvalue = NULL;
	var = NULL;

	/* Set value. */
	value = 0;

	/* Set next to NULL. */
	next = NULL;
}

//**********************************************************
/**
	Third constructor.
*/
COMMAND::COMMAND(char *command, int *vari, int v)
{
	/* Allocate memory for command string. */
	cmd = new char[strlen(command) + 1];

	/* Check allocation errors. */
	if (!cmd)
	{
		throw OutOfMemory("Out of memory, couldn't allocate"
			              " memory for class COMMAND command.\n",
						  __LINE__, __FILE__);
	}

	/* Copy command. */
	strcpy(cmd, command);

	/* Set function pointers. */
	execvoid = NULL;
	execvalue = NULL;
	var = vari;

	/* Set value. */
	value = v;

	/* Set next to NULL. */
	next = NULL;
}

//**********************************************************
/**
	Destructor.
*/
COMMAND::~COMMAND()
{
	/* Delete next if such. */
	if (next)
	{
		delete next;
		next = NULL;
	}

	/* Delete command string. */
	if (cmd)
	{
		delete cmd;
	}

	/* Set pointers to NULL. */
	execvoid = NULL;
	execvalue = NULL;
}

//**********************************************************
/**
	Add command.
*/
void COMMAND::add(char *command, void (*x)(int), int v)
{
	next = new COMMAND(command, x, v);
}

//**********************************************************
/**
	Add command.
*/
void COMMAND::add(char *command, void (*x)(void))
{
	next = new COMMAND(command, x);
}

//**********************************************************
/**
	Add given command.
*/
void COMMAND::add(char *command, int *vari, int v)
{
	next = new COMMAND(command, vari, v);
}

//**********************************************************
/**
	Execute command.
*/
void COMMAND::exec(void)
{
	if (execvalue)
	{
		execvalue(value);
	}
	else if (execvoid)
	{
		execvoid();
	}
	else if (var)
	{
		*var = value;
	}
}

//**********************************************************
/**
	Execute command.
*/
void COMMAND::execv(int v)
{
	if (execvalue)
	{
		execvalue(v);
	}
	else if (execvoid)
	{
		execvoid();
	}
	else if (var)
	{
		*var = value;
	}
}

//**********************************************************
/**
	Find command.
*/
COMMAND *COMMAND::find(char *command)
{
	/* Variables. */
	int i;

	/* Compare. */
	i = strcmp(cmd, command);

	/* Check. */
	if (i == 0)
	{
		return (this);
	}
	else if (next)
	{
		return (next->find(command));
	}

	/* No command found. */
	return (NULL);
}

//**********************************************************
/**
	Find and execute command.

	@return Pointer to command, if command found. NULL, if not.
*/
COMMAND *COMMAND::find_and_exec(char *command, int v)
{
	/* Variables. */
	COMMAND *c;

	/* Find. */
	c = find(command);

	/* If found. */
	if (c)
	{
		c->execv(v);
	}

	/* Return. */
	return (c);
}

