/*
	Blokus SDL
		bldef.h

	This file includes most defines for Blokus.

	Copyright (c) 2003-2004 Antti Partanen


	This file is part of Blokus SDL.

	Blokus SDL is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	Blokus SDL is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with Blokus SDL; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef BLDEF_H
#define BLDEF_H

//**********************************************************
//* DEFINES

/* This is only for some safety. Not actually needed. */
#ifdef _DEBUG
#define DEBUG
#else
#ifdef NDEBUG
#ifdef DEBUG
#undef DEBUG
#endif
#endif
#endif

/* Define string for different platforms. */
#define b_WINDOWS "Microsoft Windows"
#define b_LINUX "Linux"
#define b_UNKNOWN "UNKNOWN"

/* Platform define. */
#ifdef _WIN32
#define PLATFORM b_WINDOWS
#elif __linux__
#define PLATFORM b_LINUX
#else
#define PLATFORM b_UNKNOWN
#endif

/*
	If windows, then use clipboard.
	(maybe someday I will implement this into linux too)
*/
#ifdef _WIN32
#define USE_CLIPBOARD
#endif

/*
	Also, if windows, maybe winamp interface
	is wanted, it's nice and easy. Actually not
	very usefull, becose in new winamp 5 there
	is winamp own hotkey plugin as default.
*/
#ifdef _WIN32
//#define USE_WINAMP /* Disabled since winamp 5. */
#define WINAMP_MSG_X 50
#define WINAMP_MSG_Y 460
#define WINAMP_MSG_DELAY 3000
#endif
/*
	WINAMP key shortcuts are (found in APP::HandleKeyDown()):
	F5:		play
	F6:		pause
	F7:		stop
	F8:		track title
	F9:		previous track
	F10:	next track
	F11:	volume down
	F12:	volume up
*/

/* Font defines. */
/* Most fonts are not loaded until needed. */

/* Font for some menus. */
#define FONT_NJ_NAME VERA
#define FONT_NJ_SIZE 20
#define FONT_NJ_STYLE TTF_STYLE_NORMAL

/* Font for nicks in join-menu. */
#define FONT_LN_NAME VERA
#define FONT_LN_SIZE 14
#define FONT_LN_STYLE TTF_STYLE_NORMAL

/* Tooltip font (should be always loaded in memory). */
#define FONT_I_NAME VERA
#define FONT_I_SIZE 11
#define FONT_I_STYLE TTF_STYLE_NORMAL

/* Highscore font. */
#define FONT_H_NAME VERA
#define FONT_H_SIZE 10
#define FONT_H_STYLE TTF_STYLE_NORMAL

/* Settings font. */
#define FONT_S_NAME COLLEGE
#define FONT_S_SIZE 12
#define FONT_S_STYLE TTF_STYLE_NORMAL

/* Chat screen font. */
#define CHAT_FONT_NAME VERA
#define CHAT_FONT_SIZE 9
#define CHAT_FONT_STYLE TTF_STYLE_NORMAL

/* Menu button font (should be always loaded in memory). */
#define MENU_FONT_NAME F1979RG
#define MENU_FONT_SIZE 28
#define MENU_FONT_STYLE TTF_STYLE_NORMAL

/* Some default string lenght values. */
#define LEN_SHORT_STRING 32
#define LEN_STRING 256
#define LEN_LONG_STRING 1024

/** Mouse double click speed in ms. */
#define DOUBLE_CLICK_SPEED 300

/**
	Waiting delay in milliseconds for incoming packets
	when searhing games in join screen. This value can
	be quite small becose these packets come from LAN,
	and their latency should be small.
*/
#define GAME_SEARCH_DELAY 1000

/** Delay in seconds while showing errors. */
#define ERROR_DELAY 5

/* Define default colors for text. */
#define TEXT_LIGHT_R 224
#define TEXT_LIGHT_G 128
#define TEXT_LIGHT_B 64

#define TEXT_DARK_R 16
#define TEXT_DARK_G 8
#define TEXT_DARK_B 64

#define MTEXT_LIGHT_R 255
#define MTEXT_LIGHT_G 128
#define MTEXT_LIGHT_B 48

#define MTEXT_MLIGHT_R 192
#define MTEXT_MLIGHT_G 64
#define MTEXT_MLIGHT_B 48

#define MTEXT_DARK_R 16
#define MTEXT_DARK_G 24
#define MTEXT_DARK_B 255

/**
	Maximum keyboard character input buffer.
	This buffer receives string written from
	keyboard every time when application wants
	an string as input.
	(chat screen, join address, new nickname)
*/
#define MAX_INPUT LEN_STRING

/** Defines for modes. */
enum
{
	MMENU = 0,
	MGAME,
	MSETTINGS,
	MNICK,
	MJOIN,
	MREDRAW,
	MEXIT
};

#endif



