/*
	Blokus SDL
		blevent.cpp

	This file includes class APP and class GAME event handling functions.

	Copyright (c) 2003-2004 Antti Partanen


	This file is part of Blokus SDL.

	Blokus SDL is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	Blokus SDL is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with Blokus SDL; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

//**********************************************************
//* INCLUDES
#ifdef _WIN32
#include <windows.h>
#endif

#include "txtfile.h"

#include "blapp.h"
#include "blsettings.h"
#include "blmisc.h"

//**********************************************************
//* METHODS

//**********************************************************
/*
	class APP.
*/

//**********************************************************
/**
	This function handles input string, if in use.

	@param keysym Keyboard symbol in SDL_keysym-struct pointer.
*/
void APP::handle_get_string(SDL_keysym *keysym)
{
	if (string_selend != string_selstart)
	{
		if (keysym->sym != SDLK_LEFT &&
			keysym->sym != SDLK_RIGHT &&
			keysym->sym != SDLK_DELETE &&
			keysym->sym != SDLK_BACKSPACE &&
			keysym->sym != SDLK_HOME &&
			keysym->sym != SDLK_END &&
			(SDL_GetModState() & (KMOD_SHIFT | KMOD_ALT | KMOD_CTRL)) == KMOD_NONE)
		{
			strcpy(input + string_selstart, input + string_selend);
			string_selend = string_selstart = 0;
		}
	}

	if (string_cursor > (int)strlen(input))
	{
		string_cursor = (int)strlen(input);
	}

	if (keysym->sym == SDLK_a && (KMOD_CTRL & SDL_GetModState()))
	{
		string_selstart = 0;
		string_selend = (int)strlen(input);
		string_cursor = (int)strlen(input);
		return;
	}
#ifdef USE_CLIPBOARD
	else if (keysym->sym == SDLK_v && (KMOD_CTRL & SDL_GetModState()))
	{
		if (OpenClipboard(NULL))
		{
			if (string_selstart < string_selend)
			{
				strcpy(input + string_selstart, input + string_selend);
				string_cursor = string_selstart;
				string_selstart = string_selend = 0;
			}

			int n = strlen(input);
			char *string = (char *)GetClipboardData(CF_TEXT);
			char *input2 = new char[string_max_len + 2];
			
			if (string)
			{
				if ((int)strlen(string) >= (string_max_len - n - 1))
				{
					string[string_max_len - n - 1] = '\0';
				}

				strcpy(input2, input);
				input2[string_cursor] = '\0';
				strncat(input2, string,
					    string_max_len - strlen(input2) - 1);
				strncat(input2, input + string_cursor,
					    string_max_len - strlen(input2) - 1);
				strcpy(input, input2);
				string_cursor += strlen(input) - n;
			}

			delete[] input2;
			CloseClipboard();

			string_selend = string_selstart = 0;
		}

		return;
	}
	else if (keysym->sym == SDLK_c && (KMOD_CTRL & SDL_GetModState()))
	{
		if (string_selstart == string_selend ||
			string_selstart < 0 ||
			string_selend > (int)strlen(input));
		else if (OpenClipboard(NULL))
		{
			LPTSTR string;
			HGLOBAL copy = GlobalAlloc(GMEM_MOVEABLE, strlen(input) + 1);

			EmptyClipboard();

			char ch = input[string_selend];
			input[string_selend] = '\0';

			string = (char *)GlobalLock(copy); 
			strcpy(string, input + string_selstart); 
			GlobalUnlock(copy);
			
			SetClipboardData(CF_TEXT, copy);
			char *st = (char *)GetClipboardData(CF_TEXT);

			CloseClipboard();
			
			GlobalFree(copy);
			
			input[string_selend] = ch;
		}

		return;
	}
	else if (keysym->sym == SDLK_x && (KMOD_CTRL & SDL_GetModState()))
	{
		if (string_selstart == string_selend ||
			string_selstart < 0 ||
			string_selend > (int)strlen(input));
		else if (OpenClipboard(NULL))
		{
			LPTSTR string;
			HGLOBAL copy = GlobalAlloc(GMEM_MOVEABLE, strlen(input) + 1);

			EmptyClipboard();

			char ch = input[string_selend];
			input[string_selend] = '\0';

			string = (char *)GlobalLock(copy); 
			strcpy(string, input + string_selstart); 
			GlobalUnlock(copy);
			
			SetClipboardData(CF_TEXT, copy);
			char *st = (char *)GetClipboardData(CF_TEXT);

			CloseClipboard();
			
			GlobalFree(copy);
			
			input[string_selend] = ch;

			strcpy(input + string_selstart, input + string_selend);
			string_selend = string_selstart = 0;
		}

		return;
	}
#endif

	switch (keysym->sym)
	{
	case SDLK_RETURN:
	case SDLK_ESCAPE:
		break;

	case SDLK_DELETE:
	{
		if (string_selstart < string_selend)
		{
			string_cursor = string_selstart;
			strcpy(input + string_selstart, input + string_selend);
			string_selstart = string_selend = 0;
		}
		else if ((int)strlen(input) > string_cursor)
		{
			strcpy(&input[string_cursor], &input[string_cursor + 1]);
		}

		return;
	}

	case SDLK_HOME:
	{
		if (SDL_GetModState() & KMOD_SHIFT)
		{
			if (string_selstart == string_selend)
			{
				string_selend = string_cursor;
				string_selstart = 0;
				string_cursor = 0;
			}
			else if (string_cursor == string_selend)
			{
				string_selend = string_selstart;
				string_cursor = 0;
				string_selstart = 0;
			}
			else
			{
				string_cursor = 0;
				string_selstart = 0;
			}
		}
		else
		{
			string_cursor = 0;
			string_selstart = 0;
			string_selend = 0;
		}

		return;
	}

	case SDLK_END:
	{
		if (SDL_GetModState() & KMOD_SHIFT)
		{
			if (string_selstart == string_selend)
			{
				string_selstart = string_cursor;
				string_selend = strlen(input);
				string_cursor = strlen(input);
			}
			else if (string_cursor == string_selstart)
			{
				string_selstart = string_selend;
				string_cursor = strlen(input);
				string_selend = strlen(input);
			}
			else
			{
				string_cursor = strlen(input);
				string_selend = strlen(input);
			}
		}
		else
		{
			string_cursor = strlen(input);
			string_selstart = 0;
			string_selend = 0;
		}

		return;
	}

	case SDLK_LEFT:
	{
		if (string_cursor > 0)
		{
			string_cursor--;

			if (KMOD_SHIFT & SDL_GetModState())
			{
				if (string_selstart == string_selend)
				{
					string_selend = string_cursor + 1;
					string_selstart = string_cursor;
				}
				else if (string_cursor >= string_selstart)
				{
					string_selend--;
				}
				else
				{
					string_selstart--;
				}
			}
			else
			{
				string_selstart = string_selend = 0;
			}
		}

		return;
	}

	case SDLK_RIGHT:
	{
		if (string_cursor < (int)strlen(input))
		{
			string_cursor++;

			if (KMOD_SHIFT & SDL_GetModState())
			{
				if (string_selstart == string_selend)
				{
					string_selstart = string_cursor - 1;
					string_selend = string_cursor;
				}
				else if (string_cursor <= string_selend)
				{
					string_selstart++;
				}
				else
				{
					string_selend++;
				}
			}
			else
			{
				string_selstart = string_selend = 0;
			}
		}

		return;
	}

	case SDLK_BACKSPACE:
	{
		if (string_selstart < string_selend)
		{
			string_cursor = string_selstart;
			strcpy(input + string_selstart, input + string_selend);
			string_selstart = string_selend = 0;
		}

		if (strlen(input) > 0 && string_cursor > 0)
		{
			strcpy(&input[string_cursor - 1], &input[string_cursor]);
			string_cursor--;
		}

		return;
	}

	default:
	{
		if (keysym->unicode > 0 && keysym->unicode < 0x80)
		{
			if (string_selstart < string_selend)
			{
				string_cursor = string_selstart;
				strcpy(input + string_selstart, input + string_selend);
				string_selstart = string_selend = 0;
			}

			if (Mode == MNICK &&
				keysym->sym == SDLK_SPACE &&
				strlen(input) < 1);
			else if ((int)strlen(input) < (string_max_len - 1))
			{
				int i;
				
				input[strlen(input) + 1] = '\0';

				for (i = strlen(input); i > string_cursor; i--)
				{
					input[i] = input[i - 1];
				}

				input[string_cursor] = (char)keysym->unicode;
				string_cursor++;
			}
		}
		else if (keysym->unicode > 0 && use_scandinavians)
		{
			if (string_selstart < string_selend)
			{
				string_cursor = string_selstart;
				strcpy(input + string_selstart, input + string_selend);
				string_selstart = string_selend = 0;
			}

			char ch = '\0';

			switch (keysym->unicode)
			{
			case 0x00e5:
				ch = '';
				break;
			case 0x00e4:
				ch = '';
				break;
			case 0x00f6:
				ch = '';
				break;
			case 0x00c5:
				ch = '';
				break;
			case 0x00c4:
				ch = '';
				break;
			case 0x00d6:
				ch = '';
				break;
			}

			if ((int)strlen(input) < (string_max_len - 1) &&
				use_scandinavians)
			{
				int i;
				
				input[strlen(input) + 1] = '\0';

				for (i = strlen(input); i > string_cursor; i--)
				{
					input[i] = input[i - 1];
				}

				input[string_cursor] = (char)keysym->unicode;
				string_cursor++;
			}
		}
		return;
	}
	}

	return;
}

//**********************************************************
/**
	This function handles keydown-events.

	@param keysym Keyboard symbol in SDL_keysym-struct pointer.
*/
void APP::handle_key_down(SDL_keysym *keysym)
{
	if (string_get == true)
	{
		handle_get_string(keysym);
	}
	else if (Mode == MNICK)
	{
		if (keysym->sym == SDLK_UP && st.current_nick > 0)
		{
			st.current_nick--;
		}
		else if (keysym->sym == SDLK_DOWN &&
			     strlen(st.Nicks[st.current_nick + 1]) > 0)
		{
			st.current_nick++;
		}
	}
	else if (Mode == MJOIN)
	{
		if (keysym->sym == SDLK_UP && selected_item > 0)
		{
			selected_item--;
		}
		else if (keysym->sym == SDLK_DOWN &&
			     selected_item < (MAX_GAMES - 1))
		{
			selected_item++;
		}
	}

#ifdef USE_WINAMP
	int n = 40044;
#endif

	switch (keysym->sym)
	{
#ifdef USE_WINAMP
		/* Show song title. */
	case SDLK_F8:
		n++;

		/* vol down (40059). */
	case SDLK_F11:
		n++;

		/* vol up (40058). */
	case SDLK_F12:
		n += 10;

		/* next (40048). */
	case SDLK_F10:
		n++;

		/* stop (40047). */
	case SDLK_F7:
		n++;
		
		/* pause (40046). */
	case SDLK_F6:
		n++;
		
		/* play (40045). */
	case SDLK_F5:
		n++;
		
		/* previous (40044). */
	case SDLK_F9:

		/* If winamp handle not found yet, try locating it. */
		if (!winamp_hwnd)
		{
			winamp_hwnd = FindWindow("Winamp v1.x", NULL);
		}

		/* Now, send the message. */
		if (winamp_hwnd)
		{
			/* Get title. */
			if (n == 40060)
			{
				char *this_title, *p;

				strcpy(winamp_title, "Winamp playing: ");
				int len = strlen(winamp_title);
				this_title = winamp_title + len;

				GetWindowText((HWND)winamp_hwnd, this_title,
					          sizeof(winamp_title) - len);
				p = this_title + strlen(this_title) - 8;
				
				while (p >= this_title)
				{
					if (!strnicmp(p, "- Winamp", 8))
					{
						break;
					}

					p--;
				}

				if (p >= this_title)
				{
					p--;
				}

				while (p >= this_title && *p == ' ')
				{
					p--;
				}

				*++p = 0;
			}
			/* Send message. */
			else
			{
				SendMessage((HWND)winamp_hwnd, WM_COMMAND, n, 0);
			}
			
			winamp_msg = n;
			winamp_msg_delay = SDL_GetTicks() + WINAMP_MSG_DELAY;
		}

		break;
#endif

	case SDLK_F4:
		/* Handle quit requests first. */
		if (KMOD_ALT & SDL_GetModState())
		{
			run = 0;
			Mode = MEXIT;
			break;
		}
		else if (Mode == MGAME)
		{
			game_handle_key_down(keysym);
		}

		break;

	case SDLK_RETURN:
		if (Mode != MGAME || (KMOD_ALT & SDL_GetModState()));
		else if (string_get)
		{
			char string[MAX_INPUT + MAX_NICKLEN + 3];
		
			if (strlen(input) > 0)
			{
				if (input[0] != '/')
				{
					sprintf(string, "<%s> ", st.Nicks[st.current_nick]);
					strcat(string, input);
					g.add_chat_line(string);
					NET_SendString(string, NET_ID_ALL);
				}
				else
				{
					g.handle_chat_command(input);
				}

				memset(input, '\0', MAX_INPUT);
				string_cursor = 0;
			}

			string_get = true;

			break;
		}

	default:
		switch (Mode)
		{
		case MGAME:
			game_handle_key_down(keysym);
			break;

		case MMENU:
			menu_keys(keysym, true);
			break;

		case MSETTINGS:
			settings_keys(keysym, true);
			break;

		case MNICK:
			nick_keys(keysym, true);
			break;

		case MJOIN:
			join_keys(keysym, true);
			break;
		}
		break;
	}

	if (keysym->sym == SDLK_ESCAPE)
	{
		key = SDLK_UNKNOWN;
	}
	else
	{
		key = keysym->sym;
	}
}

//**********************************************************
/**
	This function handles keyup-events.

	@param keysym Keyboard symbol in SDL_keysym-struct pointer.
*/
void APP::handle_key_up(SDL_keysym *keysym)
{
	key = SDLK_UNKNOWN;

	switch (keysym->sym)
	{
	case SDLK_RETURN:
#ifndef _DEBUG
		if (Mode == MGAME)
		{
			break;
		}
#endif

		/* See if ALT-ENTER is pressed, then toggle fullscreen. */
		if (KMOD_ALT & SDL_GetModState())
		{
			/* Toggle fullscreen. */
			st.fullscreen = (st.fullscreen + 1) & 1;
			change_video_mode();
			
			/* Refresh settings. */
			if (Mode == MSETTINGS)
			{
				bset.bt[bSbpp].value = st.bpp;
				bset.bt[bSfullscreen].value = st.fullscreen;
			}

			Mode = MREDRAW;

			/*
				Must clear the KMOD_ALT, becose SDL
				seems to leave it on sometimes.
				(becose change of video mode probably)
			*/
			SDLMod x = SDLMod(SDL_GetModState() & (KMOD_ALT ^ 0xffff));
			SDL_SetModState(x);

			break;
		}

	default:
		switch (Mode)
		{
		case MGAME:
			game_handle_key_up(keysym);
			break;

		case MMENU:
			menu_keys(keysym, false);
			break;
		
		case MSETTINGS:
			settings_keys(keysym, false);
			break;

		case MNICK:
			nick_keys(keysym, false);
			break;

		case MJOIN:
			join_keys(keysym, false);
			break;
		}

		break;

	/* Take screenshot from current main sdl video surface. */
	case SDLK_PRINT:
	case SDLK_SYSREQ:
		ScreenShot();
		break;
	}
}


//**********************************************************
/**
	This function handles all events.

	@param wait
	 Set to true, if this function shouldn't return,
	 until some event occurs.
*/
void APP::process_events(bool wait)
{
    /* SDL event placeholder. */
    SDL_Event event;

    /* Grab all the events off the queue. */
    while (true)
	{
		if (wait)
		{
			if (!SDL_WaitEvent(&event))
			{
				FILE_TextI("WARNING: SDL_WaitEvent() failed! (SDL error %s)",
					SDL_GetError());
				break;
			}

			wait = false;
		}
		else if (!SDL_PollEvent(&event))
		{
			break;
		}

        switch (event.type)
		{
        case SDL_KEYDOWN:
            /* Handle key presses. */
            handle_key_down(&event.key.keysym);
            break;

        case SDL_KEYUP:
            /* Handle key presses. */
            handle_key_up(&event.key.keysym);
            break;

        case SDL_QUIT:
            /* Handle quit requests. */
			run = 0;
			Mode = MEXIT;
            break;

		case SDL_ACTIVEEVENT:
			handle_activate((SDL_ActiveEvent *)&event);
			break;

		case SDL_MOUSEBUTTONUP:
			Mode = Mode;
			break;
        }
    }
}

//**********************************************************
/**
	This function alt-tab in fullscreen. Means, that when
	user pushes alt-tab to return to fullscreen
	propably only from windows, then this function resets
	the background graphics.

	@param event Pointer to SDL_ActiveEvent-struct.
*/
void APP::handle_activate(SDL_ActiveEvent *event)
{
	/* Handle ALT-TAB in fullscreen. */
	if (st.fullscreen &&
		(event->state & SDL_APPACTIVE) &&
		event->gain)
	{
		SDL_Flip(vscreen);
		SDL_BlitSurface(BG.srf, NULL, vscreen, NULL);
		SDL_Flip(vscreen);
		SDL_BlitSurface(BG.srf, NULL, vscreen, NULL);
	}
}

//**********************************************************
/**
	This function refreshes the mouse state.
*/
void APP::mouse_refresh(void)
{
	/* Variables. */
	static unsigned int dblclick = 0;
	static unsigned char mdbl = 0;

	/* Get mouse state. */
	mouse_bt = SDL_GetMouseState(&mouse_x, &mouse_y);

	/* Test for double clicks. */
	unsigned char mbt = SDL_BUTTON(1) & mouse_bt;

	if (mbt != mouse_dbt)
	{
		mouse_dbt = 0;
	}

	if (mbt)
	{
		if (dblclick > SDL_GetTicks() && mdbl != mbt)
		{
			dblclick = 0;
			mouse_dbt = mbt;
			mdbl = 0;
		}
		else if (mdbl != mbt)
		{
			dblclick = SDL_GetTicks() + DOUBLE_CLICK_SPEED;
			mdbl = mbt;
		}
	}
	else
	{
		mdbl = 0;
	}

	/* Return. */
	return;
}

