/*
	Blokus SDL
		blgame.cpp

	This file declares different menu functions for class GAME.

	Copyright (c) 2003-2004 Antti Partanen


	This file is part of Blokus SDL.

	Blokus SDL is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	Blokus SDL is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with Blokus SDL; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

//**********************************************************
//* INCLUDES
#include "txtfile.h"

#include "blapp.h"
#include "blgame.h"
#include "blmisc.h"
#include "blexceptions.h"
#include "blfiles.h"
#include "blokus.h"

//**********************************************************
//* METHODS

//**********************************************************
/*
	class GAME.
*/

//**********************************************************
/**
	GAME-class constructor.
*/
GAME::GAME() :
	box(*new TEXTBOX(CHAT_X, CHAT_Y, CHAT_W, CHAT_H,
	                 CHAT_LINES, CHAT_LINE,
					 NULL, NULL)),
	store(STORE_X, STORE_Y, STORE_XSIZE, STORE_YSIZE,
	      BRICK_XSIZE, BRICK_YSIZE),
	board(BOARD_X, BOARD_Y, BOARD_XSIZE, BOARD_YSIZE,
	      BRICK_XSIZE, BRICK_YSIZE)
{
	/* Check for class TEXTBOX reference for allocation failures. */
	if (!&box)
	{
		FILE_TextI("Failed to allocate class TEXTBOX.\n");
		throw OutOfMemory("Failed to allocate class TEXTBOX.\n",
			              __LINE__, __FILE__);
	}

	/* Add commands to chat screen. */
	cmds_create();
};

//**********************************************************
/**
	GAME-class destructor.
*/
GAME::~GAME()
{
	/* Delete class TEXTBOX. */
	delete &box;
}

//**********************************************************
/**
	Get selected brick.
*/
int GAME::get_brick(void)
{
	return (selected_brick);
}

//**********************************************************
/**
	Get selected brick position.
*/
int GAME::get_pos(void)
{
	return (s_pos);
}

//**********************************************************
/**
	Set selected brick position.
*/
void GAME::set_pos(int pos)
{
	s_pos = pos;
}

//**********************************************************
/**
	See if game is running.
*/
bool GAME::game_run(void)
{
	return (game_running);
}

//**********************************************************
/**
	Get ID for given color.
*/
int GAME::get_color_id(int color)
{
	return (color_ids[color]);
}

//**********************************************************
/**
	Get number of players.
*/
int GAME::get_nof_players(void)
{
	return (nofplayers);
}

//**********************************************************
/**
	Get points for given color.
*/
int GAME::get_points(int color)
{
	return (points[color]);
}

//**********************************************************
/**
	Get current color.
*/
int GAME::get_current_color(void)
{
	return (current_color);
}

//**********************************************************
/**
	Get round number.
*/
int GAME::get_round(void)
{
	return (round);
}

//**********************************************************
/**
	Draw chat.
*/
void GAME::draw_chat(SDL_Surface *srf)
{
	box.draw(srf);
}

//**********************************************************
/**
	Set color and font for chat.
*/
void GAME::set_chat(SDL_Color *color, TTF_Font *font)
{
	box.set_color(color);
	box.set_font(font);
}

//**********************************************************
/**
	Add line to chat.
*/
void GAME::add_chat_line(char *string)
{
	box.add_line(string);
}

//**********************************************************
/**
	Handle chat commands.
*/
void GAME::handle_chat_command(char *string)
{
	box.handle_command(&string[1]);
}

//**********************************************************
/**
	Set playing settings.
*/
void GAME::set_settings(GAME_SETTINGS *st)
{
	memcpy(&play_st, st, sizeof(GAME_SETTINGS));
}
