/*
	Blokus SDL
		blgame.h

	This file defines class GAME.

	Copyright (C) 2003-2004 Antti Partanen


	This file is part of Blokus SDL.

	Blokus SDL is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	Blokus SDL is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with Blokus SDL; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef BLGAME_H
#define BLGAME_H

//**********************************************************
//* INCLUDES
#include "SDL/SDL_ttf.h"

#include "dpic.h"
#include "DNET/DNET.h"

#include "blstruct.h"
#include "blbuttons.h"
#include "blsound.h"
#include "blgrid.h"
#include "blmybricks.h"
#include "bltextbox.h"

//**********************************************************
//* DEFINES
#define PFLAG_NORMAL 0x0
#define PFLAG_WAITING 0xffffffff

/* Screen. */
const int BRICK_XSIZE = 16;
const int BRICK_YSIZE = 16;

const int BOARD_X = 16;
const int BOARD_Y = 16;
const int BOARD_XSIZE = 20;
const int BOARD_YSIZE = 20;
const int STORE_X = 0;
const int STORE_Y = 352;
const int STORE_XSIZE = 40;
const int STORE_YSIZE = 8;

#define BG_UW 50
#define BG_UH 16

#define BG_SX 380
#define BG_SY 320
#define BG_CX 480
#define BG_CY 320
#define BG_EX 580
#define BG_EY 320

#define BG_PLAYERX 375
#define BG_PLAYERY 242
#define BG_PLAYERW 250
#define BG_PLAYERH 13
#define BG_PLAYERYADD 13

#define CHAT_TEXT_R 200
#define CHAT_TEXT_G 100
#define CHAT_TEXT_B 60

#define GAME_TURN_TEXT_R 200
#define GAME_TURN_TEXT_G 255
#define GAME_TURN_TEXT_B 200

#define GAME_OTHER_TEXT_R 64
#define GAME_OTHER_TEXT_G 160
#define GAME_OTHER_TEXT_B 64

#define GAME_OTHER_TEXTB_R 0
#define GAME_OTHER_TEXTB_G 100
#define GAME_OTHER_TEXTB_B 0

/*
	Maximum number of shown characters per chat line.
	The real value of characters per line is calculated
	by font, and can vary a much depending on characters
	in the line.
*/
#define CHAT_LINE 200

/* Number of lines in chat screen. */
#define CHAT_LINES 18

#define CHAT_X 370
#define CHAT_Y 16
#define CHAT_W 256
#define CHAT_H 184
#define CHATADD 10

#define BG_WCHATX 370
#define BG_WCHATY 210
#define BG_WCHATW 265
#define BG_WCHATH 16

#define COLOR_BLUE 0
#define COLOR_GREEN 1
#define COLOR_RED 2
#define COLOR_YELLOW 3

#define MAX_SONGS 10

enum
{
	bGskip = 0,
	bGsettings,
	bGexit,

	bGstore,
	bGboard,

	bGplayer1,
	bGplayer2,
	bGplayer3,
	bGplayer4,

	bGchat,

	bGN
};

//**********************************************************
//* CLASS DEFINITIONS

//**********************************************************
/**
	Class GAME.
*/
class GAME
{
public:
	//*********************************
	/* Public methods. */

	/* Constructor and destructor. */
	GAME(void);
	~GAME(void);

	/* Init class GAME. */
	EVALUE init(void);

	/* Handle network. */
	void handle_network(void);

	/* Draw everything into given surface. */
	void draw(int mx, int my, int selected_item,
		      SDL_Surface *srf);

	/* Get selected brick. */
	int get_brick(void);
	
	/* Get selected brick position. */
	int get_pos(void);

	/* Set selected brick position. */
	void set_pos(int pos);
	
	/* See if game is running or not. */
	bool game_run(void);

	/* Get color id for given color. */
	int get_color_id(int color);

	/* Get number of players. */
	int get_nof_players(void);

	/* Get points of given color. */
	int get_points(int color);

	/* Get current color. */
	int get_current_color(void);

	/* Get round number. */
	int get_round(void);

	/* Set chat color and font. */
	void set_chat(SDL_Color *color, TTF_Font *font);

	/* Add line to chat. */
	void add_chat_line(char *string);

	/* Handle chat commands. */
	void handle_chat_command(char *string);

	/* Set settings. */
	void set_settings(GAME_SETTINGS *play_st);

	/* When player clicks skip. */
	void clicked_skip(void);

	/* When player clicks store. */
	void clicked_store(int mx, int my);

	/* When player clicks board. */
	void clicked_board(int mx, int my);

	/* When player clicks exit. */
	void clicked_exit(void);

	/* When player clicks settings. */
	void clicked_settings(void);

	/* When player clicks chat. */
	void clicked_chat(void);

	/* When player clicks right mouse button. */
	void clicked_right(void);

	/* When exiting the game-menu. */
	void quit(void);

	/* Create command list. */
	void cmds_create(void);

private:
	//*********************************
	/* Private methods. */

	/* Find selected brick from store. */
	void find_selected(int mx, int my);
	
	/* Try to relocate selected brick to store. */
	void try_relocate(int mx, int my);
	
	/* Try to set selected brick to board. */
	void try_set(int mx, int my);

	/* Check if brick os over some other brick or over board. */
	bool try_over(int x, int y);
	
	/* Check if the brick can be placed, when not round zero. */
	bool try_not_zero(int x, int y);

	/* Check if the brick can be placed, when round zero. */
	bool try_zero(int x, int y, int color);

	/* Find out whos turn is next. */
	void next_turn(void);

	/* Set brick to board. */
	void set_brick(int x, int y, int color, int pos, int brick);

	/* Deal colors to players. */
	void deal_colors(void);

	/* Start/restart game. */
	void restart(void);

	/* Called, when game ends. */
	void ends(void);

	/* Draw chat. */
	void draw_chat(SDL_Surface *srf);

	/* Handles network packets. */
	char handle_packet(NET_PACKET *p);

	/* Draw color markers to board sides. */
	void draw_marker(int color, SDL_Surface *srf);

	/* Find out next color. */
	int turn(int color);

	/* Calculate next turn by number of players. */
	void turn_f1(void);
	void turn_f2(void);
	void turn_f3(void);
	void turn_f4(void);

	//*********************************
	/* Private variables. */
	/** Grid for game board. */
	GRID board;

	/** Grid for store area. */
	GRID store;

	/** Bricks. */
	MY_BRICKS my_bricks;

	/** Image for bricks drawn into screen. */
	DPIC brick_image;

	/** Playing settings. */
	GAME_SETTINGS play_st;
	
	/** Chat screen. */
	TEXTBOX &box;

	/** Currently selected brick. */
	int selected_brick;

	/** x-offset of currently selected brick relative to mouse. */
	int s_xoff;

	/** y-ffset of currently selected brick relative to mouse. */
	int s_yoff;

	/** Position of currently selected brick. */
	int s_pos;

	/** Determines if it is our turn. */
	bool my_turn;

	/** Determines our color. */
	int my_color;

	/** Number of players in game. */
	int nofplayers;

	/** Current round. */
	int round;

	/** Game ended or not. */
	bool end;

	/** Network IDs for colors. */
	int color_ids[4];

	/** Current color. */
	int current_color;

	/** Which colors are playing and which have surrended. */
	bool color_gaming[4];

	/** Is game on or not. */
	bool game_running;

	/** Current points for every color. */
	int points[4];

	/** Connection error flag. */
	bool connection_error;
};
/* End of class GAME definition. */

#endif

