/*
	Blokus SDL
		blgbrick.cpp

	This file declares some functions for handling
	bricks in the game. Drawing them in screen and such.

	Copyright (c) 2003-2004 Antti Partanen


	This file is part of Blokus SDL.

	Blokus SDL is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	Blokus SDL is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with Blokus SDL; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

//**********************************************************
//* INCLUDES
#include "blgame.h"
#include "blvideo.h"

//**********************************************************
//* METHODS

//**********************************************************
/*
	class GAME.
*/

//**********************************************************
/**
	This function draws almost everything in the screen.
*/
void GAME::draw(int mx, int my, int selected_item,
				SDL_Surface *srf)
{
	/* Variables. */
	int lx;
	int ly;
	unsigned char p;

	/* Show other players bricks, if selected some. */
	int selected_color = my_color;

	/* Check that is game even running. */
	if (game_running)
	{
		switch (selected_item)
		{
		case bGplayer1:
			selected_color = COLOR_BLUE;
			break;
		case bGplayer2:
			selected_color = COLOR_GREEN;
			break;
		case bGplayer3:
			selected_color = COLOR_RED;
			break;
		case bGplayer4:
			selected_color = COLOR_YELLOW;
			break;
		}
	}

	/* Clear store area. */
	store.clear();

	/* Set bricks to store area. */
	store.set(selected_brick, selected_color, &my_bricks);

	/* Draw store area. */
	store.draw(play_st.tile_set, &brick_image, srf);

	/* Draw board. */
	board.draw(play_st.tile_set, &brick_image, srf);

	/* Draw marker for our color. */
	draw_marker(my_color, srf);

	/* Draw selected brick, if such. */
	if (selected_brick >= 0)
	{
		for (lx = 0; lx < 5; lx++)
		{
			for (ly = 0; ly < 5; ly++)
			{
				int x = mx - s_xoff + lx * BRICK_XSIZE;
				int y = my - s_yoff + ly * BRICK_XSIZE;
	
				p = my_bricks.get_brick_value(lx, ly, s_pos, selected_brick);
				
				if (p == 0)
				{
					VID_Blit(x, y, my_color, play_st.tile_set,
						     &brick_image, srf);
				}
			}
		}
	}

	/* Draw chat screen. */
	draw_chat(srf);

	/* Return. */
	return;
}


//**********************************************************
/**
	This function is used to find the selected brick,
	when user clicks left mouse button in store-area.
*/
void GAME::draw_marker(int color, SDL_Surface *srf)
{
	/* Draw marker for our own corner. */
	Uint32 clr;
	SDL_Rect rr1;
	SDL_Rect rr2;

	switch (color)
	{
	case COLOR_BLUE:
		clr = SDL_MapRGB(srf->format, 0, 0, 255);
		rr1.w = 32;
		rr1.h = 2;
		rr1.x = BOARD_X;
		rr1.y = BOARD_Y + BOARD_YSIZE * BRICK_YSIZE - 1;
		rr2.w = 2;
		rr2.h = 32;
		rr2.x = BOARD_X - 1;
		rr2.y = BOARD_Y + BOARD_YSIZE * BRICK_YSIZE - 32;
		break;

	case COLOR_GREEN:
		clr = SDL_MapRGB(srf->format, 0, 255, 0);
		rr1.w = 32;
		rr1.h = 2;
		rr1.x = BOARD_X + BOARD_XSIZE * BRICK_XSIZE - 32;
		rr1.y = BOARD_Y + BOARD_YSIZE * BRICK_YSIZE - 1;
		rr2.w = 2;
		rr2.h = 32;
		rr2.x = BOARD_X + BOARD_XSIZE * BRICK_XSIZE - 1;
		rr2.y = BOARD_Y + BOARD_YSIZE * BRICK_YSIZE - 32;
		break;

	case COLOR_RED:
		clr = SDL_MapRGB(srf->format, 255, 0, 0);
		rr1.w = 32;
		rr1.h = 2;
		rr1.x = BOARD_X + BOARD_XSIZE * BRICK_XSIZE - 32;
		rr1.y = BOARD_Y - 1;
		rr2.w = 2;
		rr2.h = 32;
		rr2.x = BOARD_X + BOARD_XSIZE * BRICK_XSIZE - 1;
		rr2.y = BOARD_Y;
		break;

	case COLOR_YELLOW:
		clr = SDL_MapRGB(srf->format, 255, 255, 0);
		rr1.w = 32;
		rr1.h = 2;
		rr1.x = BOARD_X;
		rr1.y = BOARD_Y - 1;
		rr2.w = 2;
		rr2.h = 32;
		rr2.x = BOARD_X - 1;
		rr2.y = BOARD_Y;
		break;
	}

	/* Now draw the marker. */
	SDL_FillRect(srf, &rr1, clr);
	SDL_FillRect(srf, &rr2, clr);
	
	/* Return. */
	return;
}

//**********************************************************
/**
	This function is used to find the selected brick,
	when user clicks left mouse button in store-area.
*/
void GAME::find_selected(int mx, int my)
{
	if (mx < STORE_X || mx > (STORE_X + (STORE_XSIZE * BRICK_XSIZE)))
	{
		return;
	}

	if (my < STORE_Y || my > (STORE_Y + (STORE_YSIZE * BRICK_YSIZE)))
	{
		return;
	}

	int x = (mx - STORE_X) / BRICK_XSIZE;
	int y = (my - STORE_Y) / BRICK_YSIZE;

	if (store.get_value(x, y) != 0xff)
	{
		selected_brick = (store.get_value(x, y) & 0xf8) >> 3;
	}

	s_xoff = mx - STORE_X -
		     my_bricks.get_x(selected_brick, my_color) *
		     BRICK_XSIZE;

	s_yoff = my - STORE_Y -
		     my_bricks.get_y(selected_brick, my_color) *
		     BRICK_YSIZE;

	s_pos = my_bricks.get_pos(selected_brick, my_color);
}

//**********************************************************
/**
	If some brick is found to fit to somewhere, then
	this function is used to place the brick there.
*/
void GAME::set_brick(int x, int y, int color, int pos, int brick)
{
	/* Variables. */
	int lx;
	int ly;
	unsigned char p;

	/* Set the brick to board. */
	for (lx = 0; lx < 5; lx++)
	{
		for (ly = 0; ly < 5; ly++)
		{
			p = my_bricks.get_brick_value(lx, ly, pos, brick);

			if (p == 0)
			{
				board.set_value(color, lx + x, ly + y);
			}
		}
	}

	/* Disable brick. */
	my_bricks.disable_brick(brick, color);

	/* Return. */
	return;
}

