/*
	Blokus SDL
		blgend.cpp

	This file declares functions for game end
	and game quit.

	Copyright (c) 2003-2004 Antti Partanen


	This file is part of Blokus SDL.

	Blokus SDL is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	Blokus SDL is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with Blokus SDL; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

//**********************************************************
//* INCLUDES
#include "blgame.h"

//**********************************************************
//* METHODS

//**********************************************************
/*
	class GAME.
*/

//**********************************************************
/**
	This function is called when skip button is selected
	in main game screen.
*/
void GAME::clicked_skip(void)
{
	if (game_running && round > 0 && !end)
	{
		color_gaming[my_color] = false;
		char xx[2];
		xx[0] = 'i';
		xx[1] = my_color;
		NET_Send(xx, 2, NET_ID_ALL, NET_TYPE_STRING);
		next_turn();
	}
	else if (NET_Host() && !game_running)
	{
		if (end)
		{
			restart();
		}

		deal_colors();
		NET_SendString("go", NET_ID_ALL);
		game_running = true;
		NET_SetClosed(true);
		my_turn = true;
		current_color = COLOR_BLUE;
		my_color = COLOR_BLUE;
		box.add_line("***** Game started!");
	}
}

//**********************************************************
/**
	This function is called when left mouse button is
	clicked in store-area.
*/
void GAME::clicked_store(int mx, int my)
{
	if (selected_brick != -1)
	{
		try_relocate(mx, my);
	}
	else
	{
		find_selected(mx, my);
	}
}

//**********************************************************
/**
	This function is called when left mouse button is
	clicked in board area.
*/
void GAME::clicked_board(int mx, int my)
{
	if (game_running && my_turn)
	{
		try_set(mx, my);
	}
}

//**********************************************************
/**
	This function is called when exit-button is clicked.
*/
void GAME::clicked_exit(void)
{
}

//**********************************************************
/**
	This function is called when chatline-area is clicked.
*/
void GAME::clicked_chat(void)
{
}

//**********************************************************
/**
	This function is called when settings-button is clicked.
*/
void GAME::clicked_settings(void)
{
}

//**********************************************************
/**
	This function is called when right mouse button is
	clicked.
*/
void GAME::clicked_right(void)
{
	selected_brick = -1;
}
