/*
	Blokus SDL
		blgrid.cpp

	This file declares methods for class GRID.

	Copyright (C) 2003-2004 Antti Partanen


	This file is part of Blokus SDL.

	Blokus SDL is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	Blokus SDL is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with Blokus SDL; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

//**********************************************************
//* INCLUDES
#include <new>
#include <cstring>

#include "derrors.h"
#include "txtfile.h"
#include "dpic.h"

#include "blgrid.h"
#include "blvideo.h"
#include "blexceptions.h"

//**********************************************************
//* CLASS METHODS

//**********************************************************
/*
	class GRID.
*/

//**********************************************************
/**
	Constructor.
*/
GRID::GRID(int _x, int _y, int _w, int _h, int _one_x, int _one_y)
{
	/* Allocate memory for area. */
	area = new unsigned char[_w * _h];

	/* Check allocation failures. */
	if (!area)
	{
		throw OutOfMemory("Failed to allocate memory for storea area table.\n",
			              __LINE__, __FILE__);
	}

	/* Set variables. */
	coord_x = _x;
	coord_y = _y;
	width = _w;
	height = _h;
	one_x = _one_x;
	one_y = _one_y;
}

//**********************************************************
/**
	Destructor.
*/
GRID::~GRID()
{
	delete[] area;
}

//**********************************************************
/**
	Clear grid.
*/
void GRID::clear(unsigned char value)
{
	/* Clear store area. */
	memset(area, value, width * height);

	/* Return. */
	return;
}

//**********************************************************
/**
	Draw bricks to store-buffer.
*/
void GRID::set(int selected_brick,
				int selected_color,
				MY_BRICKS *my_bricks)
{
	/* Variables. */
	int lx;
	int ly;
	int lz;
	
	int x;
	int y;
	int id;
	int pos;

	unsigned char br;
	unsigned char *rea;

	/* Now set given bricks to area. */
	for (lx = 0; lx < 21; lx++)
	{
		if (!my_bricks->get_brick(lx, selected_color))
		{
			continue;
		}

		id = my_bricks->get_id(lx, selected_color);
		x = my_bricks->get_x(lx, selected_color);
		y = my_bricks->get_y(lx, selected_color);
		pos = my_bricks->get_pos(lx, selected_color);

		for (ly = 0; ly < 5; ly++)
		{
			if ((ly + x) >= width || (ly + x) < 0)
			{
				continue;
			}

			for (lz = 0; lz < 5; lz++)
			{
				br = my_bricks->get_brick_value(ly, lz, pos, id);

				if ((lz + y) >= height || (lz + y) < 0
					|| br != 0)
				{
					continue;
				}
				
				rea = &area[ly + x + (lz + y) * width];

				if (selected_brick == id)
				{
					*rea = 5 | (id << 3);
				}
				else if ((*rea & 0x7) == 5);
				else if (*rea != 0xff)
				{
					*rea = 4 | (id << 3);
				}
				else
				{
					*rea = 7 | (id << 3);
				}
			}
		}
	}
}

//**********************************************************
/**
	Draw grid to given surface.
*/
void GRID::draw(int tile_set,
				 DPIC *brick_image,
				 SDL_Surface *srf)
{
	/* Variables. */
	int lx;
	int ly;
	int x;
	int y;
	unsigned short pc;
	unsigned char p;

	/* Now draw area to screen. */
	for (lx = 0; lx < width; lx++)
	{
		for (ly = 0; ly < height; ly++)
		{
			p = area[lx + ly * width];

			if (p == 0xff)
			{
				continue;
			}

			pc = p & 0x7;
			
			x = lx * one_x + coord_x;
			y = ly * one_y + coord_y;

			VID_Blit(x, y, pc, tile_set,
				     brick_image, srf);
		}
	}
}

//**********************************************************
/**
	Get value of grid in specified coordinates.
*/
unsigned char GRID::get_value(int x, int y)
{
	/* Variables. */
	unsigned char ret = 0xff;
	int cz = x + y * width;

	/* Check, that values are in range. */
	if (x > width || y > height)
	{
		return (ret);
	}

	/* Get value. */
	ret = area[cz];

	/* Return. */
	return (ret);
}

//**********************************************************
/**
	Set value of grid in specified coordinates.
*/
void GRID::set_value(unsigned char value, int x, int y)
{
	/* Variables. */
	int cz = x + y * width;

	/* Check, that values are in range. */
	if (x > width || y > height)
	{
		return;
	}

	/* Get value. */
	area[cz] = value;

	/* Return. */
	return;
}

//**********************************************************
/**
	Get pointer to grid data.
*/
unsigned char *GRID::get_area_pointer(void)
{
	/* Just return pointer. */
	return (area);
}

