/*
	Blokus SDL
		blgstart.cpp

	This file declares functions for game start
	and restart.

	Copyright (c) 2003-2004 Antti Partanen


	This file is part of Blokus SDL.

	Blokus SDL is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	Blokus SDL is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with Blokus SDL; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

//**********************************************************
//* INCLUDES
#include "txtfile.h"

#include "dmf.h"

#include "blfiles.h"
#include "bldef.h"
#include "blgame.h"
#include "blokus.h"
#include "blsound.h"
#include "blmybricks.h"
#include "blexceptions.h"
#include "blmisc.h"

//**********************************************************
//* METHODS

//**********************************************************
/*
	class GAME.
*/

//**********************************************************
/**
	Game initializer. This function is run when
	game is joined or created from join-menu.
*/
EVALUE GAME::init(void)
{
	/* Variables. */
	EVALUE err;

	/* Load brick image. */
	err = (EVALUE)LoadDMF(&brick_image, BRICKIMAGE, DMF_A_NAME, true);

	/* Check errors. */
	if (err != A_OK)
	{
		FILE_TextI("Failed to load brick image.\n");
		throw OutOfMemory("Failed to load brick image.\n",
			              __LINE__, __FILE__);
	}

	/* Resets some things. */
	restart();
	nofplayers = NET_GetNumOfConnections();
	end = false;
	memset(color_ids, 0, sizeof(color_ids));
	color_ids[COLOR_BLUE] = NET_GetMyID();
	my_color = COLOR_BLUE;

	/* Return. */
	return (A_OK);
}

//**********************************************************
/**
	This function is called when game is started or
	restarted. It resets many class GAME variables.
*/
void GAME::restart(void)
{
	/* Clear grid. */
	board.clear();

	/* Clear store area. */
	store.clear();

	/* Reset bricks to starting position. */
	my_bricks.reset();

	/* Reset some other variables. */

	/* Not my turn at start (unless the blue-one). */
	my_turn = false;

	/* No selected brick. */
	selected_brick = -1;

	/* Direction and coordinate offsets. */
	s_pos = 0;
	s_xoff = 0;
	s_yoff = 0;

	/* Game's not running yet. */
	game_running = false;
	
	/* Round is zero. */
	round = 0;
	
	/* No current color yet. */
	current_color = -1;

	/* Everybody is in the game. */
	color_gaming[COLOR_BLUE] = true;
	color_gaming[COLOR_GREEN] = true;
	color_gaming[COLOR_RED] = true;
	color_gaming[COLOR_YELLOW] = true;

	/* Reset points. */
	memset(points, 0, sizeof(points));

	/* Restart. */
	end = false;

	/* Return. */
	return;
}

