/*
	Blokus SDL
		blgtry.cpp

	This file declares some functions for handling
	bricks in the game. Mostly for trying to set them
	on board and such.

	Copyright (c) 2003-2004 Antti Partanen


	This file is part of Blokus SDL.

	Blokus SDL is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	Blokus SDL is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with Blokus SDL; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

//**********************************************************
//* INCLUDES
#include "blgame.h"

//**********************************************************
//* METHODS

//**********************************************************
/*
	class GAME.
*/

//**********************************************************
/**
	This function tries to set selected brick to new
	location in store-area. This happens when player has
	selected a brick and then click right mouse button.
*/
void GAME::try_relocate(int mx, int my)
{
	/* Variables. */
	int lx;
	int ly;
	unsigned char p;

	/*
		Now calculate the position of the brick from
		the mouse position given in mx and my.
		This is not as simple as I first thought.
	*/
	float x1 = float(mx - STORE_X - s_xoff) / float(BRICK_XSIZE);
	float y1 = float(my - STORE_Y - s_yoff) / float(BRICK_YSIZE);
	int x;
	int y;

	/*
		See if the brick has negative coordinates relative
		to store-area. If so, then the coordinates must be
		fixed with -0.5, not +0.5.
	*/
	if (x1 < 0)
	{
		x = int(x1 - 0.5f);
	}
	else
	{
		x = int(x1 + 0.5f);
	}

	if (y1 < 0)
	{
		y = int(y1 - 0.5f);
	}
	else
	{
		y = int(y1 + 0.5f);
	}

	/*
		This determines whether the brick is still
		allowed to be set after the check.
	*/
	bool can = true;

	/*
		Check that the brick doesn't
		go over the store-area.
	*/
	for (lx = 0; lx < 5; lx++)
	{
		for (ly = 0; ly < 5; ly++)
		{
			/* This makes the code a bit cleaner. */
			p = my_bricks.get_brick_value(lx, ly, s_pos, selected_brick);

			/* These make the code a bit cleaner too. */
			int xx = lx + x;
			int yy = ly + y;

			if (p == 0 && /* No check needed, if empty at here. */
				(xx < 0 || /* Over at left side? */
				xx >= STORE_XSIZE || /* Over at right side? */
				yy < 0 || /* Over at top? */
				yy >= STORE_YSIZE)) /* Over at bottom? */
			{ /* If over somewhere, then failed. */
				can = false; /* Failed. */
			}
		}
	}

	/* If not failed, the set the brick here. */
	if (can)
	{
		/* x-coordinate. */
		my_bricks.set_x(x, selected_brick, my_color);
		/* y-coordinate. */
		my_bricks.set_y(y, selected_brick, my_color);
		/* Position. */
		my_bricks.set_pos(s_pos, selected_brick, my_color);
		/* And unselect. */
		selected_brick = -1;
	}
}

//**********************************************************
/**
	This function tries to set selected brick to the
	board. In some ways very similar to try_relocate().
*/
void GAME::try_set(int mx, int my)
{
	/* Variables. */
	bool can;

	/*
		Now calculate the position of the brick from
		the mouse position given in mx and my.
		This is not as simple as I first thought.
	*/
	float x1 = float(mx - BOARD_X - s_xoff) / float(BRICK_XSIZE);
	float y1 = float(my - BOARD_Y - s_yoff) / float(BRICK_YSIZE);
	int x;
	int y;

	/*
		See if the brick has negative coordinates relative
		to store-area. If so, then the coordinates must be
		fixed with -0.5, not +0.5.
	*/
	if (x1 < 0)
	{
		x = int(x1 - 0.5f);
	}
	else
	{
		x = int(x1 + 0.5f);
	}

	if (y1 < 0)
	{
		y = int(y1 - 0.5f);
	}
	else
	{
		y = int(y1 + 0.5f);
	}

	/*
		See if the brick is over some
		other brick or over the board.
	*/
	can = try_over(x, y);	

	/* If failed, in other word over something, return. */
	if (!can)
	{
		return;
	}

	/*
		See if the blokus rules come true.
		If round zero, then there must be
		a brick in our own corner.
	*/

	/* If not round zero. */
	if (round != 0)
	{
		can = try_not_zero(x, y);
	}
	/* Check for round zero. */
	else
	{
		can = try_zero(x, y, my_color);
	}

	/* If check failed, the return. */
	if (!can)
	{
		return;
	}

	/* The brick can be placed here. */
	set_brick(x, y, my_color, s_pos, selected_brick);

	/*
		Add points according to the square
		amount of the placed brick.
	*/
	points[my_color] += my_bricks.get_points(selected_brick, my_color);

	/*
		Inform other users, that the
		brick was placed here.
	*/
	char xp[2 + sizeof(BRICK)];
	xp[0] = 'p';
	xp[1] = my_color;

	BRICK b;
	b.id = selected_brick;
	b.pos = s_pos;
	b.x = x;
	b.y = y;
	memcpy(&xp[2], &b, sizeof(BRICK));

	NET_Send(xp, 2 + sizeof(BRICK), NET_ID_ALL, NET_TYPE_STRING);
	
	/* Check, if that was the last brick. */
	can = true;

	if (my_bricks.n_of_bricks(my_color))
	{
		can = false;
	}

	/* If it was, then we're finished. */
	if (can)
	{
		/*
			If last brick was the one square,
			then add some extra points.
		*/
		if (selected_brick == 0)
		{
			points[my_color] += 5;
		}

		/*
			Add some extra points for
			finishing the color.
		*/
		points[my_color] += 15;

		/* Send notice to other players. */
		color_gaming[my_color] = false;
		char xx[2];
		xx[0] = 'i';
		xx[1] = my_color;
		NET_Send(xx, 2, NET_ID_ALL, NET_TYPE_STRING);
	}

	/* Send new points for this user to other players. */
	xp[0] = 's';
	xp[1] = my_color;
	xp[2] = points[my_color];
	
	NET_Send(xp, 3, NET_ID_ALL, NET_TYPE_STRING);

	/* Unselect selected brick. */
	selected_brick = -1;

	/* Set next turn. */
	next_turn();
}

//**********************************************************
/**
	Check if the brick is over some other brick or over
	the board.
*/
bool GAME::try_over(int x, int y)
{
	/* Variables. */
	int lx;
	int ly;
	unsigned char p1;
	unsigned char p2;
	bool can;

	/* Set can to true. */
	can = true;

	/*
		See if the brick is over some
		other brick or over the board.
	*/
	for (lx = 0; lx < 5; lx++)
	{
		for (ly = 0; ly < 5; ly++)
		{
			/* This makes the code a bit cleaner. */
			p1 = my_bricks.get_brick_value(lx, ly, s_pos, selected_brick);

			/* These make the code a bit cleaner too. */
			int xx = lx + x;
			int yy = ly + y;

			if (xx < 0 || /* Over at left side? */
				xx >= BOARD_XSIZE || /* Over at right side? */
				yy < 0 || //* Over at top?
				yy >= BOARD_YSIZE) /* Over at bottom? */
			{
				/*
					If over somewhere and something
					at this spot, the failed.
				*/
				if (p1 == 0)
				{
					can = false;
					break;
				}
				/* Just over, no fear yet. */
				else
				{
					continue;
				}
			}

			/* Check that there is no over placement. */
			p2 = board.get_value(lx + x, ly + y);
			
			if (p1 == 0 && p2 != 0xff)
			{
				can = false;
				break;
			}
		}

		/* If already failed, the bail out. */
		if (can == false)
		{
			break;
		}
	}

	/* Return. */
	return (can);
}

//**********************************************************
/**
	Check if the brick can be placed, when not round zero.
*/
bool GAME::try_not_zero(int x, int y)
{
	/* Variables. */
	bool can = true;
	bool xcan = false;
	int lx;
	int ly;
	unsigned char p1;

	/* Do check. */
	for (lx = 0; lx < 5; lx++)
	{
		for (ly = 0; ly < 5; ly++)
		{
			int xx = lx + x;
			int yy = ly + y;

			p1 = my_bricks.get_brick_value(lx, ly, s_pos, selected_brick);

			if (p1 != 0)
			{
				continue;
			}

			/*
				Test if there is some my_color
				brick in wrong place.
			*/
			if (xx > 0)
			{
				if ((board.get_value(xx - 1, yy) & 0x7)
					== my_color)
				{
					can = false;
					break;
				}
			}
			
			if (xx < (BOARD_XSIZE - 1))
			{
				if ((board.get_value(xx + 1, yy) & 0x7)
					== my_color)
				{
					can = false;
					break;
				}
			}

			if (yy > 0)
			{
				if ((board.get_value(xx, yy - 1) & 0x7)
					== my_color)
				{
					can = false;
					break;
				}
			}

			if (yy < (BOARD_YSIZE - 1))
			{
				if ((board.get_value(xx, yy + 1) & 0x7)
					== my_color)
				{
					can = false;
					break;
				}
			}

			/* 
				Test if there is some my_color
				brick in right place.
			*/
			if (xx > 0 && yy > 0)
			{
				if ((board.get_value(xx - 1, yy - 1) & 0x7)
					== my_color)
				{
					xcan = true;
				}
			}
			
			if (xx > 0 && yy < (BOARD_YSIZE - 1))
			{
				if ((board.get_value(xx - 1, yy + 1) & 0x7)
					== my_color)
				{
					xcan = true;
				}
			}

			if (xx < (BOARD_XSIZE - 1) && yy > 0)
			{
				if ((board.get_value(xx + 1, yy - 1) & 0x7)
					== my_color)
				{
					xcan = true;
				}
			}

			if (xx < (BOARD_XSIZE - 1) && yy < (BOARD_YSIZE - 1))
			{
				if ((board.get_value(xx + 1, yy + 1) & 0x7)
					== my_color)
				{
					xcan = true;
				}
			}
		}

		if (can == false)
		{
			break;
		}
	}

	/* If not. */
	if (can == false || xcan == false)
	{
		return (false);
	}

	/* Return true. */
	return (true);
}


//**********************************************************
/**
	If some brick is found to fit to somewhere, then
	this function is used to place the brick there.
*/
bool GAME::try_zero(int x, int y, int color)
{
	/* Variables. */
	int lx;
	int ly;
	int xf;
	int yf;
	bool can = false;
	unsigned char p1;

	/* Do check by color. */
	switch (color)
	{
	case COLOR_BLUE:
		xf = 0;
		yf = BOARD_YSIZE - 1;
		break;
	case COLOR_GREEN:
		xf = BOARD_XSIZE - 1;
		yf = BOARD_YSIZE - 1;
		break;
	case COLOR_RED:
		xf = BOARD_XSIZE - 1;
		yf = 0;
		break;
	case COLOR_YELLOW:
		xf = 0;
		yf = 0;
		break;
	}

	/* Do check. */
	for (lx = 0; lx < 5; lx++)
	{
		for (ly = 0; ly < 5; ly++)
		{
			p1 = my_bricks.get_brick_value(lx, ly, s_pos, selected_brick);

			int xx = lx + x;
			int yy = ly + y;

			if (p1 == 0 &&
				xf == xx &&
				yf == yy)
			{
				can = true;
				break;
			}
		}

		if (can == true)
		{
			break;
		}
	}

	/* Return. */
	return (can);
}

