/*
	Blokus SDL
		blgturn.cpp

	This file declares class GAME functions for changing turn.

	Copyright (c) 2003-2004 Antti Partanen


	This file is part of Blokus SDL.

	Blokus SDL is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	Blokus SDL is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with Blokus SDL; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

//**********************************************************
//* INCLUDES
#include "blgame.h"
#include "blvideo.h"

//**********************************************************
//* METHODS

//**********************************************************
/*
	class GAME.
*/

//**********************************************************
/**
	Find out whose turn is next, and what is our color
	next.
*/
void GAME::next_turn(void)
{
	/* Variables. */
	/* For network packet. */
	char x[2];

	/* First check that it is our turn. */
	if (!my_turn)
	{
		return;
	}

	/* It was our turn, not anymore. Probably. */
	my_turn = false;

	/* Check next turn by number of players. */
	switch (nofplayers)
	{
	/* If only one player. */
	default:
	case 1:
		turn_f1();
		break;

	/* Two players. */
	case 2:
		turn_f2();
		break;

	/* Three players. */
	case 3:
		turn_f3();
		break;

	/* Four players. */
	case 4:
		turn_f4();
		break;
	}

	/* 'n' means next turn. */
	if (nofplayers > 1)
	{
		x[0] = 'n';
		x[1] = (char)current_color;
		NET_Send(x, 2, NET_ID_ALL, NET_TYPE_STRING);
	}
}

//**********************************************************
/**
	Find out which color comes next.
*/
int GAME::turn(int color)
{
	/* Variables. */
	int next = -1;

	/* Find out next color. */
	switch (color)
	{
	case COLOR_BLUE:
		next = COLOR_GREEN;

		if (color_gaming[next])
		{
			break;
		}

	case COLOR_GREEN:
		next = COLOR_RED;

		if (color_gaming[next])
		{
			break;
		}

	case COLOR_RED:
		next = COLOR_YELLOW;

		if (color_gaming[next])
		{
			break;
		}

	case COLOR_YELLOW:
		/* Next round. */
		round++;

		/* Notice other players for next round. */
		if (nofplayers > 1)
		{
			NET_SendString("round", NET_ID_ALL);
		}

		next = COLOR_BLUE;

		if (color_gaming[next])
		{
			break;
		}

		next = COLOR_GREEN;

		if (color_gaming[next])
		{
			break;
		}

		next = COLOR_RED;

		if (color_gaming[next])
		{
			break;
		}

		next = COLOR_YELLOW;

		if (color_gaming[next])
		{
			break;
		}

		my_turn = false;
		ends();
		return (-1);
	}

	/* Return. */
	return (next);
}

//**********************************************************
/**
	Find out whose turn is next, when 1 players.
*/
void GAME::turn_f1(void)
{
	/* Variables. */
	int next = COLOR_BLUE;

	/* Set my_turn to true, becose only one player. */
	my_turn = true;

	/* Find out next color. */
	next = turn(my_color);

	/* If no new color (turn return -1). */
	if (next < 0)
	{
		/* Return. */
		return;
	}

	/* Set current color. */
	current_color = next;

	/* Set our color. */
	my_color = next;

	/* Return. */
	return;
}

//**********************************************************
/**
	Find out whose turn is next, when 2 players.
*/
void GAME::turn_f2(void)
{
	/* Variables. */
	int next = COLOR_BLUE;

	/* Find out next color. */
	next = turn(current_color);

	/* If no new color (turn return -1). */
	if (next < 0)
	{
		/* Inform the other player about the game end. */
		NET_SendString("end", NET_ID_ALL);

		/* Return. */
		return;
	}

	/* Set current color. */
	current_color = next;

	/* Find out our color. */
	my_color ^= 2;

	/* If this color is not playing, then choose the other one. */
	if (!color_gaming[my_color])
	{
		my_color ^= 2;
	}

	/* If this color is next. */
	if (my_color == current_color)
	{
		my_turn = true;
	}
	else
	{
		my_turn = false;
	}

	/* Return. */
	return;
}

//**********************************************************
/**
	Find out whose turn is next, when 3 players.
*/
void GAME::turn_f3(void)
{
	/* Variables. */
	int next = COLOR_BLUE;
	static int real_color;

	/* If not playing with yellow, then set real color. */
	if (my_color != COLOR_YELLOW)
	{
		real_color = my_color;
	}

	/* Find out next color. */
	next = turn(current_color);

	/* If no new color (turn return -1). */
	if (next < 0)
	{
		/* Inform the other player about the game end. */
		NET_SendString("end", NET_ID_ALL);

		/* Return. */
		return;
	}

	/* Set current color. */
	current_color = next;

	/* Find out our color. */
	if (my_color == COLOR_YELLOW)
	{
		my_color = real_color;
	}
	else if (my_color == ((round + 3) % 3) &&
		     color_gaming[COLOR_YELLOW])
	{
		my_color = COLOR_YELLOW;
	}

	/* If this color is next. */
	if (my_color == current_color)
	{
		my_turn = true;
	}
	else
	{
		my_turn = false;
	}

	/* Return. */
	return;
}

//**********************************************************
/**
	Find out whose turn is next, when 4 players.
*/
void GAME::turn_f4(void)
{
	/* Variables. */
	int next = COLOR_BLUE;

	/* Find out next color. */
	next = turn(current_color);

	/* If no new color (turn return -1). */
	if (next < 0)
	{
		/* Inform the other player about the game end. */
		NET_SendString("end", NET_ID_ALL);

		/* Return. */
		return;
	}

	/* Set current color. */
	current_color = next;

	/* If this color is next. */
	if (my_color == current_color)
	{
		my_turn = true;
	}
	else
	{
		my_turn = false;
	}

	/* Return. */
	return;
}
