/*
	Blokus SDL
		bljoin.cpp

	This file includes function Join() for class APP

	Copyright (c) 2003-2004 Antti Partanen


	This file is part of Blokus SDL.

	Blokus SDL is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	Blokus SDL is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with Blokus SDL; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#define ERROR_STRINGS_INUSE

//**********************************************************
//* INCLUDES
#include "derrors.h"

#include "blapp.h"
#include "bljoin.h"
#include "blvideo.h"
#include "blnetwork.h"

//**********************************************************
//* FUNCTIONS

//**********************************************************
/**
	This function handles the joining into a game, creating
	a game and starting a single player game.

	@param button If some button is clicked, then deliver
	              it's value in here. If none, then -1.
*/
void APP::join(int button)
{
	/* Static variables. */
	static NET_HOST hosts[MAX_GAMES];
	static NET_HOST hosts2[MAX_GAMES];
	static EVALUE err_delay = 0;
	static EVALUE err = A_OK;
	static int nofgames = 0;
	
	/* Variables. */
	SDL_Rect r;
	int i;
	unsigned int ly;
	unsigned int li;
	unsigned int la;
	unsigned int lu;

	/* Find selected item, if such. */
	for (;
	     strlen(hosts2[selected_item].name) < 1;
	     selected_item++)
	{
		if (selected_item < 0)
		{
			break;
		}
	}

	/* If some error, then wait while and show it. */
	if (err_delay > 0)
	{
		err_delay--;
	}
	else
	if (NET_GetState(NET_STATEOF_ENUM) == NET_STATE_CLOSED)
	{
		/* Set to zero. */
		lu = 0;

		/* Sort the sessions by IP-address. */
		for (i = 0; i < nofgames; i++)
		{
			li = 0xffffffff;
			la = 0;

			/* Find IP which matches next. */
			for (ly = 0; ly < MAX_GAMES; ly++)
			{
				if (hosts[ly].ip == 0)
				{
					continue;
				}

				if (hosts[ly].ip < li &&
					hosts[ly].ip > lu)
				{
					li = hosts[ly].ip;
					la = ly;
				}
			}

			lu = li;

			/*
				Copy found struct
				to the other array.
			*/
			memcpy((void *)&hosts2[i],
				   (void *)&hosts[la],
				   sizeof(NET_HOST));
		}

		/* Clear empty slots. */
		for (i = nofgames; i < MAX_GAMES; i++)
		{
			memset((void *)&hosts2[i], 0, sizeof(NET_HOST));
		}

		/*
			If no selected item, then set the first
			found session as selected.
		*/
		if (selected_item < 0 && nofgames > 0)
		{
			selected_item = 0;
		}

		/* Reset variables. */
		nofgames = MAX_GAMES;
		memset((void *)&hosts, 0, sizeof(NET_HOST) * MAX_GAMES);

		/* Now, start enumerating sessions from LAN. */
		NET_StartEnum((NET_HOST *)&hosts,
			          (unsigned int *)&nofgames,
					  GAME_SEARCH_DELAY,
					  NET_PORT);
	}

	/* Check clicked buttons. */
	if (button > -1)
	{
		err = join_check_clicked(button, hosts2);

		if (err != A_OK)
		{
			err_delay = 30 * ERROR_DELAY;
		}
	}

	/* Draw buttons. */
	for (i = 0; i < bCN; i++)
	{
		join_draw_buttons(i, err);
	}

	/* Draw some more buttons. */
	for (i = bCN; i < (bCN + MAX_GAMES); i++)
	{
		r.w = bcre.B.w[i];
		r.h = bcre.B.h[i];
		r.x = bcre.B.x1[i];
		r.y = bcre.B.y1[i];
		SDL_BlitSurface(BG.srf, &r, vscreen, &r);

		if (err != A_OK);
		else if ((i - bCN) == selected_item &&
			     strlen(hosts2[i - bCN].name) > 0)
		{
			VID_RenderTTF2Srf(bcre.B.x1[i], bcre.B.y1[i],
				hosts2[i - bCN].name, &MLText, vscreen, dFont);
		}
		else if (strlen(hosts2[i - bCN].name) > 0)
		{
			VID_RenderTTF2Srf(bcre.B.x1[i], bcre.B.y1[i],
				hosts2[i - bCN].name, &MDText, vscreen, dFont);
		}
	}

	/* Draw some text. */
	VID_RenderTTF2Srf(128, 38, "LAN games:", &White, vscreen, dFont);
	VID_RenderTTF2Srf(128, 326, "Status:", &White, vscreen, dFont);

	/* Return. */
	return;
}

//**********************************************************
/**
	Draw buttons in join menu.
*/
void APP::join_draw_buttons(int i, EVALUE err)
{
	/* Variables. */
	SDL_Rect r;

	/*
		State-button. The one which views
		current action and errors.
	*/
	if (i == bCstate && !string_get)
	{
		/* Set rect struct values. */
		r.w = bcre.B.w[i];
		r.h = bcre.B.h[i];
		r.x = bcre.B.x1[i];
		r.y = bcre.B.y1[i];

		/* Clear that area from screen to background view. */
		SDL_BlitSurface(BG.srf, &r, vscreen, &r);

		/* View error. */
		switch (err)
		{
		case E_NETFULL:
			VID_RenderTTF2Srf(bcre.B.x1[i], bcre.B.y1[i],
					          "Can't join, game is full.",
					          &MLText, vscreen, dFont);
			break;

		case E_NETCLOSED:
			VID_RenderTTF2Srf(bcre.B.x1[i], bcre.B.y1[i],
					          "Can't join, game is closed.",
					          &MLText, vscreen, dFont);
			break;

		case E_NETTIMEOUT:
			VID_RenderTTF2Srf(bcre.B.x1[i], bcre.B.y1[i],
					          "Connection timeout, can't join.",
					          &MLText, vscreen, dFont);
			break;
		
		case E_NETRESOLVE:
			VID_RenderTTF2Srf(bcre.B.x1[i], bcre.B.y1[i],
					          "Host not found.",
					          &MLText, vscreen, dFont);
			break;

		case A_OK:
		case E_QUIT:
			VID_RenderTTF2Srf(bcre.B.x1[i], bcre.B.y1[i],
					          "Searching... (write your own address here)",
					          &MDText, vscreen, dFont);
			break;

		default:
			VID_RenderTTF2SrfSolid(bcre.B.x1[i], bcre.B.y1[i],
					               NULL, &MLText, vscreen, dFont, "%s",
					               ERROR_STRINGS[err]);
			break;
		}
	}
	/*
		Draw other buttons as highlighted,
		if they are selected.
	*/
	else if (bcre.B.pushed[i])
	{
		VID_RenderTTF2Srf(bcre.B.x1[i], bcre.B.y1[i],
				          bcre.B.name[i], &MLText,
						  vscreen, MFont, true, &MDText);
	}
	/* Normal draw. */
	else
	{
		r.w = bcre.B.w[i];
		r.h = bcre.B.h[i];
		r.x = bcre.B.x1[i];
		r.y = bcre.B.y1[i];
		SDL_BlitSurface(BG.srf, &r, vscreen, &r);
		VID_RenderTTF2Srf(bcre.B.x1[i], bcre.B.y1[i],
				          bcre.B.name[i], &MDText,
						  vscreen, MFont);
	}
}

//**********************************************************
/**
	Handle clicks in join-menu.
*/
EVALUE APP::join_check_clicked(int button, NET_HOST *hosts)
{
	/* Variables. */
	int i;
	EVALUE err = A_OK;
	NET_SETTINGS nst;

	/*
		Check, that the item was selected
		with right way. Not example by clicking
		it with right mouse button.
	*/
	if (bcre.state[button] > 1 && bcre.state[button] != 8)
	{
		bcre.state[button] = 0;
		return (err);
	}

	/* Set the struct to defaults. */
	nst.maxconnections = MAX_PLAYERS;
	nst.maxincomingpacketsinqueue = 0;
	nst.maxrecvpacketsinqueue = 0;
	nst.maxsendpacketsinqueue = 0;
	nst.maxchannels = 1;
	nst.maxperchannel = MAX_PLAYERS;
	nst.maxechofailures = 0;
	nst.maxpacketsendfailures = 0;
	nst.failuresforproblem = 0;
	nst.migrate_host = true;

	/* Check, which button has been pushed. */
	switch (button)
	{
	/* Exit the menu. */
	case bCcancel:
		NET_Close(true);
		Mode = MNICK;
		break;

	/* Create a session. */
	case bCcreate:
		/*
			Close enumeration. This close must be
			blocking, becose otherwise NET_Open()
			will try to use same resources in other
			thread, and application will crash very
			probably.
		*/
		NET_Close(true);

		/* Create game. */
		NET_Open(st.Nicks[st.current_nick], NET_PORT, &nst);
		NET_SetTimeoutType(NET_ERROR_DROP);

		/* Check for errors. */
		while ((err = NET_GetState(NET_STATEOF_CONNECTION))
			   == NET_STATE_CONNECTING);

		/* Check errors. */
		/* If not error. */
		if (err == NET_STATE_HOST)
		{
			err = A_OK;
			input[0] = '\0';
			g.init();
			Mode = MGAME;
		}
		/* If error. */
		else
		{
			err = E_FAILED;
		}
		
		break;

	/* User wants to connect to their own address. */
	case bCstate:
		strcpy(input, "<write address here>");
		string_get = true;
		string_selstart = 0;
		string_selend = strlen(input);
		break;

	/* Connect to selected host, if such. */
	default:
	{
		bool go_to_join = false;

		/*
			If selected session found,
			try to join it.
		*/
		for (i = bCN; i < (bCN + MAX_GAMES); i++)
		{
			if (bcre.state[i] == 1)
			{
				if (strlen(hosts[i - bCN].name) > 0)
				{
					selected_item = i - bCN;
				}
			}
			else if (bcre.state[i] == 8)
			{
				if (strlen(hosts[i - bCN].name) > 0)
				{
					selected_item = i - bCN;
					go_to_join = true;
				}
			}
		}
		
		/* If session not found, break out. */
		if (!go_to_join)
		{
			break;
		}
	}

	/* Join. */
	case bCjoin:
		if (string_get && strlen(input) > 0)
		{
			string_get = false;
			
			/*
				Close enumeration. This close must be
				blocking, becose otherwise NET_Open()
				will try to use same resources in other
				thread, and application will crash very
				probably.
			*/
			NET_Close(true);

			/* Join game. */
#ifdef _DEBUG
			err = NET_OpenAtAddress(st.Nicks[st.current_nick],
					                input, NET_PORT, 0, &nst);
#else
			err = NET_OpenAtAddress(st.Nicks[st.current_nick],
					                input, NET_PORT, NET_PORT, &nst);
#endif

			/* Set timeout type. */
			NET_SetTimeoutType(NET_ERROR_DROP);

			/* Check errors. */
			/* If errors. */
			if (err != A_OK)
			{
				err = E_NETTIMEOUT;
			}
			/* If no errors. */
			else
			{
				/* Wait for join to be concluded. */
				while ((err = NET_GetState(NET_STATEOF_CONNECTION))
					   == NET_STATE_CONNECTING);

				/* Check that were really are connected. */
				if (err == NET_STATE_CONNECTED)
				{
					err = A_OK;
					input[0] = '\0';
					g.init();
					Mode = MGAME;
				}
				/* If not connected, then error. */
				else if (err == NET_STATE_CLOSED)
				{
					err = E_NETCLOSED;
				}
				else
				{
					err = E_NETTIMEOUT;
				}
			}
		}
		else if (hosts[selected_item].ip > 0)
		{
			/* Close enumeration. */
			NET_Close(true);

			/* Join game. */
			NET_Open(st.Nicks[st.current_nick],
					 &hosts[selected_item],
					 NET_PORT, &nst);

			/* Set timeout type. */
			NET_SetTimeoutType(NET_ERROR_DROP);

			/* Wait for join to be concluded. */
			while ((err = NET_GetState(NET_STATEOF_CONNECTION))
				   == NET_STATE_CONNECTING);

			/* Check that were really are connected. */
			if (err == NET_STATE_CONNECTED)
			{
				err = A_OK;
				input[0] = '\0';
				g.init();
				Mode = MGAME;
				reload = true;
			}
			/* If not connected, then error. */
			else
			{
				err = E_NETTIMEOUT;
			}
		}

		break;
	}

	/* Clear button state. */
	bcre.state[button] = 0;

	/* Return error-value in err-variable. */
	return (err);
}

//**********************************************************
/**
	Handle keypresses in join-menu.
*/
void APP::join_keys(SDL_keysym *keysym, bool ud)
{
	/* Handle keypresses. */
	if (ud)
	{
//		switch (keysym->sym)
		{
		}
	}
	/* Handle key releases. */
	else
	{
		switch (keysym->sym)
		{
		case SDLK_ESCAPE:
			if (string_get)
			{
				string_get = false;
			}
			else
			{
				bcre.state[bCcancel] = -1;
			}
			
			break;

		case SDLK_RETURN:
			bcre.state[bCjoin] = -1;
			break;
		}
	}

	/* Return. */
	return;
}

