/*	
	Blokus SDL
		blmain.cpp

	This file includes functions main() and WinMain().
	
	Programming by Antti Partanen (alias Duge, duge@mbnet.fi)
	Copyright (C) 2003-2004 Antti Partanen


	This file is part of Blokus SDL.

	Blokus SDL is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	Blokus SDL is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with Blokus SDL; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#define ERROR_STRINGS_INUSE

//**********************************************************
//* INCLUDES
#ifdef _WIN32
#include <windows.h>
#endif

#include "derrors1.h"
#include "txtfile.h"

#include "blapp.h"
#include "blversion.h"
#include "blfiles.h"
#include "blexceptions.h"

//**********************************************************
//* FUNCTIONS

//**********************************************************
/**
	This is the main().
*/
#ifdef _WIN32
/* This is the main() for Windows. */
INT APIENTRY WinMain(HINSTANCE, HINSTANCE, LPSTR, int)
#else
/* This is the main() for other than windows (unix, linux..). */
int main(void)
#endif
{
	/*
		If in windows and not debugging, then
		check that this application isn't
		already running.
	*/
#if defined(WIN32) && !defined(_DEBUG)
	void *hWnd = FindWindow(NULL, "Blokus SDL"); 
	
	/* If application already running, exit. */
	if (hWnd)
	{
		ShowWindow((HWND)hWnd, SW_SHOWNORMAL);
		/*
			TODO:
			Add somekinda PostMessage() or something
			here, which makes the found application
			to redraw itself. Becose if in fullscreen,
			this only popups the application already
			running and leaves the screen blank.
		*/
		return (0);
	}
#endif

	/* APP class, this class runs everything. */
	APP app;

	/* Open error log. */
	FILE_Open(ERROR_LOG_FILE);

	/* Set LOG-file message level. */
#ifdef DEBUG
	FILE_Level(TXTFILE_EXTRADEBUG);
#else
	FILE_Level(TXTFILE_INFO);
#endif

	/* First print version of the application. */
	FILE_TextI("INFO: %s, version %s.\n", ID_STRING, VERSION_STRING);

	/*
		If version less than 1.00, then set level
		to EXTRADEBUG and print info to LOG. */
#if (VERSION_ID < 0x0100)
	FILE_Level(TXTFILE_EXTRADEBUG);
	FILE_TextD("BETA: Becose version is still beta (%s),\n"
		       "BETA: setting DEBUG-messages on.\n",
		       VERSION_STRING);
#endif

	/* Print platform information. (atleast a simple one) */
	FILE_TextI("INFO: Application starting.\n"
		       "INFO: This binary is compiled for %s.\n",
			   PLATFORM);

	/*
		Dont try to catch exceptions in when debugging.
		Atleast with Visual Studio, I want them to be
		catched by the debugger, not my application.
	*/
#ifdef DEBUG
	/* Run main application. */
	app.main();

	/* Not debugging. Catch exceptions. */
#else
	try
	{
		/* Run main application. */
		app.main();
	}
	catch (OutOfMemory &oom)
	{
		FILE_Text(TXTFILE_INFO, true, false, true,
			      "ERROR: Out of memory exception:"
				  " \"%s\" (on line %d in file %s)\n",
			      oom.str, oom.line, oom.file);
	}
	catch (Exception &exp)
	{
		FILE_Text(TXTFILE_INFO, true, false, true,
			"ERROR: Exception caught: \"%s\" (on line %d in file %s)\n",
			exp.str, exp.line, exp.file);
	}
	catch (...)
	{
		FILE_Text(TXTFILE_INFO, true, false, true,
			"ERROR: Unknown exception caught.\n");
	}
#endif

	/* Print exit info to LOG. */
	FILE_TextI("INFO: Application closing.\n");

	/* Close LOG file. */
	FILE_Close();

	/* Exit application. */
    return (0);
}

