/*
	Blokus SDL
		blmenu.cpp

	This file includes function Menu() for class APP

	Copyright (c) 2003-2004 Antti Partanen


	This file is part of Blokus SDL.

	Blokus SDL is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	Blokus SDL is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with Blokus SDL; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

//**********************************************************
//* INCLUDES
#include "blapp.h"
#include "blmenu.h"
#include "blvideo.h"

//**********************************************************
//* FUNCTIONS

//**********************************************************
/**
	Run the main menu.
*/
void APP::menu(int button)
{
	/*
		Check, if there is some button which has
		been selected.
	*/
	while (button > -1)
	{
		/* If selected in wrong way, then bail out. */
		if (bmen.state[button] > 1)
		{
			bmen.state[button] = 0;
			break;
		}

		/* Let's see, what should be done. */
		switch (button)
		{
		/* Exit whole program. */
		case bMexit:
			Mode = MEXIT;
			rMode = MEXIT;
			break;

		/* Go to settings menu. */
		case bMsettings:
			Mode = MSETTINGS;
			rMode = MMENU;
			break;

		/* Go to select nick and play. */
		case bMplay:
			Mode = MNICK;
			rMode = MMENU;
			break;

		/* Faulty selection. */
		default:
			break;
		}
		
		/* Clear button state. */
		bmen.state[button] = 0;

		break;
	}

	/* Draw buttons. */
	for (int i = 0; i < bMN; i++)
	{
		/* Define struct and set it. */
		SDL_Rect r;
		
		r.w = bmen.B.w[i];
		r.h = bmen.B.h[i];
		r.x = bmen.B.x1[i];
		r.y = bmen.B.y1[i];

		/* Reset background. */
		SDL_BlitSurface(BG.srf, &r, vscreen, &r);

		/*
			See if the button should be drawn as
			highlighted or normal.
		*/
		if (i == selected_item) /* Highlighted. */
		{
			VID_RenderTTF2Srf(bmen.B.x1[i], bmen.B.y1[i],
				              bmen.B.name[i], &MLText,
							  vscreen, MFont, false, &MDText);
		}
		else /* Normal. */
		{
			VID_RenderTTF2Srf(bmen.B.x1[i], bmen.B.y1[i],
							  bmen.B.name[i], &MDText,
							  vscreen, MFont);
		}
	}

	/* Return. */
	return;
}

//**********************************************************
/**
	Handle keypresses in main menu.
*/
void APP::menu_keys(SDL_keysym *keysym, bool ud)
{
	/* Handle keypresses. */
	if (ud)
	{
		/* Which key has been pressed. */
		switch (keysym->sym)
		{
		/* Select one item down. */
		case SDLK_DOWN:
			if (selected_item < 0)
			{
				selected_item = 0;
			}
			else
			{
				selected_item++;
			}

			if (selected_item >= bMN)
			{
				selected_item = 0;
			}

			break;
		
		/* Select one item up. */
		case SDLK_UP:
			selected_item--;

			if (selected_item < 0)
			{
				selected_item = bMN - 1;
			}

			break;
		}
	}
	/* Handle key releases. */
	else
	{
		/* Which key has been released. */
		switch (keysym->sym)
		{
		/* Exit the app. */
		case SDLK_ESCAPE:
			Mode = MEXIT;
			break;

		/* Click the selected item. */
		case SDLK_RETURN:
			if (selected_item > -1)
			{
				bmen.state[selected_item] = -1;
			}

			break;
		}
	}

	/* Return. */
	return;
}

