/*
	Blokus SDL
		blmenus.cpp

	This file includes function MenusLoop() for class APP

	Copyright (c) 2003-2004 Antti Partanen


	This file is part of Blokus SDL.

	Blokus SDL is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	Blokus SDL is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with Blokus SDL; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#define ERROR_STRINGS_INUSE

//**********************************************************
//* INCLUDES
#include "derrors.h"
#include "dmf.h"
#include "txtfile.h"

#include "blapp.h"
#include "blsettings.h"
#include "bljoin.h"
#include "blnick.h"
#include "blvideo.h"
#include "blmisc.h"
#include "blfiles.h"
#include "blokus.h"

//**********************************************************
//* FUNCTIONS

//**********************************************************
/**
	This function runs all menu screens.
*/
EVALUE APP::menus_loop(void)
{
	/* Variables. */
	int i;
	bool block;
	BUTTON *bb;
	EVALUE err;

	/* Some DEBUG info. */
	FILE_TextD("DEBUG: MenusLoop() starting (mode: %d)...\n", Mode);

	/*
		These brackets are just for
		clearing up the code.
		TODO: Create new method for this.
	*/
	{
		int bck;

		/* Somethings which are done by mode. */
		switch (Mode)
		{
		default:
		case MMENU:
			bck = BCKMENU;
			block = true;
			break;

		case MSETTINGS:
			bck = BCKSETTINGS;
			block = true;
			break;
		
		case MNICK:
			bck = BCKNICK;
			block = true;
			break;

		case MJOIN:
			bck = BCKJOIN;
			block = false;
			break;

		case MGAME:
			switch (game_st.tile_set)
			{
			default:
			case 0:
				bck = TSROCK;
				break;
			case 1:
				bck = TSWOOD;
				break;
			}
			block = false;
			break;
		}

		/* Destruct old background. */
		BG.~DPIC();

		/* Some DEBUG info. */
		FILE_TextD("DEBUG: Loading image from DMF...\n");
		
		/* Load new background from DMF. */
		err = LoadDMF(&BG, bck, DMF_A_NAME);

		/* If failed to load new background. */
		if (err != A_OK)
		{
			FILE_TextI("ERROR: Failed to load texture: %lu,"
				" error code: \"%s\" (%d)\n",
				bck, ERROR_STRINGS[err], err);
			Mode = MEXIT;
			return (err);
		}
		/* If not failed to load new background. */
		else
		{
			/* Some DEBUG info. */
			FILE_TextD("DEBUG: Image loading succesful.\n");
		}

		/* Somethings done by mode. */
		switch (Mode)
		{
		case MMENU:
			bb = &bmen;
			break;

		case MSETTINGS:
			settings_create_list();
			bb = &bset;
			break;

		case MNICK:
			bb = &bnic;
			break;

		case MJOIN:
			bb = &bcre;
			break;

		case MGAME:
			bb = &bgam;
			break;

		case MEXIT:
		default:
			FILE_TextI("ERROR: Unknown mode?!?!\n");
			run = false;
			return (A_OK);
		}

		/* Clear background. */
		SDL_BlitSurface(BG.srf, NULL, vscreen, NULL);
		SDL_Flip(vscreen);
		SDL_BlitSurface(BG.srf, NULL, vscreen, NULL);
	} /* End of clearing. */
	
	/* If first time in this menu or redraw wanted. */
	if (reload)
	{
		/* Some DEBUG info. */
		FILE_TextD("DEBUG: Reload some things.\n");
		
		/* Reset some variables. */
		string_get = false;
		string_cursor = 0;
		string_max_len = 0;
		string_selstart = 0;
		string_selend = 0;
		input_w = 0;
		input_h = 0;
		input_x = 0;
		input_y = 0;
		input_where = 0;
		input_font = NULL;
		input_color = White;
		selected_item = -1;
		tool_tip = -1;
		tool_tip_x = 0;
		tool_tip_y = 0;
		key_get = -1;
		key = SDLK_UNKNOWN;

		/* Some DEBUG info. */
		FILE_TextD("DEBUG: LoadGraphByMode()...\n");

		/*
			Load graphics by mode. This must be
			before ResetByMode().
			(in mode MLOBBY fChat-font is needed
			 by ResetByMode() and it's loaded in here)
		*/
		err = graph_load_by_mode(Mode);
		
		/* If error, then bail out. */
		if (err != A_OK)
		{
			FILE_TextI("ERROR: LoadGraphByMode() failed.\n");
			Mode = MEXIT;
			return (err);
		}

		/* Some DEBUG info. */
		FILE_TextD("DEBUG: ResetByMode()...\n");

		/* Reset variables by mode. */
		err = reset_by_mode(Mode);

		/* If error, then bail out. */
		if (err != A_OK)
		{
			FILE_TextI("ERROR: ResetByMode() failed.\n");
			Mode = MEXIT;
			return (err);
		}

		/* If buttons not null, reset them. */
		if (bb)
		{
			for (i = 0 ; i < bb->B.current; i++)
			{
				bb->state[i] = 0;
			}
		}
	}

	/* Clear some variables to defaults. */
	reload = true;
	
	/* Set mouse position. */
	int mbx[2] = { mouse_x, mouse_x };
	int mby[2] = { mouse_y, mouse_y };
	
	/* What was this for? */
	int sw = 0;

	/* Rect area mostly for recovering background. */
	SDL_Rect r;

	/* Current mode. */
	int mode = Mode;

	/*
		Counter for drawing everything twice in
		blocking events loops.
	*/
	int twice = 1;

	/* Set sleeping ticks. */
	ticks = SDL_GetTicks() + tpf;

	/* Print DEBUG message. */
	FILE_TextD("DEBUG: MenusLoop() started into mode: %d\n", Mode);

	/* Now start normal looping. */
	while (mode == Mode)
	{
		/* Draw mouse. */
		mouse_refresh();
		
		/* Recover background. */
		if (BG.srf)
		{
			r.x = mbx[sw];
			r.y = mby[sw];
			r.w = 20;
			r.h = 20;

			SDL_BlitSurface(BG.srf, &r, vscreen, &r);

			mbx[sw] = mouse_x;
			mby[sw] = mouse_y;
				
			sw = (sw + 1) & 1;
		}

		/* Recover tooltip background. */
		if (tool_tip_srf)
		{
			r.x = tool_tip_x;
			r.y = tool_tip_y;
			r.w = tool_tip_srf->w;
			r.h = tool_tip_srf->h;

			SDL_BlitSurface(BG.srf, &r, vscreen, &r);

			if (tool_tip < 0)
			{
				SDL_FreeSurface(tool_tip_srf);
				tool_tip_srf = NULL;
			}
		}

		/*
			Find out, if some button
			has been pressed. This
			value is passed to menu
			functions.
		*/
		int button = check_pushed(bb);

		/*
			Run a function, that is
			separate for every mode.
		*/
		switch (Mode)
		{
		case MMENU:
			menu(button);
			break;

		case MSETTINGS:
			settings(button);
			break;

		case MNICK:
			nick(button);
			break;

		case MJOIN:
			join(button);
			break;

		case MGAME:
			game(button);
			break;

		case MEXIT:
		default:
			run = false;
			break;
		}

		/* Handle typed strings. */
		get_string();

		/* Draw tooltip, if such. */
		if (tool_tip > -1)
		{
			r.x = tool_tip_x;
			r.y = tool_tip_y;

			SDL_BlitSurface(tool_tip_srf, NULL, vscreen, &r);
		}

		/* Show winamp messages if enabled. */
#ifdef USE_WINAMP
		if (winamp_hwnd && winamp_msg_delay > 0)
		{
			r.x = WINAMP_MSG_X;
			r.y = WINAMP_MSG_Y;
			r.w = vscreen->w - WINAMP_MSG_X;
			r.h = 20;

			SDL_BlitSurface(BG.srf, &r, vscreen, &r);

			switch (winamp_msg)
			{
			case 40044:
				VID_RenderTTF2Srf(WINAMP_MSG_X, WINAMP_MSG_Y,
					"Winamp: PREVIOUS", &White, vscreen, iFont);
				break;
			case 40045:
				VID_RenderTTF2Srf(WINAMP_MSG_X, WINAMP_MSG_Y,
					"Winamp: PLAY", &White, vscreen, iFont);
				break;
			case 40046:
				VID_RenderTTF2Srf(WINAMP_MSG_X, WINAMP_MSG_Y,
					"Winamp: PAUSE", &White, vscreen, iFont);
				break;
			case 40047:
				VID_RenderTTF2Srf(WINAMP_MSG_X, WINAMP_MSG_Y,
					"Winamp: STOP", &White, vscreen, iFont);
				break;
			case 40048:
				VID_RenderTTF2Srf(WINAMP_MSG_X, WINAMP_MSG_Y,
					"Winamp: NEXT", &White, vscreen, iFont);
				break;
			case 40058:
				VID_RenderTTF2Srf(WINAMP_MSG_X, WINAMP_MSG_Y,
					"Winamp: VOL UP", &White, vscreen, iFont);
				break;
			case 40059:
				VID_RenderTTF2Srf(WINAMP_MSG_X, WINAMP_MSG_Y,
					"Winamp: VOL DOWN", &White, vscreen, iFont);
				break;
			case 40060:
				VID_RenderTTF2Srf(WINAMP_MSG_X, WINAMP_MSG_Y,
					winamp_title, &White, vscreen, iFont);
				break;
			}

			if (winamp_msg_delay == 1)
			{
				SDL_BlitSurface(BG.srf, &r, vscreen, &r);
				winamp_msg_delay = 0;
			}
			else if (winamp_msg_delay < SDL_GetTicks())
			{
				SDL_BlitSurface(BG.srf, &r, vscreen, &r);
				winamp_msg_delay = 1;
			}
		}
#endif

		/* Draw mouse cursor. */
		if (mouse_on && BG.srf)
		{
			r.x = mouse_x;
			r.y = mouse_y;

			SDL_BlitSurface(Cursor.srf, NULL, vscreen, &r);
		}

		/*
			Draw version ID to screen. Normaly it
			is drawn only in main menu, but if
			version is under 1.00, then it is draw
			always.
		*/
#if (VERSION_ID >= 0x0100)
		if (Mode == MMENU)
#endif
		{
			r.x = vscreen->w - (Version->w + 8);
			r.y = vscreen->h - (Version->h + 8);

			SDL_BlitSurface(Version, NULL, vscreen, &r);
		}

		/* Refresh screen from double buffer. */
		SDL_Flip(vscreen);

		/*
			Process incoming events. This if .. else
			runs the while() loop twice on every
			incoming event, becose screen updates
			sometimes need more than one looping.
			Otherwise the ProcessEvents() blocks
			the while() loop until some event happens.
		*/
		if (block)
		{
			if (twice > 0)
			{
				/* Don't block, yet... */
				process_events(false);
				twice--;
			}
			else
			{
				/*
					Now, wait for incoming
					events maybe for ever.
				*/
				process_events(true);

				/*
					Something came up. Set
					counter for draw everything
					twice and go find out what
					happened.
				*/
				twice = 1;
			}
		}
		/* Do things in not blocking way. */
		else
		{
			/*
				Sleep a while, so all
				idle wont be reserved.
			*/
			while (SDL_GetTicks() < ticks)
			{
				unsigned int t = SDL_GetTicks() + 2;

				if (ticks > t)
				{
					SDL_Delay(ticks - t);
				}
			}
			
			/*
				Process incoming events
				from the system in non
				blocking way.
			*/
			process_events(false);
		}

		/*
			Calculate ticks for estimating the time
			for sleeping in non-blocking loop.
		*/
		ticks += tpf;
	}

	/* If only redraw is needed. */
	if (Mode == MREDRAW)
	{
		FILE_TextD("DEBUG: Mode = REDRAW, mode: %d\n", mode);
		Mode = mode;
		reload = false;
	}
	else
	{
		/* Some modes may need release some things. */
		FILE_TextD("DEBUG: ReleaseByMode()...\n");
		err = release_by_mode(mode);
		if (err != A_OK)
		{
			/*
				This should not happen ever.
				(well, are failures ever wanted?)
			*/
			FILE_TextI("ERROR: ReleaseByMode() failed.\n");
			Mode = MEXIT;
			return (err);
		}

		FILE_TextD("DEBUG: Exited mode %d, going to mode %d\n", mode, Mode);
	}

	/* Return. */
	return (A_OK);
}

//**********************************************************
/**
	This function resets variables by mode.
*/
EVALUE APP::reset_by_mode(int mode)
{
	switch (mode)
	{
	case MMENU:
		//* Nothing to be done
		break;

	case MGAME:
	{
		/* Set settings. */
		g.set_settings(&game_st);

		string_max_len = MAX_INPUT;

		/* Set volume to zero. */
		SND_Volume(0);

		if (st.snd_enabled)
		{
			/* Start playing playlist. */
			play_list.play(DMF_MUSIC);
			
			/* Create little fade. */
			for (int i = 0; i <= st.volume; i++)
			{
				SND_Volume(i);

				SDL_Delay(5);
			}
		}

		break;
	}

	case MSETTINGS:
		//* Video settings
		bset.bt[bSbpp].value = st.bpp;
		bset.bt[bSfullscreen].value = st.fullscreen;

		//* Audio settings
		bset.bt[bSsen].value = st.snd_enabled;
		bset.bt[bSvol].value = st.volume;

		//* Game settings
		bset.bt[bSai].value = game_st.ai_level;
		bset.bt[bSno].value = game_st.n_opponents;
		bset.bt[bSts].value = game_st.tile_set;

		//* Keyboard settings
		bset.bt[bSkl].value = game_st.key.Left;
		bset.bt[bSkr].value = game_st.key.Right;
		bset.bt[bSkh].value = game_st.key.Hflip;
		bset.bt[bSkv].value = game_st.key.Vflip;

		break;

	case MNICK:
		if (strlen(st.Nicks[st.current_nick]) < 1)
		{
			st.current_nick = 0;
		}

		if (strlen(st.Nicks[0]) < 1)
		{
			st.current_nick = 0;
			input_x = bnic.B.x1[st.current_nick + bNN];
			input_y = bnic.B.y1[st.current_nick + bNN];
			string_get = true;
		}
		else
		{
			string_get = false;
		}
	
		memset(input, '\0', MAX_INPUT);
		string_max_len = MAX_NICKLEN;

		break;

	case MJOIN:
		string_max_len = 255 > MAX_INPUT ? MAX_INPUT : 255;

		break;
	}

	return (A_OK);
}

//**********************************************************
/**
	This function checks pushed buttons for menus
	and handles tooltips and so on...
*/
int APP::check_pushed(BUTTON *bts)
{
	/* Make tooltip. */
	while (tool_tip > -1)
	{
		if (tool_tip_srf)
		{
			break;
		}

		int ptool_tip = tool_tip << 2;

		if (!bts->infos)
		{
			tool_tip = -1;
			break;
		}

		if (!bts->infos[ptool_tip])
		{
			tool_tip = -1;
			break;
		}

		SDL_Surface *text[4] = 
		{
			NULL, NULL, NULL, NULL
		};

		SDL_Color *c1 = &LText;
		SDL_Color *c2 = &DText;

		int i, n;
		int w = 0;
		int h = 0;

		for (n = 0; n < 4; n++)
		{
			if (bts->infos[ptool_tip + n])
			{
				text[n] = VID_RenderTTF((char *)bts->infos[ptool_tip + n],
					c1, iFont, false, c2);

				if (!text[n])
				{
					break;
				}
			}
			else
			{
				break;
			}

			if (text[n]->w > w)
			{
				w = text[n]->w;
			}

			h += text[n]->h;
		}

		if (n < 1)
		{
			break;
		}

		SDL_PixelFormat *f = vscreen->format;

		tool_tip_srf = SDL_CreateRGBSurface(SDL_HWSURFACE,
			w, h, f->BitsPerPixel, f->Rmask,
			f->Gmask, f->Bmask, f->Amask);

		if (!tool_tip_srf)
		{
			for (i = 0; i < n; i++)
			{
				SDL_FreeSurface(text[i]);
			}

			break;
		}

		SDL_FillRect(tool_tip_srf, NULL,
			SDL_MapRGB(tool_tip_srf->format, c2->r, c2->g, c2->b));

		SDL_Rect r;
		r.x = 0;
		r.y = 0;
		r.w = 0;
		r.h = 0;

		for (i = 0; i < n; i++)
		{
			SDL_BlitSurface(text[i], NULL, tool_tip_srf, &r);
			
			r.y += text[i]->h;
			
			SDL_FreeSurface(text[i]);
		}

		if ((mouse_x + tool_tip_srf->w) > vscreen->w)
		{
			tool_tip_x = mouse_x - tool_tip_srf->w;
		}
		else
		{
			tool_tip_x = mouse_x;
		}

		if ((mouse_y - tool_tip_srf->h) < 0)
		{
			tool_tip_y = mouse_y;
		}
		else
		{
			tool_tip_y = mouse_y - tool_tip_srf->h;
		}

		break;
	}
	
	if (!bts)
	{
		return (-1);
	}

	/* Set up some pointers and variables. */
	BT *bt = bts->bt;
	int n = bts->B.current;

	/* Check buttons. */
	bts->B.check(mouse_x, mouse_y);

	int i;
	int ret = -1;

	for (i = 0; i < n; i++)
	{
		if (bts->state[i] != 0)
		{
			int x, y;
			SDL_GetRelativeMouseState(&x, &y);

			if (x != 0 || y != 0)
			{
				selected_item = i;
			}
			else if (selected_item >= 0 &&
				selected_item != i)
			{
				bmen.state[i] = 0;
			}
		}

		if (mouse_dbt)
		{
			if (tool_tip > -1)
			{
				tool_tip = -1;
			}
			else if (bts->B.pushed[i])
			{
				bts->state[i] = 8;
			}
			else
			{
				bts->state[i] = 0;
			}
		}
		else if (mouse_bt & SDL_BUTTON(SDL_BUTTON_LEFT))
		{
			if (tool_tip > -1)
			{
				tool_tip = -1;
			}
			else if (bts->B.pushed[i])
			{
				bts->state[i] = 1;
			}
			else
			{
				bts->state[i] = 0;
			}
		}
		else if (mouse_bt & SDL_BUTTON(SDL_BUTTON_RIGHT))
		{
			if (tool_tip > -1)
			{
				tool_tip = -1;
			}
			else if (bts->B.pushed[i])
			{
				bts->state[i] = 2;
			}
			else
			{
				bts->state[i] = 0;
			}
		}
		else if (mouse_bt & SDL_BUTTON(SDL_BUTTON_MIDDLE))
		{
			if (bts->B.pushed[i])
			{
				bts->state[i] = 3;
			}
			else
			{
				bts->state[i] = 0;
			}
		}
		else if (mouse_bt & SDL_BUTTON(SDL_BUTTON_WHEELUP))
		{
			if (bts->B.pushed[i])
			{
				bts->state[i] = 4;
			}
			else
			{
				bts->state[i] = 0;
			}
		}
		else if (mouse_bt & SDL_BUTTON(SDL_BUTTON_WHEELDOWN))
		{
			if (bts->B.pushed[i])
			{
				bts->state[i] = 5;
			}
			else
			{
				bts->state[i] = 0;
			}
		}
		else if (bts->state[i] == 1 || bts->state[i] == 8)
		{
			if (!bt[i].valstr[0])
			{
				ret = i;
			}
			else if (bt[i].valstr[0][0] == '$')
			{
				ret = i;
			}
			else if (bt[i].valstr[bt[i].value + 1])
			{
				bt[i].value++;
			}
			else
			{
				bt[i].value = 0;
			}
		}
		else if (bts->state[i] == 2)
		{
			if (bset.infos)
			{
				tool_tip = i;
			}

			ret = i;
/*			if (!bt[i].valstr[0])
			{
				ret = i;
			}
			else if (bt[i].value < 1)
			{
				int t;

				for (t = 0; t < 11; t++)
				{
					if (bt[i].valstr[t])
					{
						continue;
					}

					bt[i].value = t - 1;

					break;
				}
			}
			else
			{
				bt[i].value--;
			}*/
		}
		else if (bts->state[i] == 3)
		{
			ret = i;
		}
		else if (bts->state[i] == 4)
		{
			if (!bt[i].valstr[0])
			{
				ret = i;
			}
			else if (bt[i].valstr[0][0] == '$')
			{
				int start;
				int end;
				int step;

				sscanf(&bt[i].valstr[0][1], "%u,%u,%u", &start, &end, &step);

				if (end > bt[i].value)
				{
					bt[i].value += step;
				}
			}
			else if (bt[i].valstr[bt[i].value + 1])
			{
				bt[i].value++;
			}
			else
			{
				bt[i].value = 0;
			}
		}
		else if (bts->state[i] == 5)
		{
			if (!bt[i].valstr[0])
			{
				ret = i;
			}
			else if (bt[i].valstr[0][0] == '$')
			{
				int start;
				int end;
				int step;

				sscanf(&bt[i].valstr[0][1], "%u,%u,%u", &start, &end, &step);

				if (start < bt[i].value)
				{
					bt[i].value -= step;
				}
			}
			else if (bt[i].value < 1)
			{
				int t;

				for (t = 0; t < 10; t++)
				{
					if (bt[i].valstr[t])
					{
						continue;
					}

					bt[i].value = t - 1;

					break;
				}
			}
			else
			{
				bt[i].value--;
			}
		}
		else if (bts->state[i] == 7)
		{
			if (!bt[i].valstr[0])
			{
				ret = i;
			}
			else if (bt[i].valstr[0][0] == '$')
			{
				int start;
				int end;
				int step;

				sscanf(&bt[i].valstr[0][1], "%u,%u,%u", &start, &end, &step);

				if (start < bt[i].value)
				{
					bt[i].value -= step;
				}
			}
			else if (bt[i].value < 1)
			{
				int t;

				for (t = 0; t < 11; t++)
				{
					if (bt[i].valstr[t])
					{
						continue;
					}

					bt[i].value = t - 1;

					break;
				}
			}
			else
			{
				bt[i].value--;
			}
		}
		else if (bts->state[i] < 0)
		{
			ret = i;
		}

		if (ret >= 0)
		{
			break;
		}

		if (mouse_bt == 0)
		{
			if (bts->B.pushed[i])
			{
				bts->state[i] = 6;
			}
			else
			{
				bts->state[i] = 0;
			}
		}		
	}

	return (ret);
}

//**********************************************************
/**
	Handles input stream from keyboard when input
	is wanted as a string.
*/
void APP::get_string(void)
{
	if (!string_get)
	{
		input_where = 0;
		return;
	}

	SDL_Rect r;

	r.w = input_w;
	r.h = input_h;
	r.x = input_x;
	r.y = input_y;
	SDL_BlitSurface(BG.srf, &r, vscreen, &r);

	int j = 0;
	int w, h;
	char t;

	if (strlen(input) > 0)
	{
		t = input[string_cursor];
		input[string_cursor] = '\0';

		TTF_SizeText(input_font, input, &w, &h);

		input[string_cursor] = t;

		if (w < (input_w - 10 - 8))
		{
			input_where = 0;
		}

		if ((w - input_where) > (input_w - 10 - 8))
		{
			input_where += (w - input_where) - (input_w - 10 - 8);
		}

		j = w - input_where;

		if ((input_where - w) > 0 && input_where > 0)
		{
			input_where -= input_where - w;
			if (input_where < 0) input_where = 0;
			j = 0;
		}

		SDL_Surface *temp = TTF_RenderText_Solid(input_font, input, input_color);
		w = temp->w;

		r.w = w > (input_w - 10 - 8) ? (input_w - 10 - 8) : w;
		r.h = input_h;
		r.x = input_where;
		r.y = 0;

		if (string_selstart < string_selend)
		{
			SDL_Rect r2;

			t = input[string_selstart];
			input[string_selstart] = '\0';
			w = 0;
			int w2;

			TTF_SizeText(input_font, input, &w, &h);

			input[string_selstart] = t;

			t = input[string_selend];
			input[string_selend] = '\0';
			TTF_SizeText(input_font, input + string_selstart, &w2, &h);
			input[string_selend] = t;

			r2.x = input_x;

			if ((w2 + w) > (input_where + r.w))
			{
				r2.w = w2 - ((w2 + w) - (input_where + r.w));
			}
			else
			{
				r2.w = w2;
			}

			if (input_where < w)
			{
				r2.x += w - input_where;
			}
			else
			{
				r2.w -= input_where - w;
			}

			r2.h = input_h - 4;
			r2.y = input_y + 2;

			SDL_FillRect(vscreen, &r2, SDL_MapRGB(vscreen->format, 255, 255, 255));
		}

		SDL_Rect r2;
		
		r2.x = input_x;
		r2.y = input_y;

		SDL_BlitSurface(temp, &r, vscreen, &r2);

		SDL_FreeSurface(temp);
	}

	t = input[string_cursor];
	input[string_cursor] = '\0';
	TTF_SizeText(input_font, input, &w, &h);
	input[string_cursor] = t;

	VID_RenderTTF2Srf(input_x + j, input_y,
		"_", &White, vscreen, input_font);

	/* Return. */
	return;
}

