/*
	Blokus SDL
		blmisc.cpp

	This file includes different misc functions.

	Copyright (c) 2003-2004 Antti Partanen


	This file is part of Blokus SDL.

	Blokus SDL is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	Blokus SDL is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with Blokus SDL; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

//**********************************************************
//* INCLUDES
#include <stdio.h>
#include <string.h>

#include "dmf.h"
#include "txtfile.h"

#include "blmisc.h"
#include "bldef.h"

//**********************************************************
//* VARIABLES
THREAD Threads[N_OF_THREADS];
Sint32 Nthreads = 0;
int TLoadTexture(void *data);

//**********************************************************
//* FUNCTIONS

//**********************************************************
/**
	Saves screenshot to BMP from main video surface.
*/
void ScreenShot(void)
{
	char name[LEN_STRING];
	FILE *file;
	Uint16 d = 0;

	while (1)
	{
		if (d < 10)
		{
			sprintf((char *)&name, "screen000%d.bmp", d);
		}
		else if (d < 100)
		{
			sprintf((char *)&name, "screen00%d.bmp", d);
		}
		else if (d < 1000)
		{
			sprintf((char *)&name, "screen0%d.bmp", d);
		}
		else
		{
			sprintf((char *)&name, "screen%d.bmp", d);
		}

		file = fopen((char *)&name, "r");

		if (!file)
		{
			SDL_SaveBMP(SDL_GetVideoSurface(), name);
			break;
		}
		
		fclose(file);
		d++;

		if (d > 9999)
		{
			break;
		}
	}

	return;
}

//**********************************************************
/**
	Texture loader thread.
*/
int TLoadTexture(void *data)
{
	static DPIC srf;

	if (LoadDMF(&srf, ((TTEXTDATA*)data)->Tgid, ((TTEXTDATA*)data)->Tdtf) != A_OK)
	{
		FILE_TextI("ERROR: TLoadTexture(): Failed loading texture.\n");
		FILE_TextI("ERROR:  File ID-number in DTF-file (\"%s\"): %d.\n",
					((TTEXTDATA*)data)->Tdtf, ((TTEXTDATA*)data)->Tgid);
		
		Threads[((TTEXTDATA*)data)->Tid].Run = false;
		delete ((TTEXTDATA*)data);

		return (-1);
	}

	((TTEXTDATA*)data)->Tsrf->h = srf.h;
	((TTEXTDATA*)data)->Tsrf->w = srf.w;
	((TTEXTDATA*)data)->Tsrf->xn = srf.yn;
	((TTEXTDATA*)data)->Tsrf->yn = srf.yn;
	((TTEXTDATA*)data)->Tsrf->x = srf.x;
	((TTEXTDATA*)data)->Tsrf->y = srf.y;

	((TTEXTDATA*)data)->Tsrf->pic = srf.pic;
	((TTEXTDATA*)data)->Tsrf->srf = srf.srf;

	srf.srf = NULL;
	srf.pic = NULL;
	srf.~DPIC();

	Threads[((TTEXTDATA*)data)->Tid].Run = false;

	delete ((TTEXTDATA*)data);
//	Nthreads--;

	return (0);
}

//**********************************************************
/**
	Waits for given texture load thread to finish.
*/
void WaitLoadTexture(int id)
{
	SDL_WaitThread(Threads[id].Pid, NULL);
}

//**********************************************************
/**
	Creates new thread to load a texture.
*/
int LoadTexture(DPIC *srf, unsigned short Gid, char *dtf)
{
	int tid;

//	if (Nthreads >= N_OF_THREADS) return (-1);

	tid = 0;
	while (1)
	{
		if (Threads[tid].Run == false) break;
		tid++;
		if (tid >= N_OF_THREADS) return (-1);
	}

	TTEXTDATA *thdata = new TTEXTDATA;
	thdata->Tsrf = srf;
	thdata->Tdtf = dtf;
	thdata->Tgid = Gid;
	thdata->Tid = tid;

	Threads[tid].Run = true;
	Threads[tid].Pid = SDL_CreateThread(TLoadTexture, thdata);
	
	if (!Threads[tid].Pid)
	{
		Threads[tid].Run = false;
		return (-1);
	}

//	Nthreads++;

	return (tid);
}

//**********************************************************
/**
	Loads given image with given id from DMF.
*/
int LoadDMF(DPIC *srf, unsigned short id, char *dtf,
			bool setcolorkey, Uint8 alpha,
			SDL_PixelFormat *fmt)
{
	DMFINFO dtfinfo;

#ifdef DEBUG
	FILE_TextD("DEBUG: Entered LoadDMF()...\n");
#endif

/* These checks are not actually needed, used only in debug. */
#ifdef DEBUG
	DMFHEAD dtfhead;
	if (DMF_ReadHead(&dtfhead, dtf) != 0)
	{
		FILE_TextV("ERROR: LoadDMF() failed to open DTF: %s\n", dtf);
		return (E_FILEFAIL);
	}
 
	if (DMF_ReadInfo(id, &dtfinfo, dtf) != 0)
	{
		FILE_TextV("ERROR: LoadDMF() failed to read info for image id %i\n", id);
		return (E_FAILED);
	}
#endif
	
	unsigned short int w;
	unsigned short int h;
	unsigned short int xn;
	unsigned short int yn;

	char *buffer = DMF_ImageRead(id, dtf, &w, &h, &xn, &yn);

	if (!buffer)
	{
		FILE_TextV("ERROR: LoadDMF() failed to load image: %s\n", dtfinfo.title);
		return (E_FAILED);
	}

	srf->w = w;
	srf->h = h;
	srf->xn = xn;
	srf->yn = yn;
	srf->x = w / xn;
	srf->y = h / yn;
	srf->srf = NULL;

	SDL_Surface *vscreen = SDL_GetVideoSurface();
	SDL_PixelFormat *f = fmt;
	
	if (vscreen && !f)
	{
		f = vscreen->format;
	}
	else
	{
		//* TODO: Handle error
	}
	
	if (alpha == SDL_ALPHA_OPAQUE)
	{
		srf->srf = SDL_CreateRGBSurface(SDL_HWSURFACE | SDL_SRCALPHA,
										srf->w, srf->h,
										f->BitsPerPixel,
										f->Rmask, f->Gmask,
										f->Bmask, f->Amask);
	}
	else
	{
		srf->srf = SDL_CreateRGBSurface(SDL_HWSURFACE | SDL_SRCALPHA,
										srf->w, srf->h,
										f->BitsPerPixel,
										f->Rmask, f->Gmask,
										f->Bmask, f->Amask);
	}

	if (srf->srf == NULL)
	{
		FILE_TextV("ERROR: LoadDMF() failed to create "
			"SDL_Surface from texture: %s", dtfinfo.title);
		DMF_Free(buffer);
		return (E_NULL);
	}

	if (srf->srf->format->BitsPerPixel == 8)
	{
		srf->pic = new unsigned char[srf->w * 4 * srf->h];
		
		if (!srf->pic)
		{
			FILE_TextV("ERROR: LoadDMF() failed to create temporary "
				       "buffer for texture: %s", dtfinfo.title);
			DMF_Free(buffer);
			return (E_NULL);
		}

		memcpy(srf->pic, buffer, srf->w * 4 * srf->h);
		DMF_Free(buffer);

		srf->srf = SDL_CreateRGBSurfaceFrom((void*)srf->pic, srf->w, srf->h,
			32, srf->w * 4, 0x000000ff, 0x0000ff00, 0x00ff0000, 0xff000000);

		if (!srf->srf)
		{
			FILE_TextV("ERROR: LoadDMF() failed to create temporary "
				       "SDL_Surface from texture: %s", dtfinfo.title);
			delete[] srf->pic;
			return (E_NULL);
		}
	}
	else
	{
		if (SDL_LockSurface(srf->srf) != 0)
		{
			FILE_TextV("ERROR: LoadDMF() failed to lock "
				       "SDL_Surface for texture: %s", dtfinfo.title);
			DMF_Free(buffer);
			return (E_FAILED);
		}

		int pntr = 0, vert = 0, pntr_pic = 0, vert_pic = 0, i = 0;

		for (i = 0; i < (srf->w * srf->h); i++)
		{
			switch (srf->srf->format->BitsPerPixel)
			{
			case 16:
				((unsigned short *)srf->srf->pixels)[pntr + vert] = SDL_MapRGBA(
					srf->srf->format,
					buffer[pntr_pic + vert_pic],
					buffer[pntr_pic + vert_pic + 1],
					buffer[pntr_pic + vert_pic + 2],
					buffer[pntr_pic + vert_pic + 3]);
				break;

			case 24:
				((unsigned char *)srf->srf->pixels)[pntr + vert] =
					buffer[pntr_pic + vert_pic + 2];
				((unsigned char *)srf->srf->pixels)[pntr + vert + 1] =
					buffer[pntr_pic + vert_pic + 1];
				((unsigned char *)srf->srf->pixels)[pntr + vert + 2] =
					buffer[pntr_pic + vert_pic];
				vert += 2;
				break;

			case 32:
				((unsigned int *)srf->srf->pixels)[pntr + vert] = SDL_MapRGBA(
					srf->srf->format,
					buffer[pntr_pic + vert_pic],
					buffer[pntr_pic + vert_pic + 1],
					buffer[pntr_pic + vert_pic + 2],
					buffer[pntr_pic + vert_pic + 3]);
				break;
			}

			vert++;
			vert_pic += 4;

			if (vert >= srf->w && srf->srf->format->BitsPerPixel != 24)
			{
				pntr_pic += (srf->w * 4);
				pntr += (srf->srf->pitch / srf->srf->format->BytesPerPixel);
				vert = 0;
				vert_pic = 0;
			}
			else if ((vert / 3) >= srf->w)
			{
				pntr_pic += (srf->w * 4);
				pntr += srf->srf->pitch;
				vert = 0;
				vert_pic = 0;
			}
		}

		SDL_UnlockSurface(srf->srf);

		DMF_Free(buffer);
		srf->pic = NULL;
	}

	/* Set Colorkey, if wanted. */
	if (setcolorkey == true)
	{
		SDL_SetColorKey(srf->srf,
						SDL_SRCCOLORKEY,
						SDL_MapRGB(srf->srf->format,
						TRANSPARENT_R,
						TRANSPARENT_G,
						TRANSPARENT_B));
	}

	/* Set alpha. */
	if (alpha == SDL_ALPHA_OPAQUE)
	{
		SDL_SetAlpha(srf->srf, 0, alpha);
	}
	else
	{
		SDL_SetAlpha(srf->srf, SDL_SRCALPHA, alpha);
	}

#ifdef DEBUG
	FILE_TextD("DEBUG: Exiting LoadDMF()...\n");
#endif

	return (A_OK);
}

