/*
	Blokus SDL
		blmybricks.cpp

	This file declares methods for class MY_BRICKS and
	class BRICK.

	Copyright (C) 2003-2004 Antti Partanen


	This file is part of Blokus SDL.

	Blokus SDL is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	Blokus SDL is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with Blokus SDL; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

//**********************************************************
//* INCLUDES
#include <new>
#include <cstring>

#include "dmf.h"
#include "txtfile.h"

#include "blmybricks.h"
#include "blexceptions.h"
#include "blfiles.h"
#include "blokus.h"

//**********************************************************
//* ARRAYS

/**
	This array defines how the bricks are placed in the
	strore-area at start.
*/
BRICK BRICKS_START_SET[21] =
{
	{ 0, 0, 0, 2, 1 },
	{ 1, 1, 2, 2, 2 },
	{ 2, 1, 1, 5, 3 },
	{ 3, 1, -1, 4, 4 },
	{ 4, 0, 0, -2, 5 },
	{ 5, 0, 5, 0, 4 },
	{ 6, 0, 4, 4, 5 },
	{ 7, 0, 13, 0, 5 },
	{ 8, 2, 8, 5, 5 },
	{ 9, 0, 7, 3, 4 },
	{ 10, 1, 9, 0, 5 },
	{ 11, 1, 12, 6, 4 },
	{ 12, 0, 13, 2, 5 },
	{ 13, 0, 17, 0, 5 },
	{ 14, 0, 17, 3, 4 },
	{ 15, 7, 17, 5, 5 },
	{ 16, 0, 20, 3, 5 },
	{ 17, 0, 21, 0, 3 },
	{ 18, 0, 24, 1, 5 },
	{ 19, 0, 23, 5, 5 },
	{ 20, 0, 26, 4, 5 },
};

//**********************************************************
//* CLASS METHODS

//**********************************************************
/*
	class MY_BRICKS.
*/

//**********************************************************
/**
	Constructor.
*/
MY_BRICKS::MY_BRICKS()
{
	/* Variables. */
	int i;
	char *pic;

	/* Set up bricks into table. */
	bricks = new BRICK[NUMBER_OF_BRICKS * NUMBER_OF_COLORS];
	
	/* Check allocation failures. */
	if (!bricks)
	{
		throw OutOfMemory("Failed to allocate memory for brick table.\n",
			              __LINE__, __FILE__);
	}

	/* Allocate memory for brick map. */
	/* Allocate rows. */
	map = new unsigned char *[BRICK_POSITIONS * ONE_BRICK_SIZE];

	/* Check allocation failures. */
	if (!map)
	{
		throw OutOfMemory("Failed to allocate memory for brick map.\n",
			              __LINE__, __FILE__);
	}

	/* Now allocate memory for every column. */
	for (i = 0; i < (BRICK_POSITIONS * ONE_BRICK_SIZE); i++)
	{
		/* Allocate memory. */
		map[i] = new unsigned char[NUMBER_OF_BRICKS * ONE_BRICK_SIZE];

		/* Check allocation failures. */
		if (!map[i])
		{
			throw OutOfMemory("Failed to allocate memory for brick map.\n",
							  __LINE__, __FILE__);
		}
	}

	/* Load the texture map for the bricks. */
	pic = DMF_ImageRead(BRICKMAP, DMF_A_NAME,
		                NULL, NULL, NULL, NULL);
	
	/* Check errors. */
	if (!pic)
	{
		throw Exception("ERROR: DMF_ImageRead() failed.\n",
			            __LINE__, __FILE__);
	}

	/* Copy and convert the image to map. */
	for (int lx = 0; lx < 40; lx++)
	{
		for (int ly = 0; ly < 105; ly++)
		{
			map[lx][ly] = (unsigned char)pic[(lx + ly * 40) * 4];
		}
	}

	/* Free picture. */
	DMF_Free(pic);

	/* Now use reset to clear tables. */
	reset();

	/* Exit constructor. */
	return;
}

//**********************************************************
/**
	Destructor.
*/
MY_BRICKS::~MY_BRICKS()
{
	/* Delete bricks. */
	delete[] bricks;

	/* Delete map. */
	for (int i = 0; i < (BRICK_POSITIONS * ONE_BRICK_SIZE); i++)
	{
		delete[] map[i];
	}

	delete[] map;
}

//**********************************************************
/**
	Reset tables to defaults.
*/
void MY_BRICKS::reset(void)
{
	/* Variables. */
	int i;
	BRICK *b_pointer;

	/* Copy given brick data to every table. */
	for (i = 0; i < NUMBER_OF_COLORS; i++)
	{
		/* Set pointer. */
		b_pointer = &bricks[i * NUMBER_OF_BRICKS];

		/* Move data. */
		memcpy(b_pointer,
			   BRICKS_START_SET,
			   sizeof(BRICK) * NUMBER_OF_BRICKS);
	}

	/* Return. */
	return;
}

//**********************************************************
/**
	Get ID-value of wanted brick.

	@return Positive value or -1 on error.
*/
int MY_BRICKS::get_id(int id, int color)
{
	/* Variables. */
	int ret = -1;
	int idz = id + color * NUMBER_OF_BRICKS;

	/* Check, that values are in range. */
	if (idz < 0 || idz > (NUMBER_OF_COLORS * NUMBER_OF_BRICKS))
	{
		return (ret);
	}

	/* Get the value. */
	ret = bricks[idz].id;

	/* Return. */
	return (ret);
}

//**********************************************************
/**
	Get x-coordinate of wanted brick.

	@return Positive value or -1 on error.
*/
int MY_BRICKS::get_x(int id, int color)
{
	/* Variables. */
	int ret = -1;
	int idz = id + color * NUMBER_OF_BRICKS;

	/* Check, that values are in range. */
	if (idz < 0 || idz > (NUMBER_OF_COLORS * NUMBER_OF_BRICKS))
	{
		return (ret);
	}

	/* Get the value. */
	ret = bricks[idz].x;

	/* Return. */
	return (ret);
}

//**********************************************************
/**
	Get y-coordinate of wanted brick.

	@return Positive value or -1 on error.
*/
int MY_BRICKS::get_y(int id, int color)
{
	/* Variables. */
	int ret = -1;
	int idz = id + color * NUMBER_OF_BRICKS;

	/* Check, that values are in range. */
	if (idz < 0 || idz > (NUMBER_OF_COLORS * NUMBER_OF_BRICKS))
	{
		return (ret);
	}

	/* Get the value. */
	ret = bricks[idz].y;

	/* Return. */
	return (ret);
}

//**********************************************************
/**
	Get position of wanted brick.

	@return Positive value or -1 on error.
*/
int MY_BRICKS::get_pos(int id, int color)
{
	/* Variables. */
	int ret = -1;
	int idz = id + color * NUMBER_OF_BRICKS;

	/* Check, that values are in range. */
	if (idz < 0 || idz > (NUMBER_OF_COLORS * NUMBER_OF_BRICKS))
	{
		return (ret);
	}

	/* Get the value. */
	ret = bricks[idz].pos;

	/* Return. */
	return (ret);
}

//**********************************************************
/**
	Get position of wanted brick.

	@return Positive value or -1 on error.
*/
int MY_BRICKS::get_points(int id, int color)
{
	/* Variables. */
	int ret = -1;
	int idz = id + color * NUMBER_OF_BRICKS;

	/* Check, that values are in range. */
	if (idz < 0 || idz > (NUMBER_OF_COLORS * NUMBER_OF_BRICKS))
	{
		return (ret);
	}

	/* Get the value. */
	ret = bricks[idz].points;

	/* Return. */
	return (ret);
}

//**********************************************************
/**
	Get position of wanted brick.
*/
unsigned char MY_BRICKS::get_brick_value(int x, int y, int pos, int id)
{
	/* Variables. */
	unsigned char ret = 0xff;
	int xz = x + pos * ONE_BRICK_SIZE;
	int yz = y + id * ONE_BRICK_SIZE;

	/* Check, that values are in range. */
	if (xz < 0 || xz > (ONE_BRICK_SIZE * BRICK_POSITIONS) ||
		yz < 0 || yz > (ONE_BRICK_SIZE * NUMBER_OF_BRICKS))
	{
		return (ret);
	}

	/* Get the value. */
	ret = map[x + pos * ONE_BRICK_SIZE][y + id * ONE_BRICK_SIZE];

	/* Return. */
	return (ret);
}

//**********************************************************
/**
	Get state of wanted brick.
*/
bool MY_BRICKS::get_brick(int id, int color)
{
	/* Variables. */
	bool ret = false;
	int idz = id + color * NUMBER_OF_BRICKS;

	/* Check, that values are in range. */
	if (idz < 0 || idz > (NUMBER_OF_COLORS * NUMBER_OF_BRICKS))
	{
		return (ret);
	}

	/* Get the value. */
	ret = bricks[idz].id < 0 ? false : true;

	/* Return. */
	return (ret);
}

//**********************************************************
/**
	Get x-coordinate of wanted brick.
*/
void MY_BRICKS::set_x(int value, int id, int color)
{
	/* Variables. */
	int idz = id + color * NUMBER_OF_BRICKS;

	/* Check, that values are in range. */
	if (idz < 0 || idz > (NUMBER_OF_COLORS * NUMBER_OF_BRICKS))
	{
		return;
	}

	/* Set value. */
	bricks[idz].x = value; 

	/* Return. */
	return;
}

//**********************************************************
/**
	Get x-coordinate of wanted brick.
*/
void MY_BRICKS::set_y(int value, int id, int color)
{
	/* Variables. */
	int idz = id + color * NUMBER_OF_BRICKS;

	/* Check, that values are in range. */
	if (idz < 0 || idz > (NUMBER_OF_COLORS * NUMBER_OF_BRICKS))
	{
		return;
	}

	/* Set value. */
	bricks[idz].y = value; 

	/* Return. */
	return;
}

//**********************************************************
/**
	Set position of wanted brick.
*/
void MY_BRICKS::set_pos(int value, int id, int color)
{
	/* Variables. */
	int idz = id + color * NUMBER_OF_BRICKS;

	/* Check, that values are in range. */
	if (idz < 0 || idz > (NUMBER_OF_COLORS * NUMBER_OF_BRICKS))
	{
		return;
	}

	/* Set value. */
	bricks[idz].pos = value; 

	/* Return. */
	return;
}

//**********************************************************
/**
	Set points of wanted brick.
*/
void MY_BRICKS::set_points(int value, int id, int color)
{
	/* Variables. */
	int idz = id + color * NUMBER_OF_BRICKS;

	/* Check, that values are in range. */
	if (idz < 0 || idz > (NUMBER_OF_COLORS * NUMBER_OF_BRICKS))
	{
		return;
	}

	/* Set value. */
	bricks[idz].points = value; 

	/* Return. */
	return;
}

//**********************************************************
/**
	Set state of wanted brick.
*/
void MY_BRICKS::disable_brick(int id, int color)
{
	/* Variables. */
	int idz = id + color * NUMBER_OF_BRICKS;

	/* Check, that values are in range. */
	if (idz < 0 || idz > (NUMBER_OF_COLORS * NUMBER_OF_BRICKS))
	{
		return;
	}

	/* Set value. */
	bricks[idz].id = -1; 

	/* Return. */
	return;
}

//**********************************************************
/**
	Count number of enabled bricks.
*/
int MY_BRICKS::n_of_bricks(int color)
{
	/* Variables. */
	int ret = 0;
	int i;
	int idz = color * NUMBER_OF_BRICKS;

	/* Check, that values are in range. */
	if (color >= NUMBER_OF_COLORS)
	{
		return (ret);
	}
	
	/* Check bricks. */
	for (i = 0; i < NUMBER_OF_BRICKS; i++)
	{
		ret += bricks[idz + i].id < 0 ? 0 : 1;
	}

	/* Return. */
	return (ret);
}

