/*
	Blokus SDL
		blmybricks.h

	This file defines class MY_BRICKS and class BRICK.

	Copyright (C) 2003-2004 Antti Partanen


	This file is part of Blokus SDL.

	Blokus SDL is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	Blokus SDL is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with Blokus SDL; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef BLMYBRICKS_H
#define BLMYBRICKS_H

//**********************************************************
//* CONSTANTS
const int NUMBER_OF_BRICKS = 21;
const int NUMBER_OF_COLORS = 4;
const int ONE_BRICK_SIZE = 5;
const int BRICK_POSITIONS = 8;

//**********************************************************
//* CLASS DEFINITIONS

//**********************************************************
/**
	Class for bricks in class MY_BRICKS.
*/
class BRICK
{
public:
	/* Public variables. */
	/** ID of brick. */
	int id;

	/** Position of brick. */
	int pos;

	/** x-coordinate. */
	int x;

	/** y-coordinate. */
	int y;

	/** Points. */
	int points;
};

//**********************************************************
/**
	Class MY_BRICKS.
*/
class MY_BRICKS
{
public:
	/* Contructor and destructor. */
	MY_BRICKS();
	~MY_BRICKS();
	
	/* Reset tables to defaults. */
	void reset(void);

	/* Get values. */
	int get_id(int id, int color);
	int get_x(int id, int color);
	int get_y(int id, int color);
	int get_pos(int id, int color);
	int get_points(int id, int color);
	
	/* Get value of wanted brick in wanted position. */
	unsigned char get_brick_value(int x, int y, int pos, int id);

	/* Get brick state. */
	bool get_brick(int id, int color);

	/* Set values. */
//	void set_id(int value, int id, int color);
	void set_x(int value, int id, int color);
	void set_y(int value, int id, int color);
	void set_pos(int value, int id, int color);
	void set_points(int value, int id, int color);
	
	/* Set value of wanted brick in wanted position. */
	void set_brick_value(int value, int x, int y, int pos, int id);
	
	/* Disable brick. */
	void disable_brick(int id, int color);

	/* Counts number of enabled bricks. */
	int n_of_bricks(int color);

private:
	/** Array of bricks. */
	BRICK *bricks;

	/** Map for table. */
	unsigned char **map;
};

#endif

