/*
	Blokus SDL
		blgnetwork.cpp

	This file includes networking functions.

	Copyright (C) 2003-2004 Antti Partanen


	This file is part of Blokus SDL.

	Blokus SDL is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	Blokus SDL is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with Blokus SDL; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

//**********************************************************
//* INCLUDES
#include "blgame.h"

//**********************************************************
//* METHODS

//**********************************************************
//* class GAME

//**********************************************************
/**
	Handles networking things.
*/
void GAME::handle_network(void)
{
	/* Variables. */
	NET_PACKET *p;
	
	/* Get packet. */
	p = NET_Recv();

	/* If no packet received, return. */
	if (!p)
	{
		return;
	}

	/* Packet, received. Handle it. */
	switch (handle_packet(p))
	{
	/* Start the game. */
	case 'g':
		if (end)
		{
			restart();
		}

		nofplayers = NET_GetNumOfConnections();
		game_running = true;
		current_color = COLOR_BLUE;

		box.add_line("***** Game started!");
		break;

	/* My colors. */
	case 'c':
		if (!NET_Host())
		{
			int id = *((unsigned int *)&p->data[2]);

			if (id == NET_GetMyID())
			{
				my_color = p->data[1];

				color_ids[my_color] = NET_GetMyID();
			}
			else
			{
				color_ids[(int)p->data[1]] = id;
			}
		}
		break;

	/* Next round. */
	case 'r':
		round++;
		break;

	/* Change of turn. */
	case 'n':
	{
		/* Play a sound. */
//		SND_Play(snd_next);

		if (p->data[1] == my_color)
		{
//			AddLine("***** It's your turn now...");
			my_turn = true;
		}
/*		else
		{
			NET_CONNECTION_PACKET *pp;
			pp = NET_GetConnectionInfo(color_ids[p->data[1]]);

			char string[256];

			sprintf(string, "***** %s turn...", pp->name);

			AddLine(string);

			NET_Free(pp);
		}*/

		current_color = p->data[1];

		break;
	}

	/* Set. */
	case 'p':
	{
		int color = p->data[1];
		BRICK *b = (BRICK *)&p->data[2];

		set_brick(b->x, b->y, color, b->pos, b->id);

		break;
	}

	/* Chat. */
	case '<':
		box.add_line(p->data);
		break;

	/* Color gives up. */
	case 'i':
		color_gaming[p->data[1]] = false;
		break;

	/* End of game. */
	case 'e':
		ends();
		break;

	/* Points changes. */
	case 's':
		points[p->data[1]] = p->data[2];
		break;

	default:
		break;
	}

	return;
}

//**********************************************************
/**
	Handles a network packet.
*/
char GAME::handle_packet(NET_PACKET *p)
{
	NET_CONNECTION_PACKET *pp = (NET_CONNECTION_PACKET *)p->data;
	char string[256];

	switch (p->type)
	{
	case NET_MSG_NEW:
		nofplayers = NET_GetNumOfConnections();
		
		if (end)
		{
			NET_Send((char *)board.get_area_pointer(),
				     BOARD_XSIZE * BOARD_YSIZE,
				     pp->id, NET_TYPE_DATA);
		}

		sprintf(string, "* %s joined the game...", pp->name);

		box.add_line(string);

		deal_colors();

		break;

	case NET_MSG_QUIT:
	case NET_MSG_DROP:
		nofplayers = NET_GetNumOfConnections();

		if (game_running)
		{
			game_running = false;
			end = true;
			NET_SetClosed(false);
		}
		else
		{
			deal_colors();
		}

		sprintf(string, "* %s left the game...", pp->name);

		box.add_line(string);

		break;

	case NET_MSG_STRING:
		return (p->data[0]);
		break;

	case NET_MSG_HOSTLEFT:
		break;

	case NET_MSG_PROBLEM:
		connection_error = true;
		break;

	case NET_MSG_TIMEOUT:
		connection_error = true;
		break;

	case NET_MSG_OK:
		connection_error = false;
		break;

	case NET_MSG_DATA:
		if (p->datasize == (BOARD_XSIZE * BOARD_YSIZE))
		{
			end = true;
			memcpy(board.get_area_pointer(), p->data,
				   BOARD_XSIZE * BOARD_YSIZE);
		}
		break;
	}

	return ('\0');
}

//**********************************************************
/**
	This function deals colors to players.
	Called every time when somebody joins or quits.
*/
void GAME::deal_colors()
{
	if (!NET_Host())
	{
		return;
	}

	NET_CONNECTION_PACKET *packet;
	int n;
	int xx = 0;

	packet = NET_GetConnections(&n);

	nofplayers = n;

	for (int i = 0; i < n; i++)
	{
		int t;
		char x[2 + sizeof(unsigned int)];
		x[0] = 'c';
		
		if (packet[i].id == NET_GetMyID())
		{
			t = COLOR_BLUE;
			color_ids[t] = NET_GetMyID();
		}
		else
		{
			switch (xx)
			{
			default:
			case 0:
				t = COLOR_GREEN;
				break;
			case 1:
				t = COLOR_RED;
				break;
			case 2:
				t = COLOR_YELLOW;
				break;
			}

			xx++;
	
			color_ids[t] = packet[i].id;
		}
		
		x[1] = (char)t;
		memcpy(&x[2], &color_ids[t], sizeof(unsigned int));

		NET_Send(x, 2 + sizeof(unsigned int), NET_ID_ALL, NET_TYPE_STRING);
	}

	NET_Free(packet);
}


