/*
	Blokus SDL
		blsettings.cpp

	This file includes function Settings() for class APP

	Copyright (c) 2003-2004 Antti Partanen


	This file is part of Blokus SDL.

	Blokus SDL is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	Blokus SDL is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with Blokus SDL; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

//**********************************************************
//* INCLUDES
#include "txtfile.h"

#include "blapp.h"
#include "blvideo.h"
#include "blsound.h"
#include "blst.h"
#include "blfiles.h"
#include "blexceptions.h"
#include "blokus.h"
#include "blsettings.h"
#include "bltables.h"

//**********************************************************
//* METHODS

//**********************************************************
/*
	class APP.
*/

//**********************************************************
/**
	Run the settings menu.
*/
void APP::settings(int value)
{
	int i;
	BT *psetbt = bset.bt;

	while (value > -1)
	{
		key_get = -1;

		if (bset.state[value] > 1 && bset.state[value] != 8)
		{
			for (i = 0; i < bSN; i++)
			{
				bset.state[i] = 0;
			}

			break;
		}
		
		switch (value)
		{
		case bSkl:
		case bSkr:
		case bSkh:
		case bSkv:
			key_get = value;
			break;

		case bSexit:
			//* Video settings
			if (st.bpp != psetbt[bSbpp].value ||
				st.fullscreen != psetbt[bSfullscreen].value)
			{
				st.bpp = psetbt[bSbpp].value;
				st.fullscreen = psetbt[bSfullscreen].value;
				change_video_mode();
			}

			//* Audio settings
			st.volume = psetbt[bSvol].value;
			st.snd_enabled = psetbt[bSsen].value ? true : false;
			SND_Volume(st.volume);

			//* Single mode settings
			game_st.ai_level = psetbt[bSai].value;
			game_st.n_opponents = psetbt[bSno].value;
			game_st.tile_set = psetbt[bSts].value;

			game_st.key.Left = psetbt[bSkl].value;
			game_st.key.Right = psetbt[bSkr].value;
			game_st.key.Hflip = psetbt[bSkh].value;
			game_st.key.Vflip = psetbt[bSkv].value;

			st.rseed = SDL_GetTicks();

			if (SaveSettings(SETTINGS_FILE,
				             (char *)&st, sizeof(SETTINGS),
							 (char *)&game_st, sizeof(GAME_SETTINGS))
							 != A_OK)
			{
				FILE_TextD("DEBUG: Failed to save settings!\n");
			}
			else
			{
				FILE_TextD("DEBUG: Settings saved.\n");
			}

			Mode = rMode;
			rMode = MEXIT;

			break;

		case bScancel:
			Mode = rMode;
			rMode = MEXIT;

			break;
		}

		for (i = 0; i < bSN; i++)
		{
			bset.state[i] = 0;
		}

		break;
	}

	if (key_get > -1)
	{
		if (key == SDLK_ESCAPE)
		{
			key = SDLK_UNKNOWN;
			key_get = -1;
			bset.state[bScancel] = 0;
		}
		else if (key != SDLK_UNKNOWN)
		{
			for (i = bSkl; i <= bSkv; i++)
			{
				if (psetbt[i].value == key)
				{
					psetbt[i].value = psetbt[key_get].value;
					break;
				}
			}

			psetbt[key_get].value = key;
			
			if (key != SDLK_RETURN)
			{
				key_get = -1;
			}

			key = SDLK_UNKNOWN;
		}
	}
	
	for (i = 0; i < bSN; i++)
	{
		SDL_Rect rect;

		rect.w = bset.B.w[i];
		rect.h = bset.B.h[i];
		rect.x = bset.B.x1[i];
		rect.y = bset.B.y1[i];
		SDL_BlitSurface(BG.srf, &rect, vscreen, &rect);

		char *str = NULL;

		if (psetbt[i].valstr[0])
		{
			if (psetbt[i].valstr[0][0] != '$')
			{
				str = psetbt[i].valstr[psetbt[i].value];
			}
			else
			{
				str = NULL;
			}
		}
		else
		{
			str = NULL;
		}

		SDL_Color *c1;

		if (i >= bSkl && i <= bSkh)
		{
			if (selected_item == i)
			{
				c1 = &MLText;
			}
			else
			{
				c1 = &MMText;
			}

			VID_RenderTTF2Srf(bset.B.x1[i],
				bset.B.y1[i], bset.B.name[i],
				c1, vscreen, dFont);

			str = SDL_GetKeyName((SDLKey)psetbt[i].value);

			if (str != NULL)
			{
				VID_RenderTTF2Srf(bset.B.x1[i] + 140, bset.B.y1[i],
					str, c1, vscreen, dFont);
			}
		}
		else if (i == bSvol)
		{
			if (selected_item == i && mouse_bt & SDL_BUTTON(SDL_BUTTON_LEFT))
			{
				int start;
				int end;
				int step;
				int size;
				int xoff;

				sscanf(&psetbt[bSvol].valstr[0][1],
					   "%i,%i,%i,%i,%i", &start,
					   &end, &step, &size, &xoff);
				
				int vol_value = mouse_x - psetbt[bSvol].x - xoff - 4;

				if (vol_value > start && vol_value < end)
				{
					psetbt[bSvol].value = vol_value;
					SND_Volume(vol_value);
				}
			}

			if (selected_item == i)
			{
				c1 = &MLText;
			}
			else
			{
				c1 = &MMText;
			}

			VID_RenderTTF2Srf(bset.B.x1[i],
				bset.B.y1[i], bset.B.name[i],
				c1, vscreen, dFont);

			int start;
			int end;
			int step;
			int size;

			sscanf(&psetbt[i].valstr[0][1],
				   "%u,%u,%u,%u", &start,
				   &end, &step, &size);

			int coord_value = psetbt[i].value * size / (end - start);
			
			VID_RenderTTF2Srf(bset.B.x1[i] + 145, bset.B.y1[i],
				"0", c1, vscreen, dFont);

			VID_RenderTTF2Srf(bset.B.x1[i] + 265, bset.B.y1[i],
				"100", c1, vscreen, dFont);

			VID_RenderTTF2Srf(bset.B.x1[i] + 155 + coord_value, bset.B.y1[i],
				"#", c1, vscreen, dFont);
		}
		else if (i == bSexit || i == bScancel)
		{
			if (selected_item == i)
			{
				c1 = &MLText;
			}
			else
			{
				c1 = &MDText;
			}

			VID_RenderTTF2Srf(bset.B.x1[i],
				bset.B.y1[i], bset.B.name[i],
				c1, vscreen, MFont);
		}
		else
		{
			if (selected_item == i)
			{
				c1 = &MLText;
			}
			else
			{
				c1 = &MMText;
			}

			VID_RenderTTF2Srf(bset.B.x1[i], bset.B.y1[i], bset.B.name[i],
				c1, vscreen, dFont);

			if (str != NULL)
			{
				VID_RenderTTF2Srf(bset.B.x1[i] + 140, bset.B.y1[i],
					str, c1, vscreen, dFont);
			}
		}
	}

	if (key_get > -1)
	{
		VID_RenderTTF2Srf(bset.B.x1[key_get],
			bset.B.y1[key_get], bset.B.name[key_get],
			&White, vscreen, dFont);

		char *str = SDL_GetKeyName((SDLKey)psetbt[key_get].value);

		if (str != NULL)
		{
			VID_RenderTTF2Srf(bset.B.x1[key_get] + 140, bset.B.y1[key_get],
				str, &White, vscreen, dFont);
		}
	}

	return;
}

//**********************************************************
/**
	Handle keypresses in join-menu.
*/
void APP::settings_keys(SDL_keysym *keysym, bool ud)
{
	/* Handle keypresses. */
	if (ud)
	{
		switch (keysym->sym)
		{
		case SDLK_DOWN:
			if (selected_item < 0)
			{
				selected_item = 0;
			}
			else
			{
				selected_item++;
			}

			if (selected_item >= bSN)
			{
				selected_item = 0;
			}

			break;

		case SDLK_UP:
			selected_item--;

			if (selected_item < 0)
			{
				selected_item = bSN - 1;
			}

			break;

		case SDLK_LEFT:
			if (selected_item < 0 || key_get > -1)
			{
				break;
			}
			
			bset.state[selected_item] = 7;
			
			break;

		case SDLK_RIGHT:
			if (selected_item < 0 || key_get > -1)
			{
				break;
			}
			
			bset.state[selected_item] = 1;

			break;
		}
	}
	/* Handle key releases. */
	else
	{
		switch (keysym->sym)
		{
		case SDLK_ESCAPE:
			if (tool_tip > -1)
			{
				tool_tip = -2;
			}
			else if (key_get > -1)
			{
				key_get = -1;
			}
			else
			{
				key = SDLK_ESCAPE;
				bset.state[bScancel] = -1;
			}

			break;

		case SDLK_RETURN:
			if (tool_tip > -1)
			{
				tool_tip = -2;
				break;
			}

			switch (selected_item)
			{
			case bSexit:
				bset.state[bSexit] = -1;
				break;

			case bScancel:
				bset.state[bScancel] = -1;
				break;

			case bSkl:
			case bSkr:
			case bSkh:
			case bSkv:
				if (key_get < 0)
				{
					key_get = selected_item;
				}
				else
				{
					key_get = -1;
				}

				break;
			}

			break;
			
		case SDLK_TAB:
			if (selected_item >= bSfullscreen &&
				selected_item <= bSbpp)
			{
				selected_item = (SDL_GetModState() & KMOD_SHIFT)
					? bSkl : bSvol;
			}
			else if (selected_item >= bSvol &&
				selected_item <= bSvol)
			{
				selected_item = (SDL_GetModState() & KMOD_SHIFT)
					? bSfullscreen : bSai;
			}
			else if (selected_item >= bSai &&
				selected_item <= bSts)
			{
				selected_item = (SDL_GetModState() & KMOD_SHIFT)
					? bSvol : bSexit;
			}
			else if (selected_item >= bSexit &&
				selected_item <= bScancel)
			{
				selected_item = (SDL_GetModState() & KMOD_SHIFT)
					? bSai : bSkl;
			}
			else
			{
				selected_item = (SDL_GetModState() & KMOD_SHIFT)
					? bSkl : bSfullscreen;
			}

			break;
		}
	}

	/* Return. */
	return;
}

//**********************************************************
/**
	Create settings-menu list background graphics.
*/
EVALUE APP::settings_create_list(void)
{
	/* Variables. */
	int x_add = foreMenuList.x;
	int y_add = foreMenuList.y;
	int x_number = BS_W / x_add - 1;
	int j;
	int i;

	/* Load font. */
	TTF_Font *font = load_ttf(FONT_H_NAME, FONT_H_SIZE, DMF_A_NAME);
	TTF_SetFontStyle(font, FONT_H_STYLE);

	/* Check errors. */
	if (!font)
	{
		FILE_TextI("ERROR: Couldn't load font (id: %u). "
			       "(line:%i@file:%s)\n",
				   FONT_H_NAME, __LINE__, __FILE__);

		throw Exception("Couldn't load font.\n", __LINE__, __FILE__);
	}

	/* Draw. */
	for (i = 0; i < bSN; i++)
	{
		if (bsetT[i].flags == BT_FLAG_NONE)
		{
			continue;
		}

		VID_Blit(bsetT[i].x + BS_XFIXLIST,
			     bsetT[i].y + BS_YFIXLIST,
			     3, &foreMenuList, BG.srf);

		for (j = 1; j < x_number; j++)
		{
			VID_Blit(bsetT[i].x + j * x_add + BS_XFIXLIST,
				     bsetT[i].y + BS_YFIXLIST,
				     4, &foreMenuList, BG.srf);
		}

		VID_Blit(bsetT[i].x + x_number * x_add + BS_XFIXLIST,
			     bsetT[i].y + BS_YFIXLIST,
			     5, &foreMenuList, BG.srf);

		if (bsetT[i].flags & BT_FLAG_START)
		{
			VID_Blit(bsetT[i].x + BS_XFIXLIST,
					 bsetT[i].y - y_add + BS_YFIXLIST,
					 0, &foreMenuList, BG.srf);

			for (j = 1; j < x_number; j++)
			{
				VID_Blit(bsetT[i].x + j * x_add + BS_XFIXLIST,
						 bsetT[i].y - y_add + BS_YFIXLIST,
						 1, &foreMenuList, BG.srf);
			}

			VID_Blit(bsetT[i].x + x_number * x_add + BS_XFIXLIST,
					 bsetT[i].y - y_add + BS_YFIXLIST,
					 2, &foreMenuList, BG.srf);

			VID_RenderTTF2Srf(bsetT[i].x + BS_XFIXLIST + BS_XFIXTITLE,
							  bsetT[i].y - y_add + BS_YFIXLIST + BS_YFIXTITLE,
							  bsetT[i].xstr,
							  &White, BG.srf, font);
		}
		
		if (bsetT[i].flags & BT_FLAG_END)
		{
			VID_Blit(bsetT[i].x + BS_XFIXLIST,
					 bsetT[i].y + y_add + BS_YFIXLIST,
					 6, &foreMenuList, BG.srf);

			for (j = 1; j < x_number; j++)
			{
				VID_Blit(bsetT[i].x + j * x_add + BS_XFIXLIST,
						 bsetT[i].y + y_add + BS_YFIXLIST,
						 7, &foreMenuList, BG.srf);
			}

			VID_Blit(bsetT[i].x + x_number * x_add + BS_XFIXLIST,
					 bsetT[i].y + y_add + BS_YFIXLIST,
					 8, &foreMenuList, BG.srf);
		}
	}

	/* Release font. */
	if (font)
	{
		TTF_CloseFont(font);
		font = NULL;
	}

	/* Return. */
	return (A_OK);
}

