/*
	Blokus SDL
		blchat.cpp

	This file declares methods for class TEXTBOX.

	Copyright (c) 2003-2004 Antti Partanen


	This file is part of Blokus SDL.

	Blokus SDL is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	Blokus SDL is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with Blokus SDL; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

//**********************************************************
//* INCLUDES
#include <cstring>
#include <new>

#include "blvideo.h"
#include "bltextbox.h"
#include "blexceptions.h"

//**********************************************************
//* CLASS METHODS

//**********************************************************
/*
	class TEXTBOX.
*/

//**********************************************************
/**
	Constructor.
*/
TEXTBOX::TEXTBOX(int _x, int _y, int _w, int _h,
				 int _lines, int _line_len,
				 SDL_Color *_color, TTF_Font *_font)
{
	/* Set variables. */
	x = _x;
	y = _y;
	width = _w;
	height = _h;
	lines = _lines;
	line_len = _line_len;
	font = _font;

	if (_color)
	{
		memcpy(&color, _color, sizeof(SDL_Color));
	}
	else
	{
		memset(&color, 0, sizeof(SDL_Color));
	}

	/* Allocate memory for chat screen. */
	screen = new char[line_len * lines];

	/* Check for allocation failures. */
	if (!screen)
	{
		throw OutOfMemory("Out of memory, couldn't allocate"
			              " memory for class TEXTBOX screen.\n",
						  __LINE__, __FILE__);
	}

	/* Reset allocated memory. */
	memset(screen, 0, line_len * lines);

	/* Set cmds to NULL. */
	cmds = NULL;
}

//**********************************************************
/**
	Destructor.
*/
TEXTBOX::~TEXTBOX()
{
	/* Delete screen. */
	delete[] screen;
}

//**********************************************************
/**
	Adds string to bottom of the chat screen. This function
	automatically splits up the string to fit in the chat
	screen.
*/
void TEXTBOX::add_line(char *string)
{
	int i;
	int len = strlen(string);

	if (len < 1)
	{
		return;
	}

	if (len >= line_len)
	{
		len = line_len - 1;
		string[len] = '\0';
	}

	char *str = new char[line_len];
	strcpy(str, string);

	while (len > 0)
	{
		SDL_Color color;
		SDL_Surface *temp;

		char t = str[len];
		str[len] = '\0';
		temp = VID_RenderTTF(str, &color, font);
		str[len] = t;

		if (temp->w < (width - 4))
		{
			SDL_FreeSurface(temp);
			break;
		}

		len--;
	}

	delete[] str;

	if ((string[len - 1] != ' ' && string[len - 1] != '\0') &&
		(string[len] != ' ' && string[len] != '\0'))
	{
		for (i = (len - 1); i > 0; i--)
		{
			if (string[i] == ' ')
			{
				len = i + 1;
				break;
			}
		}
	}

	for (i = 0; i < (lines - 1); i++)
	{
		memcpy(&screen[i * line_len], &screen[(i + 1) * line_len], line_len);
	}

	memcpy(&screen[(lines - 1) * line_len], string, len);
	screen[(lines - 1) * line_len + len] = '\0';

	/* Recursive call. */
	add_line(string + len);

	/* Return. */
	return;
}

//**********************************************************
/**
	Add given command.
*/
void TEXTBOX::add_command(char *command, void (*func)(int), int value)
{
	if (cmds)
	{
		cmds->add(command, func, value);
	}
	else
	{
		cmds = new COMMAND(command, func, value);
	}
}

//**********************************************************
/**
	Add given command.
*/
void TEXTBOX::add_command(char *command, void (*func)(void))
{
	if (cmds)
	{
		cmds->add(command, func);
	}
	else
	{
		cmds = new COMMAND(command, func);
	}
}

//**********************************************************
/**
	Add given command.
*/
void TEXTBOX::add_command(char *command, int *vari, int v)
{
	if (cmds)
	{
		cmds->add(command, vari, v);
	}
	else
	{
		cmds = new COMMAND(command, vari, v);
	}
}

/*	if (strcmp(cmd, "help") == 0)
	{
//		ShowHelp(screen);
	}
	else if (strcmp(cmd, "close") == 0)
	{
		NET_SetClosed(true);
	}
	else if (strcmp(cmd, "open") == 0)
	{
		NET_SetClosed(false);
	}
	else if (strncmp(cmd, "msg", 3) == 0)
	{

	}
	else if (strncmp(cmd, "whois", 5) == 0)
	{
		if (strlen(cmd) < 7)
		{
			return;
		}

		int n = 5;

		while (cmd[n] == ' ')
		{
			if (cmd[n] == '\0')
			{
				return;
			}

			n++;
		}

		NET_CONNECTION_PACKET *p = NET_GetConnectionInfo((char *)&cmd[n]);
		
		if (!p)
		{
			AddLine("Not found");
			return;
		}

//		char string[LEN_STRING];

		sprintf(string, "* Name: %s, address and port: %s:%lu,",
			p->name, p->ip_string, p->port);
		AddLine(string);
		sprintf(string, "* last ping: %lu, user identification number: %lu",
			p->ping, p->id);
		AddLine(string);

		NET_Free(p);
	}
	else if (strcmp(cmd, "host") == 0)
	{
		if (NET_Host())
		{
			AddLine("*You are the host");
		}
		else
		{
			char name[32];
			char string[LEN_STRING];
			NET_GetHostID(name);
			sprintf(string, "Host is %s", name);
			AddLine(string);
		}
	}
	else if (strcmp(cmd, "state") == 0)
	{
		if (NET_GetState(NET_STATEOF_OPEN))
		{
			AddLine("game is closed");
		}
		else
		{
			AddLine("game is open");
		}
	}*/

//**********************************************************
/**
	Handles given command.
*/
void TEXTBOX::handle_command(char *command)
{
	/* Variables. */
	COMMAND *c;
	int i;

	/* Find first space or end of line. */
	for (i = 0; command[i] != '\0'; i++)
	{
		if (command[i] == ' ')
		{
			/* Break out. */
			break;
		}
	}

	/* Command with options. */
	if (command[i] == ' ' &&
		strlen(&command[i]) > 1)
	{
		/* Cut the command here. */
		command[i] = '\0';

		/* Try to exec. */
		c = cmds->find_and_exec(command, (int)&command[i + 1]);
	}
	/* Only command, no options. */
	else
	{
		/* Cut the command here. */
		command[i] = '\0';

		/* Try to exec. */
		c = cmds->find_and_exec(command);
	}

	/* If command not found. */	
	if (!c)
	{
		add_line("* Unknown command");
	}

	return;
}

//**********************************************************
/**
	Draw text to chat screen.
*/
void TEXTBOX::draw(SDL_Surface *srf)
{
	/* Variables. */
	int yz;

	/* Draw to surface. */
	for (int i = 0; i < lines; i++)
	{
		/* Calculate height. */
		yz = y + i * (height / lines);

		/* Blit. */
		VID_RenderTTF2Srf(x, yz, &screen[i * line_len],
		                  &color, srf, font);
	}

	/* Return. */
	return;
}

//**********************************************************
/**
	Set font.
*/
void TEXTBOX::set_font(TTF_Font *_font)
{
	font = _font;
}

//**********************************************************
/**
	Set color.
*/
void TEXTBOX::set_color(SDL_Color *_color)
{
	memcpy(&color, _color, sizeof(SDL_Color));
}

