/*
	Blokus SDL
		bltextbox.h

	This file defines class TEXTBOX.

	Copyright (C) 2003-2004 Antti Partanen


	This file is part of Blokus SDL.

	Blokus SDL is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	Blokus SDL is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with Blokus SDL; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef BLTEXTBOX_H
#define BLTEXTBOX_H

//**********************************************************
//* INCLUDES
#include "SDL/SDL.h"
#include "SDL/SDL_ttf.h"

#include "blcommand.h"

//**********************************************************
//* CLASS DEFINITIONS

//**********************************************************
/**
	Class TEXTBOX.
*/
class TEXTBOX
{
public:
	/* Public methods. */
	/* Contructor and destructor. */
	TEXTBOX(int x, int y, int w, int h,
		    int lines, int line_len,
		    SDL_Color *color, TTF_Font *font);
	~TEXTBOX();

	/* Add line to box. */
	void add_line(char *string);

	/* Handle given command. */
	void handle_command(char *command);

	/* Add given command. */
	void add_command(char *command, void (*func)(int), int value = 0);

	/* Add given command. */
	void add_command(char *command, void (*func)(void));

	/* Add given command. */
	void add_command(char *command, int *var, int value = 0);

	/* Draw box to given surface. */
	void draw(SDL_Surface *srf);

	/* Set font. */
	void set_font(TTF_Font *font);

	/* Set color. */
	void set_color(SDL_Color *color);

private:
	/* Private variables. */
	/** x-coordinate for drawing the box. */
	int x;

	/** y-coordinate for drawing the box. */
	int y;
	
	/** Width of box. */
	int width;

	/** Height of box. */
	int height;

	/** Maximum number of lines. */
	int lines;

	/** Maximum length of line. */
	int line_len;

	/** Screen, where to put text. */
	char *screen;

	/** Font, which to use for drawing the text to surface. */
	TTF_Font *font;

	/** Color which use for text. */
	SDL_Color color;

	/** Commands. */
	COMMAND *cmds;
};

#endif


