/*
	Blokus SDL
		blvideo.cpp

	This file includes some video functions.

	Copyright (c) 2003-2004 Antti Partanen


	This file is part of Blokus SDL.

	Blokus SDL is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	Blokus SDL is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with Blokus SDL; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

//**********************************************************
//* INCLUDES
#include <stdio.h>
#include <string.h>
#include <stdarg.h>

#include "blvideo.h"

//**********************************************************
//* GLOBAL VARIABLES
SDL_Surface *VID_screen;
unsigned short VID_width;
unsigned short VID_height;
unsigned char VID_bpp;
unsigned short VID_pitch;

#define screen VID_screen
#define width VID_width
#define height VID_height

unsigned int VID_Rmask;
unsigned int VID_Gmask;
unsigned int VID_Bmask;
unsigned int VID_Amask;

unsigned char VID_Rshift;
unsigned char VID_Gshift;
unsigned char VID_Bshift;
unsigned char VID_Ashift;

unsigned char VID_Rloss;
unsigned char VID_Gloss;
unsigned char VID_Bloss;
unsigned char VID_Aloss;

//**********************************************************
//* FUNCTIONS

//**********************************************************
/**
*/
EVALUE VID_Init(void)
{
	screen = NULL;

	return (A_OK);
}

//**********************************************************
/**
*/
void VID_Quit(void)
{
	if (screen)
	{
		screen = NULL;
	}
}

//**********************************************************
/**
*/
SDL_Surface *VID_GetVideoSurface(void)
{
	return (screen);
}

//**********************************************************
/**
*/
EVALUE VID_Init(int x, int y, int _bpp, bool double_buffer, bool fullscreen)
{
	unsigned int flags = 0;

	if (double_buffer) flags |= SDL_DOUBLEBUF;
	if (fullscreen) flags |= (SDL_FULLSCREEN | SDL_NOFRAME);
	if (_bpp == 8) flags |= SDL_HWPALETTE;

	screen = SDL_SetVideoMode(x, y, _bpp, SDL_HWSURFACE | flags);

	if (screen == NULL)
	{
		if (double_buffer) flags &= !SDL_DOUBLEBUF;
		screen = SDL_SetVideoMode(width, height, _bpp, SDL_SWSURFACE | flags);
	}

	if (screen == NULL)
	{
        return (E_FAILED);
    }

	VID_bpp = screen->format->BitsPerPixel;
	width = screen->w;
	height = screen->h;
	VID_pitch = screen->pitch;

	VID_Rmask = screen->format->Rmask;
	VID_Gmask = screen->format->Gmask;
	VID_Bmask = screen->format->Bmask;

	VID_Rshift = screen->format->Rshift;
	VID_Gshift = screen->format->Gshift;
	VID_Bshift = screen->format->Bshift;

	VID_Rloss = screen->format->Rloss;
	VID_Gloss = screen->format->Gloss;
	VID_Bloss = screen->format->Bloss;

	if (VID_bpp == 32)
	{
		VID_Amask = 0xffffffff ^ (VID_Rmask | VID_Gmask | VID_Bmask);
	}
	else if (VID_bpp == 16)
	{
		VID_Amask = 0xffff ^ (VID_Rmask | VID_Gmask | VID_Bmask);
	}
	else
	{
		VID_Amask = 0;
	}

	if (VID_bpp == 8)
	{
		SDL_Color colors[256];
		int t;
		for(t = 0; t < 256; t++)
		{
			colors[t].r = t;
			colors[t].g = t;
			colors[t].b = t;
		}
		SDL_SetPalette(screen, SDL_LOGPAL | SDL_PHYSPAL, colors, 0, 256);
	}

	VID_Ashift = VID_Amask;
	if (VID_Ashift == 0) return (A_OK);
	int i = 0;
	while (!(VID_Ashift & 1))
	{
		VID_Ashift = (VID_Ashift >> 1);
		i++;
		if (i > 31)
		{
			i = 0;
			break;
		}
	}
	VID_Ashift = i;

	return (A_OK);
}

//**********************************************************
/**
*/
EVALUE VID_Init(int x, int y, int _bpp, unsigned int flags)
{
	if (_bpp == 8) flags |= SDL_HWPALETTE;

	VID_screen = SDL_SetVideoMode(x, y, _bpp, flags);

	if (screen == NULL)
	{
        return (E_FAILED);
    }
	
	VID_bpp = screen->format->BitsPerPixel;
	VID_pitch = screen->pitch;
	
	width = screen->w;
	height = screen->h;

	VID_Rmask = screen->format->Rmask;
	VID_Gmask = screen->format->Gmask;
	VID_Bmask = screen->format->Bmask;

	VID_Rshift = screen->format->Rshift;
	VID_Gshift = screen->format->Gshift;
	VID_Bshift = screen->format->Bshift;

	VID_Rloss = screen->format->Rloss;
	VID_Gloss = screen->format->Gloss;
	VID_Bloss = screen->format->Bloss;

	if (VID_bpp == 32)
	{
		VID_Amask = 0xffffffff ^ (VID_Rmask | VID_Gmask | VID_Bmask);
	}
	else if (VID_bpp == 16)
	{
		VID_Amask = 0xffff ^ (VID_Rmask | VID_Gmask | VID_Bmask);
	}
	else VID_Amask = 0;

	if (VID_bpp == 8)
	{
		SDL_Color colors[256];
		int t;
		for(t = 0; t < 256; t++)
		{
			colors[t].r = t;
			colors[t].g = t;
			colors[t].b = t;
		}
		SDL_SetPalette(screen, SDL_LOGPAL | SDL_PHYSPAL, colors, 0, 256);
	}

	VID_Ashift = VID_Amask;
	if (VID_Ashift == 0) return (A_OK);
	int i = 0;
	while (!(VID_Ashift & 1))
	{
		VID_Ashift = (VID_Ashift >> 1);
		i++;
		if (i > 31)
		{
			i = 0;
			break;
		}
	}
	VID_Ashift = i;

	return (A_OK);
}

//**********************************************************
/**
*/
EVALUE VID_FillRect(SDL_Rect *rect, unsigned int color)
{
	static EVALUE err;

#ifdef MULTI_THREADED_VIDEO
	static int res;

	res = SDL_SemTryWait(LockScr);

	if (res == SDL_MUTEX_TIMEDOUT)
	{
        return (E_INUSE);
	}
	if (res == -1)
	{
        return (E_LOCKFAILED);
	}
#endif
	
	err = SDL_FillRect(screen, rect, color) ? E_FAILED : A_OK;

#ifdef MULTI_THREADED_VIDEO
	res = SDL_SemPost(LockScr);

	if (res < 0)
	{
        return (E_RELEASE);
	}
#endif

	return (err);
}

//**********************************************************
/**
*/
EVALUE VID_FillRect(SDL_Rect *rect, unsigned int color, SDL_Surface *srf)
{
	static EVALUE err;

#ifdef MULTI_THREADED_VIDEO
	static int res;

	res = SDL_SemTryWait(LockScr);

	if (res == SDL_MUTEX_TIMEDOUT)
	{
        return (E_INUSE);
	}
	if (res == -1)
	{
        return (E_LOCKFAILED);
	}
#endif
	
	err = SDL_FillRect(srf, rect, color) ? E_FAILED : A_OK;

#ifdef MULTI_THREADED_VIDEO
	res = SDL_SemPost(LockScr);

	if (res < 0)
	{
        return (E_RELEASE);
	}
#endif

	return (err);
}

//**********************************************************
/**
*/
EVALUE VID_Blit(int x, int y, unsigned short number, DPIC *srf)
{
	static int xx1, yy1;
	static SDL_Rect rect1, rect2;

	yy1 = int(number / srf->xn);
	xx1 = (number - (yy1 * srf->xn)) * srf->x;
	yy1 *= srf->y;
	
	rect1.x = xx1;
	rect1.y = yy1;
	rect1.w = srf->x;
	rect1.h = srf->y;

	rect2.x = x;
	rect2.y = y;

	// rect2's w and h are not used in BlitSufrace-function but they are set just for sure
	rect2.w = srf->x;
	rect2.h = srf->y;

	if (SDL_BlitSurface(srf->srf, &rect1, screen, &rect2)) return (E_FAILED);

	return (A_OK);
}

//**********************************************************
/**
*/
EVALUE VID_Blit(int x, int y, unsigned short nx, unsigned short ny, DPIC *srf)
{
	static SDL_Rect rect1, rect2;

	rect1.x = nx * srf->x;
	rect1.y = ny * srf->y;
	rect1.w = srf->x;
	rect1.h = srf->y;

	rect2.x = x;
	rect2.y = y;

	//* rect2's w and h are not used in BlitSufrace-function
	//* but they are set just for sure
	rect2.w = srf->x;
	rect2.h = srf->y;

	if (SDL_BlitSurface(srf->srf, &rect1, screen, &rect2)) return (E_FAILED);

	return (A_OK);
}

//**********************************************************
/**
*/
EVALUE VID_Blit(int x, int y, unsigned short nx, unsigned short ny, DPIC *srf, SDL_Surface *dest)
{
	static SDL_Rect rect1, rect2;

	rect1.x = nx * srf->x;
	rect1.y = ny * srf->y;
	rect1.w = srf->x;
	rect1.h = srf->y;

	rect2.x = x;
	rect2.y = y;

	//* rect2's w and h are not used in BlitSufrace-function
	//* but they are set just for sure
	rect2.w = srf->x;
	rect2.h = srf->y;

	if (SDL_BlitSurface(srf->srf, &rect1, dest, &rect2)) return (E_FAILED);

	return (A_OK);
}

//**********************************************************
/**
*/
EVALUE VID_Blit(int x, int y, SDL_Surface *srf, SDL_Rect *rect1)
{
	static int xx1, yy1;
	static SDL_Rect rect2;

	if (!srf)
	{
		return (A_OK);
	}

	rect2.w = srf->w;
	rect2.h = srf->h;
	rect2.x = x;
	rect2.y = y;

	if (SDL_BlitSurface(srf, rect1, screen, &rect2)) return (E_FAILED);

	return (A_OK);
}

//**********************************************************
/**
*/
EVALUE VID_Blit(int x, int y, unsigned short number, DPIC *srf, SDL_Rect *rect1)
{
	static int xx1, yy1;
	static SDL_Rect rect2;

	yy1 = int(number / srf->xn);
	xx1 = (number - (yy1 * srf->xn)) * srf->x;
	yy1 *= srf->y;
	
	rect2.x = x;
	rect2.y = y;

	// rect2's w and h are not used in BlitSufrace-function but they are set just for sure
	rect2.w = srf->x;
	rect2.h = srf->y;

	if (SDL_BlitSurface(srf->srf, rect1, screen, &rect2)) return (E_FAILED);

	return (A_OK);
}

//**********************************************************
/**
*/
EVALUE VID_Blit(int x, int y, unsigned short number, DPIC *srf, SDL_Surface *dest)
{
	static int xx1, yy1;
	static SDL_Rect rect1, rect2;

	yy1 = int(number / srf->xn);
	xx1 = (number - (yy1 * srf->xn)) * srf->x;
	yy1 *= srf->y;
	
	rect1.x = xx1;
	rect1.y = yy1;
	rect1.w = srf->x;
	rect1.h = srf->y;

	rect2.x = x;
	rect2.y = y;

	// rect2's w and h are not used in BlitSufrace-function but they are set just for sure
	rect2.w = srf->x;
	rect2.h = srf->y;

	if (SDL_BlitSurface(srf->srf, &rect1, dest, &rect2)) return (E_FAILED);
	
	return (A_OK);
}

//**********************************************************
/**
*/
EVALUE VID_Blit(int x, int y, SDL_Surface *srf, SDL_Surface *dest)
{
	static SDL_Rect rect1;

	rect1.x = x;
	rect1.y = y;

	if (SDL_BlitSurface(srf, NULL, dest, &rect1)) return (E_FAILED);
	
	return (A_OK);
}

//**********************************************************
/**
*/
EVALUE VID_Flip(void)
{
	static EVALUE err;

	err = SDL_Flip(screen) ? E_FAILED : A_OK;

	return (err);
}

//**********************************************************
/**
*/
void VID_OverFade(SDL_Surface *img, unsigned int ms_sec)
{
	SDL_SetAlpha(img, SDL_SRCALPHA, 128);

	unsigned int ticks = SDL_GetTicks();
	unsigned int sticks;

	for (int i = 0; i < 6; i++)
	{
		SDL_BlitSurface(img, NULL, screen, NULL);
		SDL_Flip(screen);
		
		ticks += (ms_sec / 6);
		sticks = SDL_GetTicks();

		if (ticks > (sticks + 2))
		{
			SDL_Delay(ticks - sticks - 1);
		}
	}

	SDL_SetAlpha(img, 0, SDL_ALPHA_OPAQUE);
	SDL_BlitSurface(img, NULL, screen, NULL);
	SDL_Flip(screen);
	SDL_BlitSurface(img, NULL, screen, NULL);
}

//**********************************************************
/**
*/
void VID_CrossFade(SDL_Surface *img1, SDL_Surface *img2, unsigned int seconds)
{
	/* Used to calculate the steps to make a fade in the given time: */
	unsigned int old_time, curr_time, time_ms;
	float alpha ;

	old_time = SDL_GetTicks();
	curr_time = old_time;

	time_ms = seconds * 1000;
	alpha = 0.0 ;

	while (alpha < 255.0)
	{
		SDL_BlitSurface(img1, NULL, screen, NULL);
		SDL_SetAlpha(img2, SDL_SRCALPHA, (Uint8)alpha);
		SDL_BlitSurface(img2, NULL, screen, NULL);
		old_time = curr_time ;
		curr_time = SDL_GetTicks();
		SDL_Flip(screen);

		alpha += 255 * ((float) (curr_time - old_time) / time_ms) ;
	}

	SDL_SetAlpha(img2, 0, (Uint8)alpha);
	SDL_BlitSurface(img2, NULL, screen, NULL);
	SDL_Flip(screen);
	SDL_BlitSurface(img2, NULL, screen, NULL);
}

//**********************************************************
/**
*/
void VID_Slide(SDL_Surface *img1, SDL_Surface *img2, unsigned int dir, unsigned int time)
{
	unsigned int start_time, curr_time;
	float temp_time, time_ms;

	SDL_Rect rect1;
	SDL_Rect rect2;

	time_ms = float(time);

	rect1.x = 0;
	rect1.y = 0;
	rect1.w = img1->w;
	rect1.h = img1->h;

	rect2.x = 0;
	rect2.y = 0;
	rect2.w = img2->w;
	rect2.h = img2->h;

	start_time = SDL_GetTicks();
	curr_time = SDL_GetTicks();
	temp_time = 0;

	while (temp_time < 1)
	{
		//* Set fps to ~50 at maximum
		while ((SDL_GetTicks() - curr_time) < 20);

		curr_time = SDL_GetTicks();
		temp_time = float(curr_time - start_time) / time_ms;

		rect1.x = int(float(img1->w) * (temp_time - 1.0f));
		rect2.x = int(float(img2->w) * temp_time);

		SDL_BlitSurface(img1, &rect1, screen, NULL);
		SDL_BlitSurface(img2, &rect2, screen, NULL);
		SDL_Flip(screen);
	}

	SDL_BlitSurface(img1, NULL, screen, NULL);
	SDL_Flip(screen);
	SDL_BlitSurface(img1, NULL, screen, NULL);

	return;
}

//**********************************************************
/**
*/
SDL_Surface *VID_RenderTTF(char *string, SDL_Color *forec,
						   TTF_Font *font, int rsolid,
						   SDL_Color *backc)
{
	SDL_Surface *text;

	if (rsolid)
	{
		text = TTF_RenderText_Solid(font, string, *forec);
	}
	else
	{
		text = TTF_RenderText_Shaded(font, string, *forec, *backc);
	}
	
	if (text == NULL) return (NULL);

	return (text);
}

//**********************************************************
/**
*/
SDL_Surface *VID_RenderTTFc(char *string, SDL_Color *forec,
							TTF_Font *font, int rsolid,
							SDL_Color *backc)
{
	SDL_Surface *text;
	SDL_Surface *temp;

	if (rsolid)
	{
		temp = TTF_RenderText_Solid(font, string, *forec);
	}
	else
	{
		temp = TTF_RenderText_Shaded(font, string, *forec, *backc);
	}

	if (!temp) return (NULL);

	text = SDL_ConvertSurface(temp, screen->format, SDL_HWSURFACE);
	
	if (!text)
	{
		text = SDL_ConvertSurface(temp, screen->format, SDL_SWSURFACE);
		SDL_FreeSurface(temp);

		return (text);
	}

	SDL_FreeSurface(temp);
	
	return (text);
}

//**********************************************************
/**
*/
EVALUE VID_RenderTTF2Srf(int x, int y,
						 char *string, SDL_Color *forec,
						 SDL_Surface *srf, TTF_Font *font,
						 int rsolid, SDL_Color *backc)
{
	SDL_Surface *text = NULL;

	char *cx = string;

	if (rsolid)
	{
		text = TTF_RenderText_Solid(font, string, *forec);
	}
	else
	{
		text = TTF_RenderText_Shaded(font, string, *forec, *backc);
	}
	
	if (!text)
	{
		return (E_NULL);
	}

	static SDL_Rect dstrect;
	dstrect.x = x;
	dstrect.y = y;
	dstrect.w = text->w;
	dstrect.h = text->h;

	//* Blit the text to surface
	if (SDL_BlitSurface(text, NULL, srf, &dstrect) < 0)
	{
		SDL_FreeSurface(text);
		text = NULL;
		return (E_FAILED);
	}

	SDL_FreeSurface(text);
	text = NULL;

	return (A_OK);
}

//**********************************************************
/**
*/
EVALUE VID_RenderTTF2SrfSolid(int x, int y, SDL_Color *forec,
						  SDL_Surface *srf, TTF_Font *font,
						  char *string, ...)
{
	static EVALUE err;
	va_list args;

	va_start(args, string);
	char *buffer = new char[1024];
	
	vsprintf(buffer, string, args);
	va_end(args);

	err = VID_RenderTTF2Srf(x, y, buffer, forec, srf, font, true, NULL);

	delete[] buffer;
	
	return (err);
}

//**********************************************************
/**
*/
EVALUE VID_RenderTTF2SrfSolid(int x, int y, SDL_Surface *bg, 
								 SDL_Color *forec, SDL_Surface *srf,
								 TTF_Font *font, char *string, ...)
{
	va_list args;

	va_start(args, string);
	char *buffer = new char[1024];
	
	vsprintf(buffer, string, args);
	va_end(args);

	SDL_Surface *text = NULL;

	text = TTF_RenderText_Solid(font, buffer, *forec);
	delete[] buffer;
	
	if (!text)
	{
		return (E_NULL);
	}

	static SDL_Rect dstrect;
	dstrect.x = x;
	dstrect.y = y;
	dstrect.w = text->w;
	dstrect.h = text->h;

	//* Blit background first
	if (bg)
	{
		SDL_BlitSurface(bg, &dstrect, srf, &dstrect);
	}

	//* Blit the text to surface
	if (SDL_BlitSurface(text, NULL, srf, &dstrect) < 0)
	{
		SDL_FreeSurface(text);
		text = NULL;
		return (E_FAILED);
	}

	SDL_FreeSurface(text);
	text = NULL;
	
	return (A_OK);
}

//**********************************************************
/**
*/
TTF_Font *VID_OpenFont(char *name, int fsize, int rstyle)
{
	TTF_Font *font = TTF_OpenFont(name, fsize);

	if (!font)
	{
		return (NULL);
	}

	TTF_SetFontStyle(font, rstyle);

	return (font);
}

//**********************************************************
/**
*/
EVALUE VID_CloseFont(TTF_Font *font)
{
	if (font)
	{
		TTF_CloseFont(font);
		font = NULL;
	}

	return (A_OK);
}

