/*
	Blokus SDL
		blvideo.cpp

	Include-file for video functions.

	Copyright (c) 2003-2004 Antti Partanen


	This file is part of Blokus SDL.

	Blokus SDL is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	Blokus SDL is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with Blokus SDL; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef BLVIDEO_H
#define BLVIDEO_H


#include "SDL/SDL.h"
#include "SDL/SDL_ttf.h"

#include "derrors.h"
#include "dpic.h"

enum
{
	RENDER_LATIN1,
	RENDER_UTF8,
	RENDER_UNICODE
};

#if defined(WIN32) || defined(_WIN32)
#define DLLEXP __declspec(dllexport)
#else
#define DLLEXP
#endif

EVALUE DLLEXP VID_Init(void);
void DLLEXP VID_Quit(void);
	
EVALUE DLLEXP VID_Init(int x, int y,
	int bpp, bool double_buffer, bool fullscreen);
EVALUE DLLEXP VID_Init(int x, int y,
	int bpp, unsigned int flags);
EVALUE DLLEXP VID_FillRect(SDL_Rect *, unsigned int);
EVALUE DLLEXP VID_FillRect(SDL_Rect *, unsigned int, SDL_Surface *srf);
EVALUE DLLEXP VID_Blit(int x, int y, unsigned short number, DPIC *srf);
EVALUE DLLEXP VID_Blit(int x, int y, unsigned short nx, unsigned short ny, DPIC *srf);
EVALUE DLLEXP VID_Blit(int x, int y,
	                   SDL_Surface *srf,
					   SDL_Rect *rect = NULL);
EVALUE DLLEXP VID_Blit(int x, int y, unsigned short number,
	                   DPIC *srf, SDL_Rect *rect);
EVALUE DLLEXP VID_Blit(int x, int y, unsigned short number,
	                   DPIC *srf, SDL_Surface *dest);
EVALUE DLLEXP VID_Blit(int x, int y,
	                   SDL_Surface *srf,
					   SDL_Surface *dest);
EVALUE VID_Blit(int x, int y, unsigned short nx, unsigned short ny,
				DPIC *srf, SDL_Surface *dest);

EVALUE DLLEXP VID_Flip(void);

SDL_Surface DLLEXP *VID_GetVideoSurface(void);

// Font
SDL_Surface DLLEXP *VID_RenderTTF(
	char *string, SDL_Color *forec,
	TTF_Font *font, int rsolid = 1,
	SDL_Color *backc = NULL);

SDL_Surface DLLEXP *VID_RenderTTFc(
	char *string, SDL_Color *forec,
	TTF_Font *font, int rsolid = 1,
	SDL_Color *backc = NULL);

EVALUE DLLEXP VID_RenderTTF2Srf(int x, int y, char *string,
					            SDL_Color *forec, SDL_Surface *srf,
					            TTF_Font *font, int rsolid = 1,
					            SDL_Color *backc = NULL);

EVALUE DLLEXP VID_RenderTTF2SrfSolid(int x, int y, SDL_Color *forec,
	SDL_Surface *srf, TTF_Font *font, char *string, ...);

EVALUE DLLEXP VID_RenderTTF2SrfSolid(int x, int y,
	                                 SDL_Surface *bg, SDL_Color *forec,
	                                 SDL_Surface *srf, TTF_Font *font,
									 char *string, ...);

TTF_Font DLLEXP *VID_OpenFont(char *name, int fsize,
	                          int rstyle = TTF_STYLE_NORMAL);

EVALUE DLLEXP VID_CloseFont(TTF_Font *font);
// End of Font

#endif
